unit MyAboutEntries;

interface

	uses
		MyAssertions;
		
	const
		about_id = 928;

	type
		AboutDataEntry = record
			display: Str255;
			group: integer;
			url: Str255;
		end;

	const
		kNoGroup = 0;
		
	type
		DrawDisplayItemProc = procedure (window: WindowPtr; item: integer; selected: Boolean);

	procedure StartupAboutEntries;
	procedure ConfigureAboutEntries( draw_display_item: DrawDisplayItemProc );

	procedure GetAboutDataEntry( index: integer; var entry: AboutDataEntry );
	procedure GetFirstGroupEntry( group: integer; var entry: AboutDataEntry );
	procedure GetGroupEntry( item: integer; var entry: AboutDataEntry );
	function FindAboutEntriesURL( const url: string ): integer;

	function TrackAboutEntriesGroup( window: WindowPtr; group:integer ):Boolean;

{$ifc do_debug}
	procedure ValidateAboutEntryData( window: WindowPtr );
{$endc}

implementation

	uses
		MyDialogs, MyStartup, MyLowLevel, MyResources, 
		MyRegions, MySystemGlobals, MyMemory;
		
{$ifc do_debug}
	var
		startup_check: integer;
{$endc}

	var
		about_data: Handle;
		about_data_count: longint;
		gDrawDisplayItem: DrawDisplayItemProc;

	procedure NextAboutDataEntry( var p: longint );
	begin
		Assert( about_data <> nil );
		Assert( (0 <= p) & (p < MGetHandleSize( about_data )) );
		p := p + GetUnsignedByte( about_data^, p ) + 1; { display }
		p := p + 1; { group }
		p := p + GetUnsignedByte( about_data^, p ) + 1; { url }
		Assert( p <= MGetHandleSize( about_data ) );
	end;

	procedure GetAboutDataEntry( index: integer; var entry: AboutDataEntry );
		var
			p: longint;
			i: integer;
	begin
		Assert( about_data <> nil );
		Assert( (1 <= index) & (index <= about_data_count) );
		p := 0;
		for i := 1 to index-1 do begin
			NextAboutDataEntry( p );
		end;
		Assert( p < MGetHandleSize( about_data ) );
		BlockMoveData( AddPtrLong( about_data^, p ), @entry.display, GetUnsignedByte( about_data^, p ) + 1 );
		p := p + GetUnsignedByte( about_data^, p ) + 1;
		entry.group := GetUnsignedByte( about_data^, p );
		p := p + 1;
		BlockMoveData( AddPtrLong( about_data^, p ), @entry.url, GetUnsignedByte( about_data^, p ) + 1 );
		p := p + GetUnsignedByte( about_data^, p ) + 1;
	end;

	function MGetAboutData: OSErr;

		function CountAboutDataEntries: integer;
			var
				size: longint;
				count: integer;
				p: longint;
		begin
			Assert( about_data <> nil );
			p := 0;
			count := 0;
			size := MGetHandleSize( about_data );
			while p < size do begin
				NextAboutDataEntry( p );
				Inc(count);
			end;
			Assert( p = size );
			CountAboutDataEntries := count;
		end;

		var
			err: OSErr;
	begin
		err := MGetResource( app_resfile, 'ABOU', about_id, about_data );
		Assert( err = noErr );
		if err = noErr then begin
			about_data_count := CountAboutDataEntries;
		end;
		MGetAboutData := err;
	end;

	procedure GetFirstGroupEntry( group: integer; var entry: AboutDataEntry );
		var
			i: integer;
	begin
		Assert( group <> 0 );
		for i := 1 to about_data_count do begin
			GetAboutDataEntry( i, entry );
			if entry.group = group then begin
				Exit( GetFirstGroupEntry );
			end;
		end;
		Assert( false );
	end;

	procedure GetGroupEntry( item: integer; var entry: AboutDataEntry );
	begin
		GetAboutDataEntry( item, entry );
		Assert( entry.group <> kNoGroup );
		GetFirstGroupEntry( entry.group, entry );
		Assert( entry.url <> '' );
	end;

{$ifc do_debug}
	procedure ValidateAboutEntryData( window: WindowPtr );
		var
			i: integer;
			kind: integer;
			entry: AboutDataEntry;
			first_of_group_entry: AboutDataEntry;
	begin
		AssertStr( about_data <> nil, 'about_data < is nil' );
		for i := 1 to about_data_count do begin
			GetDItemKind( window, i, kind );
			GetAboutDataEntry( i, entry );
			if entry.group = kNoGroup then begin
				AssertStr( kind = userItem+itemDisable, 'non-group item is not disabled' );
				AssertStr( entry.url = '', 'non-group item has a URL' );
			end else begin
				AssertStr( (kind = userItem) | (kind = kButtonDialogItem), 'group item is not a kButtonDialogItem or userItem' );
				GetFirstGroupEntry( entry.group, first_of_group_entry );
				AssertStr( first_of_group_entry.url <> '', 'first_of_group_entry does not have a URL' );
				if entry.display <> first_of_group_entry.display then begin { hack test to see if they are different entries }
					AssertStr( entry.url = '', 'not first of group has a URL' );
				end;
			end;
		end;
	end;
{$endc}

	function TrackAboutEntriesGroup( window: WindowPtr; group:integer ):Boolean;
		var
			click_rgn:RgnHandle;

		procedure DrawItems( selected: Boolean );
			var
				i: integer;
				entry: AboutDataEntry;
		begin
			for i := 1 to about_data_count do begin
				GetAboutDataEntry( i, entry );
				if entry.group = group then begin
					gDrawDisplayItem( window, i, selected );
				end;
			end;
		end;

		var
			inside,newinside:Boolean;
			mouse:Point;
			i: integer;
			entry: AboutDataEntry;
			frame: Rect;
	begin
		SetPort( window );
		click_rgn := NewRgn;
		for i := 1 to about_data_count do begin
			GetAboutDataEntry( i, entry );
			if entry.group = group then begin
				GetDItemRect( window, i, frame );
				UnionRgnRect( click_rgn, frame );
			end;
		end;
		DrawItems( true );
		inside := true;
		while StillDown do begin
			GetMouse(mouse);
			newinside := PtInRgn( mouse, click_rgn );
			if newinside <> inside then begin
				DrawItems( newinside );
				inside := newinside;
			end;
		end;
		if inside then begin
			DrawItems( false );
		end;
		TrackAboutEntriesGroup := inside;
	end;

	function FindAboutEntriesURL( const url: string ): integer;
		var
			item: integer;
			entry: AboutDataEntry;
	begin
		item := 0;
		repeat
			Inc(item);
			GetAboutDataEntry( item, entry );
		until (item >= about_data_count) | (entry.url = url);
		FindAboutEntriesURL := item;
	end;

	function InitAboutEntries(var msg: integer): OSStatus;
	begin
{$unused(msg)}
		AssertDidStartup( startup_check );
		InitAboutEntries := MGetAboutData;
	end;

	procedure ConfigureAboutEntries( draw_display_item: DrawDisplayItemProc );
	begin
		StartupAboutEntries;
		gDrawDisplayItem := draw_display_item;
		DidStartup( startup_check );
	end;

	procedure StartupAboutEntries;
	begin
		StartupDialogs;
		SetStartup( InitAboutEntries, nil, 0, nil );
	end;

end.
