unit MyAppearance;

interface
		
	uses
		MacOS;
		
	procedure SafeSetThemePen( inBrush: ThemeBrush; inDepth: SInt16; inIsColorDevice: Boolean);
	procedure SafeSetThemeBackground( inBrush: ThemeBrush; inDepth: SInt16; inIsColorDevice: Boolean);
	procedure SafeSetThemeTextColor( inColor: ThemeTextColor; inDepth: SInt16; inIsColorDevice: Boolean);
	procedure SafeSetThemeWindowBackground( window: WindowPtr; inBrush: ThemeBrush; update: Boolean);
//	procedure SafeDrawThemeWindowListViewHeader( const inRect: Rect; inState: ThemeDrawState);

	function SafeHandleControlClick( inControl: ControlHandle; local: Point; inModifiers: SInt16 ): SInt16;
	function SafeHandleControlIndicatorClick( inControl: ControlHandle; local: Point; inModifiers: SInt16; proc: ControlActionProcPtr ): SInt16;

	procedure SetTextControlContents( inControl: ControlHandle; const contents: string );
	procedure GetTextControlContents( inControl: ControlHandle; var contents: Str255 );
	procedure GetTextControlSelection( inControl: ControlHandle; var start, fin: integer );
	procedure SetTextControlSelection( inControl: ControlHandle; start, fin: integer );
	procedure SetTextControlFontSize( inControl: ControlHandle; font, size: integer );
	
	procedure SafeSetControlViewSize( inControl: ControlHandle; viewsize: longint );
	
	procedure NonAppearanceDrawListViewHeader( const inRect: Rect; pressed, active: Boolean; depth: integer; deviceFlags: integer );
	procedure NonAppearanceDrawPlacardHeader( const inRect: Rect; active: Boolean; depth: integer; deviceFlags: integer );
	function NonAppearanceDrawGreyStuff( depth: integer; inIsColorDevice: Boolean ): Boolean;

	procedure GetThemeViewFontSize( var font, size: integer );

	procedure StartupAppearance;

implementation

	uses
		MyCallProc, MyTypes, MyAssertions, MySystemGlobals, MyMathUtils, MyUtils, MyInternetConfig, MyStartup;

	var
		startup_check: integer;
		
	function NonAppearanceDrawGreyStuff( depth: integer; inIsColorDevice: Boolean ): Boolean;
	begin
		AssertDidStartup( startup_check );
		NonAppearanceDrawGreyStuff := depth >= Choose( inIsColorDevice, 8, 4 );
	end;

	procedure SafeSetThemePen( inBrush: ThemeBrush; inDepth: SInt16; inIsColorDevice: Boolean);
		var
			err: OSErr;
	begin
		Assert( inBrush in [kThemeListViewSeparatorBrush] );
		err := unimpErr;
		if has_Appearance then begin
			err := SetThemePen( inBrush, inDepth, inIsColorDevice );
		end;
		if err <> noErr then begin
			case inBrush of
				kThemeListViewSeparatorBrush: begin
					ForeColor( whiteColor );
				end;
				otherwise begin
					Assert( false );
				end;
			end;
		end;
	end;
	
	procedure SafeSetThemeBackground( inBrush: ThemeBrush; inDepth: SInt16; inIsColorDevice: Boolean);
		var
			err: OSErr;
			rgb: RGBColor;
	begin
		Assert( (inBrush in [kThemeListViewBackgroundBrush,kThemeListViewSortColumnBackgroundBrush]) );
		err := unimpErr;
		if has_Appearance then begin
			err := SetThemeBackground( inBrush, inDepth, inIsColorDevice );
		end;
		if err <> noErr then begin
			if NonAppearanceDrawGreyStuff( inDepth, inIsColorDevice ) then begin
				case inBrush of
					kThemeListViewBackgroundBrush: begin
						rgb.red := $EFEF;
						rgb.green := rgb.red;
						rgb.blue := rgb.red;
					end;
					kThemeListViewSortColumnBackgroundBrush: begin
						rgb.red := $DEDE;
						rgb.green := rgb.red;
						rgb.blue := rgb.red;
					end;
					otherwise begin
						Assert( false );
					end;
				end;
				RGBBackColor( rgb );
			end else begin
				BackColor( whiteColor );
			end;
		end;
	end;

	procedure SafeSetThemeTextColor( inColor: ThemeTextColor; inDepth: SInt16; inIsColorDevice: Boolean);
		var
			err: OSErr;
	begin
		Assert( (inColor in [kThemeActivePlacardTextColor,kThemeInactivePlacardTextColor,kThemePressedPlacardTextColor]) );
		err := unimpErr;
		if has_Appearance then begin
			err := SetThemeTextColor( inColor, inDepth, inIsColorDevice );
		end;
		if err <> noErr then begin
			case inColor of
				kThemeActivePlacardTextColor,kThemeInactivePlacardTextColor: begin
					ForeColor( blackColor );
					TextMode( srcOr );
				end;
				kThemePressedPlacardTextColor: begin
					ForeColor( blackColor );
					TextMode( srcBic );
				end;
				otherwise begin
					Assert( false );
				end;
			end;
		end;
	end;

	procedure SafeSetThemeWindowBackground( window: WindowPtr; inBrush: ThemeBrush; update: Boolean);
		var
			err: OSErr;
	begin
		err := unimpErr;
		if has_Appearance then begin
			err := SetThemeWindowBackground( window, inBrush, update );
		end;
		if err <> noErr then begin
			// Do nothing
		end;
	end;
(*	
	procedure SafeDrawThemeWindowListViewHeader( const inRect: Rect; inState: ThemeDrawState);
		var
			err: OSErr;
	begin
		err := unimpErr;
		if has_Appearance then begin
			err := DrawThemeWindowListViewHeader( inRect, inState );
		end;
		if err <> noErr then begin
			DebugStr( 'NYI;sc' );
		end;
	end;
*)	
	function SafeHandleControlClick( inControl: ControlHandle; local: Point; inModifiers: SInt16 ): SInt16;
		var
			result: SInt16;
	begin
		if has_Appearance then begin
			result := HandleControlClick( inControl, local, inModifiers, nil );
		end else begin
			result := TrackControl( inControl, local, nil);
		end;
		SafeHandleControlClick := result;
	end;
	
	var
		gHackIndicatorOurOldControlAction: UniversalProcPtr;
		gHackIndicatorOurNewControlAction: UniversalProcPtr;
		gHackIndicatorControl: ControlHandle;
		gHackIndicatorControlAction: ControlActionProcPtr;
	
	procedure HackOurOldControlAction;
	begin
		gHackIndicatorControlAction( gHackIndicatorControl, kControlIndicatorPart );
	end;
	
	procedure HackOurNewControlAction( control: ControlHandle; part: integer );
	begin
		gHackIndicatorControlAction( control, part );
	end;
	
	function SafeHandleControlIndicatorClick( inControl: ControlHandle; local: Point; inModifiers: SInt16; proc: ControlActionProcPtr ): SInt16;
		var
			result: SInt16;
			ourproc: UniversalProcPtr;
	begin
		AssertDidStartup( startup_check );
		gHackIndicatorControl := inControl;
		gHackIndicatorControlAction := proc;

			if proc = nil then begin
				ourproc := nil;
			end else begin
				ourproc := gHackIndicatorOurOldControlAction;
			end;

		if has_Appearance then begin
(*
			if proc = nil then begin
				ourproc := nil;
			end else begin
				ourproc := gHackIndicatorOurNewControlAction;
			end;
*)
			result := HandleControlClick( inControl, local, inModifiers, ourproc );
		end else begin
			result := TrackControl( inControl, local, ourproc );
		end;
		SafeHandleControlIndicatorClick := result;
	end;
	
	procedure SafeSetControlViewSize( inControl: ControlHandle; viewsize: longint );
	begin
{$ifc GENERATINGPOWERPC}
		if appearance_version >= $0110 then begin
			if longint(@SetControlViewSize) <> kUnresolvedCFragSymbolAddress then begin
				SetControlViewSize( inControl, viewsize );
			end;
		end;
{$elsec}
{$unused(inControl,viewsize)}
{$endc}
	end;

	procedure SetTextControlContents( inControl: ControlHandle; const contents: string );
		var
			err: OSErr;
	begin
		Assert( has_Appearance );
//		Assert( kControlEditTextTextTag = kControlStaticTextTextTag );
		err := SetControlData( inControl, kControlEntireControl, kControlEditTextTextTag, length(contents), @contents[1] );
		Assert( err = noErr );
	end;
	
	procedure GetTextControlContents( inControl: ControlHandle; var contents: Str255 );
		var
			err: OSErr;
			resultsiz: Size;
	begin
		Assert( has_Appearance );
//		Assert( kControlEditTextTextTag = kControlStaticTextTextTag );
		err := GetControlData( inControl, kControlEntireControl, kControlEditTextTextTag, SizeOf(contents)-1, @contents[1], resultsiz );
		Assert( err = noErr );
		resultsiz := Min(255,resultsiz);
		contents[0] := chr(resultsiz);
	end;
	
	procedure GetTextControlSelection( inControl: ControlHandle; var start, fin: integer );
		var
			err: OSErr;
			resultsiz: Size;
			result: ControlEditTextSelectionRec;
	begin
		Assert( has_Appearance );
		err := GetControlData( inControl, kControlEntireControl, kControlEditTextSelectionTag, SizeOf(result), @result, resultsiz );
		Assert( err = noErr );
		start := result.selStart;
		fin := result.selEnd;
	end;
	
	procedure SetTextControlSelection( inControl: ControlHandle; start, fin: integer );
		var
			err: OSErr;
			result: ControlEditTextSelectionRec;
	begin
		Assert( has_Appearance );
		result.selStart := start;
		result.selEnd := fin;
		err := SetControlData( inControl, kControlEntireControl, kControlEditTextSelectionTag, SizeOf(result), @result );
		Assert( err = noErr );
		if system_version < $0850 then begin
			Draw1Control( inControl );
		end;
	end;
	
	procedure SetTextControlFontSize( inControl: ControlHandle; font, size: integer );
		var
			err: OSErr;
			cfsr: ControlFontStyleRec;
	begin
		Assert( has_Appearance );
		cfsr.flags := kControlUseFontMask + kControlUseSizeMask;
		cfsr.font := font;
		cfsr.size := size;
		err := SetControlData( inControl, kControlEntireControl, kControlFontStyleTag, SizeOf(cfsr), @cfsr );
		Assert( err = noErr );
	end;
	
	procedure GetThemeViewFontSize( var font, size: integer );
		var
			err: OSStatus;
{$ifc GENERATINGPOWERPC}
			junkstyle: Style;
			f: Str255;
{$endc}
	begin
		err := unimpErr;
{$ifc GENERATINGPOWERPC}
		if appearance_version >= $0110 then begin
			err := GetThemeFont( kThemeViewsFont, smSystemScript, @f, size, junkstyle );
			if err = noErr then begin
				GetFNum( f, font );
			end;
		end;
{$endc}
		if err <> noErr then begin
			GetListFontSize( font, size );
			if (font = kFontIDGeneva) then begin
				size := Max( size, 10 );
			end;
		end;
	end;

	type
		ColourArray = array[1..10] of integer;
		ColourArrayPtr = ^ColourArray;
	
	var
		gHackColours: ColourArrayPtr;
		
	procedure GetColour( col: integer; var rgb: RGBColor );
	begin
		col := gHackColours^[col];
		rgb.red := col * 257;
		rgb.green := rgb.red;
		rgb.blue := rgb.red;
	end;
	
	procedure SetForeColour( col: integer );
		var
			rgb: RGBColor;
	begin
		GetColour( col, rgb );
		RGBForeColor( rgb );
	end;
	
	procedure SetBackColour( col: integer );
		var
			rgb: RGBColor;
	begin
		GetColour( col, rgb );
		RGBBackColor( rgb );
	end;

	procedure NonAppearanceDrawPlacardHeader( const inRect: Rect; active: Boolean; depth: integer; deviceFlags: integer );
		const
			colours: ColourArray = ($FF,$DE,$AD,$DE,   $DE,  0,0,0,0,0);
		var
			port: GrafPtr;
			saved: SavedGraphicsPortInfo;
			frame: Rect;
	begin
		gHackColours := @colours;
		
		frame := inRect;
		if NonAppearanceDrawGreyStuff( depth, IsDeviceLoopFlagsColour(deviceFlags) ) then begin
			GetPort( port );
			EnterGraphicsPort( port, saved );
			if active then begin
				InsetRect( frame, 1, 1 );
				SetBackColour( 4 );
				EraseRect( frame );
				InsetRect( frame, -1, -1 );
				Dec(frame.bottom);
				Dec(frame.right);
				SetForeColour( 1 );
				MoveTo( frame.left, frame.bottom - 1 );
				LineTo( frame.left, frame.top );
				LineTo( frame.right, frame.top );
				SetForeColour( 2 );
				MoveTo( frame.left, frame.bottom );
				Line( 0, 0 );
				SetForeColour( 3 );
				MoveTo( frame.left + 1, frame.bottom );
				LineTo( frame.right, frame.bottom );
				LineTo( frame.right, frame.top + 1 );
			end else begin
				SetBackColour( 5 );
				EraseRect( frame );
			end;
			ExitGraphicsPort( saved );
	end else begin
			EraseRect( frame );
		end;
	end;
	
	procedure NonAppearanceDrawListViewHeader( const inRect: Rect; pressed, active: Boolean; depth: integer; deviceFlags: integer );
		const
			pressed_colours: ColourArray = ($10,$52,$73,$AD,$42,$8C,    $73,$8C,$9C,$BD);
			normal_colours: ColourArray = ($63,$FF,$CE,$8C,$31,$CE,   $AD,$AD,$9C,$DE);
		var
			frame: Rect;
			port: GrafPtr;
			saved: SavedGraphicsPortInfo;
	begin
		Assert( not has_Appearance );

//bugger, four states (pressed/normal, active/inactive)

		frame := inRect;
		if NonAppearanceDrawGreyStuff( depth, IsDeviceLoopFlagsColour(deviceFlags) ) then begin
			if pressed then begin
				gHackColours := @pressed_colours;
			end else begin
				gHackColours := @normal_colours;
			end;
			
			GetPort( port );
			EnterGraphicsPort( port, saved );

			if active then begin
				SetBackColour( 6 );
				InsetRect( frame, 2, 2 );
				EraseRect( frame );
				InsetRect( frame, -2, -2 );
				Dec(frame.bottom);
				Dec(frame.right);
				SetForeColour( 1 );
				MoveTo( frame.left, frame.bottom );
				LineTo( frame.left, frame.top );
				LineTo( frame.right, frame.top );
				SetForeColour( 2 );
				MoveTo( frame.left + 1, frame.bottom - 2 );
				LineTo( frame.left + 1, frame.top + 1 );
				LineTo( frame.right - 2, frame.top + 1 );
				SetForeColour( 3 );
				MoveTo( frame.left + 1, frame.bottom - 1 );
				Line( 0, 0 );
				MoveTo( frame.right - 1, frame.top + 1 );
				Line( 0, 0 );
				SetForeColour( 4 );
				MoveTo( frame.left + 2, frame.bottom - 1 );
				LineTo( frame.right - 1, frame.bottom - 1 );
				LineTo( frame.right - 1, frame.top + 2 );
				SetForeColour( 5 );
				MoveTo( frame.left + 1, frame.bottom );
				LineTo( frame.right, frame.bottom );
				LineTo( frame.right, frame.top + 1 );
			end else begin
				SetBackColour( 10 );
				InsetRect( frame, 1, 1 );
				EraseRect( frame );
				InsetRect( frame, -1, -1 );
				Dec(frame.bottom);
				Dec(frame.right);
				SetForeColour( 7 );
				MoveTo( frame.left, frame.bottom - 1 );
				LineTo( frame.left, frame.top );
				LineTo( frame.right - 1, frame.top );
				SetForeColour( 8 );
				MoveTo( frame.left, frame.bottom );
				Line( 0, 0 );
				MoveTo( frame.right, frame.top );
				Line( 0, 0 );
				SetForeColour( 9 );
				MoveTo( frame.left + 1, frame.bottom );
				LineTo( frame.right, frame.bottom );
				LineTo( frame.right, frame.top );
			end;

			ExitGraphicsPort( saved );
		end else begin
			FrameRect( frame );
			InsetRect( frame, 1, 1 );
			if pressed then begin
				PaintRect( frame );
			end else begin
				EraseRect( frame );
			end;
		end;
	end;

	function InitAppearance( var msg: integer ): OSStatus;
	begin
{$unused(msg)}
		DidStartup( startup_check );
		gHackIndicatorOurOldControlAction := NewProc( @HackOurOldControlAction, uppPascal0ProcInfo );
		gHackIndicatorOurNewControlAction := NewControlActionProc( @HackOurNewControlAction );
		InitAppearance := noErr;
	end;
	
	procedure FinishAppearance;
	begin
		DisposeRoutineDescriptor( gHackIndicatorOurOldControlAction );
		DisposeRoutineDescriptor( gHackIndicatorOurNewControlAction );
	end;

	procedure StartupAppearance;
	begin
		SetStartup( InitAppearance, nil, 0, FinishAppearance );
	end;
	
(*	
  procedure Test;
    var
      result: Size;
      t1, t2: OSType;
      s: Str255;
  begin
    t1 := kControlEditTextTextTag;
    t2 := kControlStaticTextTextTag;
    result := ord(kControlEditTextTextTag = kControlStaticTextTextTag);
    s := t1 + t2;
  end;
*)

end.
