unit MyCachedExtensionIconMapping;

interface

	uses
		MacOS;
		
	procedure StartupCachedExtensionIconMapping;
	procedure ConfigureCachedExtensionIconMapping( iconcount: longint );
	procedure SetCachedExtensionIconMappingInfo( application_iconsuite: Handle; const default_application_kind, default_document_kind: string );
	
	procedure GetExtensionIcon( const name: string; defaulticonsuite: Handle; var iconsuite: Handle; var kind: Str63 );

implementation

	uses
		MyStartup, MyDesktopDB, MyInternetConfig, MyGrowZones, MyStrings, MyMemory, MyUtils;
		
{$ifc do_debug}
	var
		startup_check: integer;
{$endc}

	type
		ExtensionType = string[7];
	type
		MapEntry = record
			lastused: longint; // Ticks
			extension: ExtensionType;
			iconsuite: Handle;
			kind: Str63;
		end;
		MapEntryArray = array[0..0] of MapEntry;
		MapEntryPtr = ^MapEntryArray;
	
	type
		ExtensionMapEntry = record
			extension: ExtensionType;
			file_creator, file_type: OSType;
			entry_name: Str63;
		end;
		ExtensionMapEntryArray = array[0..0] of ExtensionMapEntry;
		ExtensionMapEntryPtr = ^ExtensionMapEntryArray;
		ExtensionMapEntryHandle = ^ExtensionMapEntryPtr;
		
	var
		gIconCount: longint;
		gIconMap: MapEntryPtr;
		gExtensionMap: ExtensionMapEntryHandle;
		gEntryMapICSeed: longint;
		gApplicationIconSuite: Handle;
		gApplicationKind, gDocumentKind: Str63;
	
	procedure ResetEntryMaps;
		var
			count: longint;
			size: longint;
			iccount: longint;
			icpos: longint;
			i: longint;
			map_handle: Handle;
			icentry: ICMapEntry;
			entry: ExtensionMapEntry;
			left, right: Str255;
			junk: OSErr;
	begin
		count := 0;
		if MyGetICMapHandle( map_handle ) & (ICCountMapEntries( internet_config_instance, map_handle, iccount ) = noErr) then begin
			junk := MSetHandleSize( gExtensionMap, iccount * SizeOf(ExtensionMapEntry) );
			size := MGetHandleSize( gExtensionMap );
			Assert( size mod SizeOf(ExtensionMapEntry) = 0 );
			icpos := 0;
			for i := 1 to iccount do begin
				if ICGetMapEntry( internet_config_instance, map_handle, icpos, icentry ) = noErr then begin
					if not btst(icentry.flags, ICmap_not_incoming_bit) then begin
						if not SplitAt( icentry.extension, '.', left, right ) then begin
							right := icentry.extension;
						end;
						if right <> '' then begin
							LowerCaseString( right );
							entry.extension := right;
							entry.file_creator := icentry.file_creator;
							entry.file_type := icentry.file_type;
							entry.entry_name := icentry.entry_name;
							if count >= size div SizeOf(ExtensionMapEntry) then begin
								size := size + 20 * SizeOf(ExtensionMapEntry);
								if MGrowHandleSize( gExtensionMap, size ) <> noErr then begin
									leave;
								end;
							end;
							gExtensionMap^^[count] := entry;
							inc(count);
						end;
					end;
					icpos := icpos + icentry.total_length;
				end else begin
					leave;
				end;
			end;
		end;
		MShrinkHandleSize( gExtensionMap, count * SizeOf(ExtensionMapEntry) );
	end;
	
	function FindExtensionMapEntry( const extension: string; var entry: ExtensionMapEntry): Boolean;
		var
			i: longint;
	begin
		FindExtensionMapEntry := false;
		for i := 0 to MGetHandleSize( gExtensionMap ) div SizeOf(ExtensionMapEntry) - 1 do begin
			if extension = gExtensionMap^^[i].extension then begin
				entry := gExtensionMap^^[i];
				FindExtensionMapEntry := true;
				leave;
			end;
		end;
	end;
	
	procedure GetExtensionIcon( const name: string; defaulticonsuite: Handle; var iconsuite: Handle; var kind: Str63 );
		var
			err: OSErr;
			left, extension: Str255;
			i, free, bestfree: longint;
			entry: ExtensionMapEntry;
			hadbin: Boolean;
	begin
		AssertDidStartup( startup_check );
		Assert( gIconMap <> nil );
		left := name;
		LowerCaseString( left );
		hadbin := IsExtension( left, '.bin' );
		if hadbin then begin
			DeleteExtension( left, '.bin' );
		end;
		if Contains( '.', left ) then begin
			SplitRightBy( left, '.', left, extension );
		end else begin
			extension := '';
		end;
		if hadbin & (extension = '') then begin
			extension := 'bin';
		end;
		if (extension <> '') & IsDigit( extension[1] ) & IsAllInSet( extension, ['0'..'9'] ) then begin // Convert all number extensions to .1
			extension := '1';
		end;
		iconsuite := nil;
		kind := '';
		err := afpItemNotFound;
		if (extension <> '') & (length(extension) <= 5) then begin
			extension := extension;
			free := -1;
			bestfree := -1;
			for i := 0 to gIconCount-1 do begin
				if gIconMap^[i].lastused = 0 then begin
					if free < 0 then begin
						free := i;
					end;
				end else if extension = gIconMap^[i].extension then begin
					gIconMap^[i].lastused := TickCount;
					iconsuite := gIconMap^[i].iconsuite;
					kind := gIconMap^[i].kind;
					err := noErr;
					leave;
				end else begin
					if (bestfree < 0) | (gIconMap^[i].lastused < gIconMap^[bestfree].lastused) then begin
						bestfree := i;
					end;
				end;
			end;
			if err <> noErr then begin
				if free < 0 then begin
					free := bestfree;
					MDisposeIconSuite( gIconMap^[bestfree].iconsuite );
				end;
				gIconMap^[free].extension := extension;
				gIconMap^[free].lastused := TickCount;
				if EnoughSpace( 10000, 10000 ) & FindExtensionMapEntry( extension, entry ) then begin
					err := GetDTDBIconSuiteAnyVolume( entry.file_creator, entry.file_type, iconsuite );
					gIconMap^[free].kind := entry.entry_name;
					gIconMap^[free].iconsuite := iconsuite;
					kind := gIconMap^[free].kind;
					if entry.file_type = 'APPL' then begin
						if iconsuite = nil then begin
							iconsuite := gApplicationIconSuite;
						end;
						if kind = '' then begin
							kind := gApplicationKind;
						end;
					end;
				end else begin
					gIconMap^[free].kind := '';
					gIconMap^[free].iconsuite := nil;
					kind := '';
				end;
			end;
		end;
		if iconsuite = nil then begin
			iconsuite := defaulticonsuite;
		end;
		if kind = '' then begin
			kind := gDocumentKind;
		end;
	end;

	procedure SetCachedExtensionIconMappingInfo( application_iconsuite: Handle; const default_application_kind, default_document_kind: string );
	begin
		AssertDidStartup( startup_check );
		gApplicationIconSuite := application_iconsuite;
		gApplicationKind := default_application_kind;
		gDocumentKind := default_document_kind;
	end;

	function InitCachedExtensionIconMapping( var msg: integer ): OSStatus;
		var
			err: OSErr;
	begin
{$unused(msg)}
		AssertDidStartup( startup_check );
		err := MNewHandle( gExtensionMap, 0 );
		Assert( err = noErr );
		gEntryMapICSeed := GetICSeed;
		ResetEntryMaps;
		InitCachedExtensionIconMapping := MNewPtrClear( gIconMap, gIconCount * SizeOf(MapEntry) ) ;
	end;
	
	procedure IdleCachedExtensionIconMapping;
		var
			new_seed: longint;
	begin
		new_seed := GetICSeed;
		if new_seed <> gEntryMapICSeed then begin
			gEntryMapICSeed := new_seed;
			ResetEntryMaps;
		end;
	end;
	
	procedure FinishCachedExtensionIconMapping;
		var
			i: longint;
	begin
		for i := 0 to gIconCount-1 do begin
			if gIconMap^[i].iconsuite <> nil then begin
				MDisposeIconSuite( gIconMap^[i].iconsuite );
			end;
		end;
		MDisposePtr( gIconMap );
		MDisposeHandle( gExtensionMap );
	end;

	procedure ConfigureCachedExtensionIconMapping( iconcount: longint );
	begin
		DidStartup( startup_check );
		Assert( iconcount > 0 );
		gIconCount := iconcount;
		StartupCachedExtensionIconMapping;
	end;

	procedure StartupCachedExtensionIconMapping;
	begin
		StartupInternetConfig;
		StartupGrowZones;
		SetStartup( InitCachedExtensionIconMapping, IdleCachedExtensionIconMapping, 60, FinishCachedExtensionIconMapping );
	end;
	
end.
