unit MyCollections;

interface

	uses
		MyTypes;
		
	const
		no_tag = 0;

	type
		PermuteArray = array[1..8000] of integer;
		PermuteArrayPtr = ^PermuteArray;

	type
		tagType = OSType;
		indexType = longint;
		MyCollection = object
				public
				error: OSErr;
				safeget: Boolean;
				testheap: Boolean;

				private
				data: Handle;
				size: longint;
				cnt: indexType;
				fixed, tagged: Boolean;
				lensize, tagsize: longint;
				searchindex: indexType;
				searchtag: tagType;
				cacheoffset: longint;
				cachelen: longint;
				cacheindex: indexType;

//				public
				procedure Create (siz: longint; fix, tag: Boolean);
				procedure CreateFromHandle (d: Handle);
				procedure Destroy;
				procedure SetDataHandle (d: Handle);
				function GetDataHandle: Handle;
				procedure Reset;

				function Count: indexType;

				function GetTag (index: indexType): tagType;
				function GetIndex (tag: tagType): indexType;

				procedure SetTag (index: indexType; tag: tagType);

				function Exists (index: indexType): Boolean;
				function ExistsTag (tag: {univ }tagType): Boolean;

				function Info (index: indexType; var len: longint): Boolean;
				function InfoTag (tag: {univ } tagType; var len: longint): Boolean;

				procedure Delete (index: indexType);
				procedure DeleteTag (tag: {univ } tagType);

				procedure InsertBefore (index: indexType);

				procedure Permute (map: PermuteArrayPtr);

				procedure AddBoolean (b: Boolean);
				procedure AddTagBoolean (tag: {univ } tagType; b: Boolean);
				procedure AddLong (n: univ longint);
				procedure AddTagLong (tag: {univ } tagType; n: univ longint);
				procedure AddString (const s: Str255);
				procedure AddTagString (tag: {univ } tagType; const s: Str255);
				procedure AddData (p: Ptr; len: longint);
				procedure AddTagData (tag: {univ } tagType; p: Ptr; len: longint);
				procedure AddItem (p: Ptr);
				procedure AddTagItem (tag: {univ } tagType; p: Ptr);

				procedure SetBoolean (index: indexType; b: Boolean);
				procedure SetTagBoolean (tag: {univ } tagType; b: Boolean);
				procedure SetLong (index: indexType; n: univ longint);
				procedure SetTagLong (tag: {univ } tagType; n: univ longint);
				procedure SetString (index: indexType; const s: Str255);
				procedure SetTagString (tag: {univ } tagType; const s: Str255);
				procedure SetData (index: indexType; p: Ptr; len: longint);
				procedure SetTagData (tag: {univ } tagType; p: Ptr; len: longint);
				procedure SetItem (index: indexType; p: Ptr);
				procedure SetTagItem (tag: {univ } tagType; p: Ptr);

				procedure SetDefaultBoolean (tag: {univ } tagType; b: Boolean);
				procedure SetDefaultLong (tag: {univ } tagType; n: univ longint);
				procedure SetDefaultString (tag: {univ } tagType; const s: Str255);
				procedure SetDefaultData (tag: {univ } tagType; p: Ptr; len: longint);
				procedure SetDefaultItem (tag: {univ } tagType; p: Ptr);

				function GetBoolean (index: indexType): Boolean;
				function GetTagBoolean (tag: {univ } tagType): Boolean;
				procedure GetLong (index: indexType; var l: univ longint);
				procedure GetTagLong (tag: {univ } tagType; var l: univ longint);
				function GetString (index: indexType): Str255;
				function GetTagString (tag: {univ } tagType): Str255;
				procedure GetData (index: indexType; p: Ptr; len: longint);
				procedure GetTagData (tag: {univ } tagType; p: Ptr; len: longint);
				procedure GetItem (index: indexType; p: Ptr);
				procedure GetTagItem (tag: {univ } tagType; p: Ptr);

				procedure InvalidateCache;
				
//				private
				function GetOffset (index: indexType; var offset: longint; var len: longint): Boolean;
				function GetTagOffset (tag: {univ } tagType; var offset: longint; var len: longint; var index: indexType; test: Boolean): Boolean;
				procedure AddChunk (tag: tagType; p: Ptr; len: longint);
				procedure SetChunk (offset, l: longint; tag: tagType; p: Ptr; len: longint);
				procedure SetChunkIndex (index: indexType; p: Ptr; len: longint);
				procedure SetChunkTag (tag: tagType; p: Ptr; len: longint);
				procedure SetChunkDefault (tag: tagType; p: Ptr; len: longint);
				procedure GetChunkIndex (index: indexType; len: longint; p: Ptr);
				procedure GetChunkTag (tag: tagType; len: longint; p: Ptr);
			end;

	procedure HackUpdateHandleToCollection (data: Handle);

implementation

	uses
		MyAssertions, MyUtils, MyMemory, MyLowLevel;

{ Format is saved in prefs files, so it must not change! }

	const
		lsize = 4;
		magic_version = $12345678;
		fixed_bit = 16;
		tagged_bit = 0;
		safeget_bit = 1;

{$PUSH}
{$ALIGN MAC68K}

	type
		CollectionHeader = record
				version: longint;
				size: longint;
				cnt: indexType;
				flags: longint;
				space: longint;
			end;
		CollectionHeaderPtr = ^CollectionHeader;
		CollectionHeaderHandle = ^CollectionHeaderPtr;

{$ALIGN RESET}
{$POP}

{ Data format: }
{ CollectionHeader}
{ [tag (lsize)] [length (lsize)] data }

	function LongAtPtr (p: univ LongIntPtr): longint;
{$IFC not GENERATINGPOWERPC}
	inline
		$205F, $224F, $12D8, $12D8, $12D8, $12D8;
{ move.l (sp)+,a0 move.l sp,a1, 4*move.b (a0)+,(a1)+ }
{$ELSEC}
	begin
		LongAtPtr:=p^;
	end;
{$ENDC}

	function TagAtPtr (p: univ LongIntPtr): tagType;
{$IFC not GENERATINGPOWERPC}
	inline
		$205F, $224F, $12D8, $12D8, $12D8, $12D8;
{ move.l (sp)+,a0 move.l sp,a1, 4*move.b (a0)+,(a1)+ }
{$ELSEC}
	begin
		TagAtPtr:=tagType(p^);
	end;
{$ENDC}

	function EqualTag(t1, t2: OSType): Boolean;
	begin
		EqualTag := longint(t1) = longint(t2);
	end;

	procedure HackUpdateHandleToCollection ( data: Handle );
		var
			head: CollectionHeader;
			pos: longint;
			size: longint;
			junk: OSErr;
	begin
		if (MGetHandleSize(data) < SizeOf(CollectionHeader)) | (CollectionHeaderHandle(data)^^.version <> magic_version) then begin
			head.version := magic_version;
			head.size := -1;
			head.flags := 0;
			BSET(head.flags, tagged_bit);
			BSET(head.flags, safeget_bit);
			head.space := 0;
			head.cnt := 0;
			pos := 0;
			while (pos >= 0) & (pos <= MGetHandleSize(data) - 8) do begin
				head.cnt := head.cnt + 1;
				size := LongAtPtr(Ptr(ord(data^) + lsize));
				if (size < 0) | (size > 1000) then begin
					pos := -1;
				end else begin
					pos := pos + 8 + size;
				end;
			end;
			if pos <> MGetHandleSize(data) then begin
				MShrinkHandleSize( data, 0 );
				head.cnt := 0;
			end;
			junk := MMungerInsert( data, 0, @head, SizeOf(head) );
		end;
	end;

	procedure MyCollection.Create (siz: longint; fix, tag: Boolean);
		var
			junk: OSErr;
	begin
		MLockObjHigh( self );
		junk := MNewHandle( data, SizeOf(CollectionHeader) );
		size := siz;
		fixed := fix;
		tagged := tag;
		safeget := false;
		testheap := false;
		lensize := lsize * ord(not fixed);
		tagsize := lsize * ord(tagged);
		Reset;
	end;

	procedure MyCollection.Destroy;
	begin
		MDisposeHandle( data );
		MDisposeObject( self );
	end;

	function MyCollection.GetDataHandle: Handle;
		var
			flags: longint;
	begin
		CollectionHeaderHandle(data)^^.version := magic_version;
		CollectionHeaderHandle(data)^^.size := size;
		CollectionHeaderHandle(data)^^.cnt := cnt;
		flags := 0;
		if fixed then begin
			BSET(flags, fixed_bit);
		end;
		if tagged then begin
			BSET(flags, tagged_bit);
		end;
		if safeget then begin
			BSET(flags, safeget_bit);
		end;
		CollectionHeaderHandle(data)^^.flags := flags;
		CollectionHeaderHandle(data)^^.space := 0;
		GetDataHandle := data;
	end;

	procedure MyCollection.SetDataHandle (d: Handle);
		var
			flags: longint;
	begin
		if CollectionHeaderHandle(d)^^.version = magic_version then begin
			MDisposeHandle(data);
			data := d;
			error := noErr;
			size := CollectionHeaderHandle(data)^^.size;
			cnt := CollectionHeaderHandle(data)^^.cnt;
			flags := CollectionHeaderHandle(data)^^.flags;
			fixed := BTST(flags, fixed_bit);
			tagged := BTST(flags, tagged_bit);
			safeget := BTST(flags, safeget_bit);
			testheap := false;
			lensize := lsize * ord(not fixed);
			tagsize := lsize * ord(tagged);
			InvalidateCache;
		end else begin
			MDisposeHandle( d );
			Reset;
			error := -17010;
		end;
	end;

	procedure MyCollection.CreateFromHandle (d: Handle);
		var
			junk: OSErr;
	begin
		junk := MNewHandle( data, SizeOf(CollectionHeader) );
		SetDataHandle(d);
	end;

	procedure MyCollection.Reset;
	begin
		error := noErr;
		cnt := 0;
		MShrinkHandleSize( data, SizeOf(CollectionHeader) );
		InvalidateCache;
	end;

	procedure MyCollection.InvalidateCache;
	begin
		cacheoffset := -1;
	end;

	procedure MyCollection.Permute (map: PermuteArrayPtr);
		type
			LongArray = array[1..8000] of longint;
			LongArrayPtr = ^LongArray;
		var
			i, j, k: integer;
			offset, src, len, handlesize: longint;
			dummy: Boolean;
			newdata: Handle;
			offsetptr: LongArrayPtr;
			tmpmap: PermuteArrayPtr;
			err, junk: OSErr;
	begin
		handlesize := MGetHandleSize(data);
		err := MNewTempHandle( newdata, handlesize );
		if newdata = nil then begin
			junk := MNewHandle( newdata, handlesize );
		end;
		offsetptr := nil;
		if newdata <> nil then begin
			err := MNewPtr(offsetptr, longint(cnt) * 4);
		end;
		if offsetptr <> nil then begin
			offset := SizeOf(CollectionHeader) + tagsize;
			for i := 1 to cnt do begin
				offsetptr^[i] := offset - tagsize;
				if fixed then begin
					offset := offset + size + tagsize;
				end else begin
					offset := offset + lsize + LongAtPtr(Ptr(ord(data^) + offset)) + tagsize; { Point to next length }
				end;
			end;
			offset := SizeOf(CollectionHeader);
			len := size + tagsize + lensize;
			for i := 1 to cnt do begin
				src := offsetptr^[map^[i]];
				if not fixed then begin
					len := tagsize + LongAtPtr(Ptr(ord(data^) + src + tagsize)) + lensize;
				end;
				BlockMoveData(Ptr(ord(data^) + src), Ptr(ord(newdata^) + offset), len);
				offset := offset + len;
			end;
			Assert(offset = handlesize);
			BlockMoveData(newdata^, data^, handlesize);
			MDisposePtr(offsetptr);
			MDisposeHandle(newdata);
		end else begin
			MDisposeHandle(newdata); { nil safe }
			err := MNewPtr( tmpmap, longint(cnt) * SizeOf(map^[1]) );
			if err = noErr then begin
				BlockMoveData( map, tmpmap, MGetPtrSize( tmpmap ) );
				for i := 1 to cnt do begin
					k := tmpmap^[i];
					cacheoffset := -1;
					dummy := GetOffset(k, offset, len);
					Assert(dummy);
					offset := offset - tagsize - lensize;
					len := len + tagsize + lensize;
					err := MGrowHandleSize(data, handlesize + len);
					Assert(err = noErr);
					BlockMoveData( AddPtrLong( data^, offset ), AddPtrLong( data^, handlesize ), len );
//						Ptr(ord(data^) + offset), Ptr(ord(data^) + handlesize), len);
					MMungerDelete(data, offset, len);
					cacheoffset := -1;
					for j := 1 to cnt do begin
						if tmpmap^[j] > k then begin
							tmpmap^[j] := tmpmap^[j] - 1;
						end;
					end;
				end;
				MDisposePtr( tmpmap );
			end;
		end;
		InvalidateCache;
	end;

	function MyCollection.GetOffset (index: indexType; var offset: longint; var len: longint): Boolean;
		var
			valid: Boolean;
			i: indexType;
	begin
		if testheap then begin
			DebugStr('GetOffset;hc;g');
		end;
		valid := (0 < index) & (index <= cnt);
		if valid then begin
			if fixed then begin
				len := size;
				offset := SizeOf(CollectionHeader) + (index - 1) * (size + tagsize) + tagsize;
			end else begin
				if (cacheoffset > 0) & (searchindex > 0) & (searchindex <= index) then begin
					offset := cacheoffset - lsize;
					i := searchindex;
				end else begin
					offset := SizeOf(CollectionHeader) + tagsize; { Point to first length }
					i := 1;
				end;
				while (i < index) do begin
					offset := offset + lsize + LongAtPtr(Ptr(ord(data^) + offset)) + tagsize; { Point to next length }
					i := i + 1;
				end;
				len := LongAtPtr(Ptr(ord(data^) + offset));
				offset := offset + lsize; { Point to data }
			end;
			cacheoffset := offset;
			cachelen := len;
			searchindex := index;
		end else begin
			Assert(false);
			InvalidateCache;
		end;
		GetOffset := valid;
	end;

	function MyCollection.GetTagOffset (tag: {univ } tagType; var offset: longint; var len: longint; var index: indexType; test: Boolean): Boolean;
		var
			valid: Boolean;
			t: tagType;
			handlesize: longint;
	begin
		if testheap then begin
			DebugStr('GetTagOffset;hc;g');
		end;
		valid := false;
		if tagged then begin
			if (cacheoffset > 0) & (searchindex < 0) & EqualTag(searchtag, tag) then begin
				offset := cacheoffset;
				len := cachelen;
				index := cacheindex;
				valid := true;
			end else begin
				len := size;
				index := 0;
				offset := SizeOf(CollectionHeader); { Point to first tag }
				handlesize := MGetHandleSize(data);
				while (not valid) & (index < cnt) do begin
					Assert((0 < offset) & (offset < handlesize));
					t := TagAtPtr(Ptr(ord(data^) + offset));
					if not fixed then begin
						len := LongAtPtr(Ptr(ord(data^) + offset + tagsize));
					end;
					offset := offset + tagsize + lensize + len; { Point to next tag }
					index := index + 1;
					valid := EqualTag(t, tag);
				end;
				offset := offset - len; { Point to data }
			end;
		end;
		if not test then begin
			Assert(valid);
		end;
		if valid then begin
			cacheoffset := offset;
			cachelen := len;
			cacheindex := index;
			searchindex := -1;
			searchtag := tag;
		end else begin
			InvalidateCache;
		end;
		GetTagOffset := valid;
	end;

	function MyCollection.Count: indexType;
	begin
		Count := cnt;
	end;

	function MyCollection.GetTag (index: indexType): tagType;
		var
			offset, len: longint;
	begin
		GetTag := tagType(no_tag);
		Assert(tagged);
		if GetOffset(index, offset, len) then begin
			GetTag := TagAtPtr(Ptr(ord(data^) + offset - lensize - tagsize));
		end;
	end;

	procedure MyCollection.SetTag (index: indexType; tag: tagType);
		var
			offset, len: longint;
	begin
		Assert(tagged);
		if GetOffset(index, offset, len) then begin
			BlockMoveData(@tag, Ptr(ord(data^) + offset - lensize - tagsize), tagsize);
		end;
	end;

	function MyCollection.GetIndex (tag: tagType): indexType;
		var
			offset, len: longint;
			index: indexType;
	begin
		GetIndex := 0;
		if GetTagOffset(tag, offset, len, index, true) then begin
			GetIndex := index;
		end;
	end;

	function MyCollection.Info (index: indexType; var len: longint): Boolean;
		var
			offset: longint;
	begin
		Info := (1 <= index) & (index <= cnt) & GetOffset(index, offset, len);
	end;

	function MyCollection.InfoTag (tag: {univ } tagType; var len: longint): Boolean;
		var
			offset: longint;
			index: indexType;
	begin
		InfoTag := GetTagOffset(tag, offset, len, index, true);
	end;

	function MyCollection.Exists (index: indexType): Boolean;
		var
			len: longint;
	begin
		Exists := Info(index, len);
	end;

	function MyCollection.ExistsTag (tag: {univ } tagType): Boolean;
		var
			len: longint;
	begin
		ExistsTag := InfoTag(tag, len);
	end;

	procedure MyCollection.Delete (index: indexType);
		var
			offset, len: longint;
	begin
		if GetOffset(index, offset, len) then begin
			MMungerDelete(data, offset - tagsize - lensize, tagsize + lensize + len);
			cnt := cnt - 1;
			InvalidateCache;
		end;
	end;

	procedure MyCollection.DeleteTag (tag: {univ } tagType);
		var
			offset, len: longint;
			index: indexType;
	begin
		if GetTagOffset(tag, offset, len, index, true) then begin
			MMungerDelete(data, offset - tagsize - lensize, tagsize + lensize + len);
			cnt := cnt - 1;
			InvalidateCache;
		end;
	end;

	procedure MyCollection.AddChunk (tag: tagType; p: Ptr; len: longint);
		var
			err: OSErr;
			orgsize: longint;
	begin
		if testheap then begin
			DebugStr('AddChunk Enter;hc;g');
		end;
		if error = noErr then begin
			orgsize := MGetHandleSize(data);
			err := MGrowHandleSize(data, orgsize + tagsize + lensize + len);
			if err = noErr then begin
				if tagged then begin
					BlockMoveData(@tag, Ptr(ord(data^) + orgsize), lsize);
					orgsize := orgsize + lsize;
				end else begin
					Assert( EqualTag(tag, tagType(no_tag)) );
				end;
				if not fixed then begin
					BlockMoveData(@len, Ptr(ord(data^) + orgsize), lsize);
					orgsize := orgsize + lsize;
				end else begin
					Assert(len = size);
				end;
				BlockMoveData(p, Ptr(ord(data^) + orgsize), len);
				cnt := cnt + 1;
			end;
		end;
		if testheap then begin
			DebugStr('AddChunk Exit;hc;g');
		end;
	end;

	procedure MyCollection.InsertBefore (index: indexType);
		var
			offset, len, oe: longint;
			t: tagType;
	begin
		t := tagType(no_tag);
		if index = Count + 1 then begin
			if fixed then begin
				AddChunk(t, @index, size);
			end else begin
				AddChunk(t, @index, 0);
			end;
		end else begin
			if GetOffset(index, offset, len) then begin
				offset := offset - lensize - tagsize;
				if tagged then begin
					oe := MMungerInsert(data, offset, @t, tagsize);
					offset := offset + tagsize;
				end;
				if fixed then begin
					oe := MMungerInsert(data, offset, @index, size);
				end else begin
					len := 0;
					oe := MMungerInsert(data, offset, @len, lensize);
				end;
				if error = noErr then begin
					error := MemError;
				end;
				cnt := cnt + 1;
				InvalidateCache;
			end;
		end;
	end;

	procedure MyCollection.SetChunk (offset, l: longint; tag: tagType; p: Ptr; len: longint);
	begin
		if tagged then begin
			BlockMoveData(@tag, Ptr(ord(data^) + offset - lensize - tagsize), tagsize);
		end else begin
			Assert( EqualTag(tag, tagType(no_tag)) );
		end;
		if fixed then begin
			Assert(len = size);
		end;
		if l = len then begin
			BlockMoveData(p, Ptr(ord(data^) + offset), len);
		end else begin
			BlockMoveData(@len, Ptr(ord(data^) + offset - lensize), lensize);
			offset := Munger(data, offset, nil, l, p, len);
			if error = noErr then begin
				error := MemError;
			end;
		end;
		InvalidateCache;
	end;

	procedure MyCollection.SetChunkIndex (index: indexType; p: Ptr; len: longint);
		var
			offset, l: longint;
	begin
		if GetOffset(index, offset, l) then begin
			SetChunk(offset, l, tagType(no_tag), p, len);
		end;
	end;

	procedure MyCollection.SetChunkTag (tag: tagType; p: Ptr; len: longint);
		var
			offset, l: longint;
			index: indexType;
	begin
		if GetTagOffset(tag, offset, l, index, true) then begin
			SetChunk(offset, l, tag, p, len);
		end else begin
			AddChunk(tag, p, len);
		end;
	end;

	procedure MyCollection.SetChunkDefault (tag: tagType; p: Ptr; len: longint);
		var
			offset, l: longint;
			index: indexType;
	begin
		if not GetTagOffset(tag, offset, l, index, true) then begin
			AddChunk(tag, p, len);
		end;
	end;

	procedure MyCollection.GetChunkIndex (index: indexType; len: longint; p: Ptr);
		var
			offset, l: longint;
	begin
		if GetOffset(index, offset, l) then begin
			Assert(l = len);
			BlockMoveData(Ptr(ord(data^) + offset), p, len);
		end;
	end;

	procedure MyCollection.GetChunkTag (tag: tagType; len: longint; p: Ptr);
		var
			offset, l: longint;
			index: indexType;
	begin
		if GetTagOffset(tag, offset, l, index, safeget) then begin
			Assert(l = len);
			BlockMoveData(Ptr(ord(data^) + offset), p, len);
		end else begin
			MZero(p, len);
		end;
	end;

	procedure MyCollection.AddBoolean (b: Boolean);
		var
			n: integer;
	begin
		n := -ord(b);
		AddChunk(tagType(no_tag), @n, 1);
	end;

	procedure MyCollection.AddTagBoolean (tag: {univ } tagType; b: Boolean);
		var
			n: integer;
	begin
		n := -ord(b);
		AddChunk(tag, @n, 1);
	end;

	procedure MyCollection.AddLong (n: univ longint);
	begin
		AddChunk(tagType(no_tag), @n, lsize);
	end;

	procedure MyCollection.AddTagLong (tag: {univ } tagType; n: univ longint);
	begin
		AddChunk(tag, @n, lsize);
	end;

	procedure MyCollection.AddString (const s: Str255);
	begin
		AddChunk(tagType(no_tag), @s[1], length(s));
	end;

	procedure MyCollection.AddTagString (tag: {univ } tagType; const s: Str255);
	begin
		AddChunk(tag, @s[1], length(s));
	end;

	procedure MyCollection.AddData (p: Ptr; len: longint);
	begin
		AddChunk(tagType(no_tag), p, len);
	end;

	procedure MyCollection.AddTagData (tag: {univ } tagType; p: Ptr; len: longint);
	begin
		AddChunk(tag, p, len);
	end;

	procedure MyCollection.AddItem (p: Ptr);
	begin
		AddChunk(tagType(no_tag), p, size);
	end;

	procedure MyCollection.AddTagItem (tag: {univ } tagType; p: Ptr);
	begin
		AddChunk(tag, p, size);
	end;

	procedure MyCollection.SetBoolean (index: indexType; b: Boolean);
		var
			n: integer;
	begin
		n := -ord(b);
		SetChunkIndex(index, @n, 1);
	end;

	procedure MyCollection.SetTagBoolean (tag: {univ } tagType; b: Boolean);
		var
			n: integer;
	begin
		n := -ord(b);
		SetChunkTag(tag, @n, 1);
	end;

	procedure MyCollection.SetLong (index: indexType; n: univ longint);
	begin
		SetChunkIndex(index, @n, lsize);
	end;

	procedure MyCollection.SetTagLong (tag: {univ } tagType; n: univ longint);
	begin
		SetChunkTag(tag, @n, lsize);
	end;

	procedure MyCollection.SetString (index: indexType; const s: Str255);
	begin
		SetChunkIndex(index, @s[1], length(s));
	end;

	procedure MyCollection.SetTagString (tag: {univ } tagType; const s: Str255);
	begin
		SetChunkTag(tag, @s[1], length(s));
	end;

	procedure MyCollection.SetData (index: indexType; p: Ptr; len: longint);
	begin
		SetChunkIndex(index, p, len);
	end;

	procedure MyCollection.SetTagData (tag: {univ } tagType; p: Ptr; len: longint);
	begin
		SetChunkTag(tag, p, len);
	end;

	procedure MyCollection.SetItem (index: indexType; p: Ptr);
	begin
		SetChunkIndex(index, p, size);
	end;

	procedure MyCollection.SetTagItem (tag: {univ } tagType; p: Ptr);
	begin
		SetChunkTag(tag, p, size);
	end;

	procedure MyCollection.SetDefaultBoolean (tag: {univ } tagType; b: Boolean);
		var
			n: integer;
	begin
		n := -ord(b);
		SetChunkDefault(tag, @n, 1);
	end;

	procedure MyCollection.SetDefaultLong (tag: {univ } tagType; n: univ longint);
	begin
		SetChunkDefault(tag, @n, lsize);
	end;

	procedure MyCollection.SetDefaultString (tag: {univ } tagType; const s: Str255);
	begin
		SetChunkDefault(tag, @s[1], length(s));
	end;

	procedure MyCollection.SetDefaultData (tag: {univ } tagType; p: Ptr; len: longint);
	begin
		SetChunkDefault(tag, p, len);
	end;

	procedure MyCollection.SetDefaultItem (tag: {univ } tagType; p: Ptr);
	begin
		SetChunkDefault(tag, p, size);
	end;

	function MyCollection.GetBoolean (index: indexType): Boolean;
		var
			n: integer;
	begin
		n := 0;
		GetChunkIndex(index, 1, @n);
		GetBoolean := n <> 0;
	end;

	function MyCollection.GetTagBoolean (tag: {univ } tagType): Boolean;
		var
			n: integer;
	begin
		n := 0;
		GetChunkTag(tag, 1, @n);
		GetTagBoolean := n <> 0;
	end;

	procedure MyCollection.GetLong (index: indexType; var l: univ longint);
	begin
		GetChunkIndex(index, 4, @l);
	end;

	procedure MyCollection.GetTagLong (tag: {univ } tagType; var l: univ longint);
	begin
		GetChunkTag(tag, 4, @l);
	end;

	function MyCollection.GetString (index: indexType): Str255;
		var
			offset, l: longint;
			s: Str255;
	begin
		s := '';
		if GetOffset(index, offset, l) then begin
			Assert(l <= 255);
			BlockMoveData(Ptr(ord(data^) + offset), @s[1], l);
			s[0] := chr(l);
		end;
		GetString := s;
	end;

	function MyCollection.GetTagString (tag: {univ } tagType): Str255;
		var
			offset, l: longint;
			index: indexType;
			s: Str255;
	begin
		s := '';
		if GetTagOffset(tag, offset, l, index, safeget) then begin
			Assert(l <= 255);
			BlockMoveData(Ptr(ord(data^) + offset), @s[1], l);
			s[0] := chr(l);
		end;
		GetTagString := s;
	end;

	procedure MyCollection.GetData (index: indexType; p: Ptr; len: longint);
	begin
		GetChunkIndex(index, len, p);
	end;

	procedure MyCollection.GetTagData (tag: {univ } tagType; p: Ptr; len: longint);
	begin
		GetChunkTag(tag, len, p);
	end;

	procedure MyCollection.GetItem (index: indexType; p: Ptr);
	begin
		GetChunkIndex(index, size, p);
	end;

	procedure MyCollection.GetTagItem (tag: {univ } tagType; p: Ptr);
	begin
		GetChunkTag(tag, size, p);
	end;

end.