unit MyControls;

interface

	uses
		MacOS;
		
	const
		kControlIsInvisible = 0;
		kControlIsVisible = 255;

	function GetControlHilite (ch: ControlHandle): integer;
	procedure SetControlHilite (ch: ControlHandle; hilite: integer);
	function GetControlEnable ( ch: ControlHandle ): Boolean;
	procedure SetControlEnable (ch: ControlHandle; on: Boolean);
	function GetControlBoolean (ch: ControlHandle): Boolean;
	procedure SetControlBoolean (ch: ControlHandle; value: Boolean);
	procedure ToggleControlBoolean (ch: ControlHandle);
	procedure CleanMoveControl( ch: ControlHandle; h, v: SInt16 );
	procedure CleanSizeControl( ch: ControlHandle; h, v: SInt16 );
	procedure CleanShowHideControl( ch: ControlHandle; visible: Boolean );
	procedure SetControlRect( ch: ControlHandle; const frame: Rect );

implementation

	uses
		MyAssertions, MyMathUtils;
		
	function GetControlHilite (ch: ControlHandle): integer;
	begin
		Assert( ch <> nil );
		GetControlHilite := ch^^.contrlHilite;
	end;

	procedure SetControlHilite (ch: ControlHandle; hilite: integer);
	begin
		Assert( ch <> nil );
		if ch^^.contrlHilite <> hilite then begin
			HiliteControl(ch, hilite);
		end;
	end;

	function GetControlEnable ( ch: ControlHandle ): Boolean;
	begin
		Assert( ch <> nil );
		GetControlEnable := GetControlHilite(ch) <> 255;
	end;

	procedure SetControlEnable (ch: ControlHandle; on: Boolean);
	begin
		Assert( ch <> nil );
		SetControlHilite(ch, 255 * ord(not on))
	end;

	function GetControlBoolean (ch: ControlHandle): Boolean;
	begin
		GetControlBoolean := GetControlValue(ch) <> 0;
	end;

	procedure SetControlBoolean (ch: ControlHandle; value: Boolean);
	begin
		SetControlValue(ch, ord(value));
	end;

	procedure ToggleControlBoolean (ch: ControlHandle);
	begin
		SetControlBoolean(ch, not GetControlBoolean(ch));
	end;
	
	procedure CleanMoveControl( ch: ControlHandle; h, v: SInt16 );
	begin
		Assert( ch <> nil );
		if (ch^^.contrlRect.left <> h) | (ch^^.contrlRect.top <> v) then begin
			MoveControl( ch, h, v );
		end;
	end;

	procedure CleanShowHideControl( ch: ControlHandle; visible: Boolean );
	begin
		if visible <> (ch^^.contrlVis <> kControlIsInvisible) then begin
			if visible then begin
				ShowControl( ch );
			end else begin
				HideControl( ch );
			end;
		end;
	end;
	
	procedure CleanSizeControl( ch: ControlHandle; h, v: SInt16 );
	begin
		Assert( ch <> nil );
		if (RectWidth( ch^^.contrlRect ) <> h) | (RectHeight( ch^^.contrlRect ) <> v) then begin
			SizeControl( ch, h, v );
		end;
	end;

	procedure SetControlRect( ch: ControlHandle; const frame: Rect );
	begin
		Assert( ch <> nil );
		CleanMoveControl( ch, frame.left, frame.top );
		CleanSizeControl( ch, RectWidth( frame ), RectHeight( frame ) );
	end;

end.
