unit MyDesktopDB;

interface

	uses
		MacOS;
		
	function GetDesktopDB (vrn: integer; var rn: integer): OSErr;
{ You can safely ignore the error and use the returned rn in calls to Get/Set, they will just bounce }
	procedure GetDTDBComment (rn: integer; const fs: FSSpec; var s: Str255);
	procedure SetDTDBComment (rn: integer; const fs: FSSpec; const s: Str255);
	
	function GetDTDBIconSuite( vrn: integer; fcreator, ftype: OSType; var iconsuite: Handle ): OSErr;
	function GetDTDBIconSuiteAnyVolume( fcreator, ftype: OSType; var iconsuite: Handle ): OSErr;

implementation

	uses
		MyTypes, MyStrings, MyMemory;

	function GetDesktopDB (vrn: integer; var rn: integer): OSErr;
		var
			pb: DTPBRec;
			err: OSErr;
	begin
		rn := bad_rn;
		pb.ioNamePtr := nil;
		pb.ioVRefNum := vrn;
		err := PBDTGetPath(@pb);
		if err = noErr then begin
			rn := pb.ioDTRefNum;
		end;
		GetDesktopDB := err;
	end;

	procedure GetDTDBComment (rn: integer; const fs: FSSpec; var s: Str255);
		var
			pb: DTPBRec;
			oe: OSErr;
	begin
		s := '';
		if rn <> bad_rn then begin
			pb.ioNamePtr := @fs.name;
			pb.ioDTRefNum := rn;
			pb.ioDTBuffer := @s[1];
			pb.ioDirID := fs.parID;
			pb.ioDTReqCount := SizeOf(s)-1;
			oe := PBDTGetCommentSync(@pb);
			if oe = noErr then begin
				s[0] := chr(pb.ioDTActCount);
			end else
				s := '';
		end;
	end;

	procedure SetDTDBComment (rn: integer; const fs: FSSpec; const s: Str255);
		var
			pb: DTPBRec;
			oe: OSErr;
	begin
		if rn <> bad_rn then begin
			pb.ioNamePtr := @fs.name;
			pb.ioDTRefNum := rn;
			pb.ioDTBuffer := @s[1];
			pb.ioDTReqCount := length(s);
			pb.ioDirID := fs.parID;
			oe := PBDTSetCommentSync(@pb);
		end;
	end;

	function GetDTDBIconSuite( vrn: integer; fcreator, ftype: OSType; var iconsuite: Handle ): OSErr;
		const
			icon_types = 'ICN#icl4icl8ics#ics4ics8';
			icon_codes = '123456';
		var
			err: OSErr;
			rn: integer;
			data: packed array[1..kLarge8BitIconSize] of Byte;
			pb: DTPBRec;
			anygood: Boolean;
			temps: Str255;
			iconres: ResType;
			icont: integer;
	begin
		iconsuite := nil;
		anygood := false;
		err := GetDesktopDB( vrn, rn );
		if err = noErr then begin
			err := MNewIconSuite( iconsuite );
		end;
		if err = noErr then begin
			anygood := false;
			for icont := 1 to length(icon_codes) do begin
				pb.ioDTRefNum := rn;
				pb.ioTagInfo := 0;
				pb.ioFileCreator := fcreator;
				pb.ioFileType := ftype;
				temps := copy( icon_types, icont*4-3, 4 );
				iconres := StringToOSType( temps );
				pb.ioIconType := ord(icon_codes[icont]) - 48;
				pb.ioDTBuffer := @data;
				pb.ioDTReqCount := SizeOf(data);
				err := PBDTGetIconSync( @pb );
				if (err = noErr) & (pb.ioDTActCount > SizeOf(data)) then begin
					err := -17011;
				end;
				if err = noErr then begin
					err := MAddIconDataToSuite( @data, pb.ioDTActCount, iconsuite, iconres );
					if err = noErr then begin
						anygood := true;
					end;
				end;
			end;
			if anygood then begin
				err := noErr;
			end else begin
				MDisposeIconSuite( iconsuite );
			end;
		end;
		GetDTDBIconSuite := err;
	end;
	
	function GetDTDBIconSuiteAnyVolume( fcreator, ftype: OSType; var iconsuite: Handle ): OSErr;
		var
			err: OSErr;
			index: integer;
			pb: HParamBlockRec;
	begin
		index := 1;
		while true do begin
			pb.ioNamePtr := nil;
			pb.ioVolIndex := index;
			err := PBHGetVInfoSync(@pb);
			if err <> noErr then begin
				leave;
			end;
			err := GetDTDBIconSuite( pb.ioVRefNum, fcreator, ftype, iconsuite );
			if err = noErr then begin
				leave;
			end;
			Inc(index);
		end;
		if err <> noErr then begin
			iconsuite := nil;
		end;
		GetDTDBIconSuiteAnyVolume := err;
	end;

end.
	procedure DisposeDTDBIconSuite( var iconsuite: Handle );
		var
			err: OSErr;
	begin
		if iconsuite <> nil then begin
			MForgetMemory( iconsuite );
			err := DisposeIconSuite( iconsuite, true );
			Assert( err = noErr );
			iconsuite := nil;
		end;
	end;
	

Database Access???

		oe := Gestalt(gestaltDBAccessMgrAttr, v);
		if (oe = noErr) & not BTST(v, gestaltDBAccessMgrPresent) then begin
			oe := -1;
		end;
		if oe = noErr then begin
