unit MyEnterPassword;

interface

	uses
		MacOS;

	procedure CreateEnterPassword (var p: Ptr; dlg: DialogPtr; item: integer);
	procedure DisposeEnterPassword (var p: Ptr);
	function IsEnterPassword( p: Ptr; dlg: DialogPtr; item: integer ): Boolean;
	procedure GetEnterPasswordItem( p: Ptr; var dlg: DialogPtr; var item: integer );
	procedure HandleEnterPassword (p: Ptr; var er: EventRecord);
	procedure GetEnterPassword (p: Ptr; var password: Str255);
	procedure SetEnterPassword (var p: Ptr; password: Str255);
	procedure PasteEnterPassword (p: Ptr; data: Handle);
	procedure DeleteEnterPassword (p: Ptr);

implementation

	uses
		MyStrings, MyTypes, MyUtils, MyDialogs, MyEvents, MyLowLevel, MyMemory, MyAssertions;

	const
		my_bullet = '';
		ep_magic = 'EPMg';

	type
		EPRecord = record
				magic: OSType;
				teh: TEHandle;
				dlg: DialogPtr;
				item: integer;
			end;
		EPRecordPtr = ^EPRecord;

{$ifc not do_debug}
{$definec AssertValidEPRecord(p)}
{$elsec}
{$definec AssertValidEPRecord(p) AssertValidEPRecordCode(p)}
{$endc}

{$ifc do_debug}
	procedure AssertValidEPRecordCode( p: univ EPRecordPtr);
	begin
		Assert( (p <> nil) & (p^.magic = ep_magic) & (p^.teh <> nil) & (p^.dlg <> nil) );
		AssertValidDialogItem( p^.dlg, p^.item );
	end;
{$endc}

	function IsEnterPassword( p: Ptr; dlg: DialogPtr; item: integer ): Boolean;
		var
			ep: EPRecordPtr;
	begin
		AssertValidEPRecord( p );
		ep := EPRecordPtr(p);
		IsEnterPassword := (dlg = ep^.dlg) & (item = ep^.item);
	end;
	
	procedure GetEnterPasswordItem( p: Ptr; var dlg: DialogPtr; var item: integer );
		var
			ep: EPRecordPtr;
	begin
		AssertValidEPRecord( p );
		ep := EPRecordPtr(p);
		dlg := ep^.dlg;
		item := ep^.item;
	end;
	
	procedure CreateEnterPassword (var p: Ptr; dlg: DialogPtr; item: integer);
		var
			dest, view: Rect;
			ep: EPRecordPtr;
			junk: OSErr;
	begin
		SetPort(dlg);
		junk := MNewPtr( ep, SizeOf(EPRecord) );
		SetRect(dest, 8000, 8010, 8100, 8020);
		view := dest;
		ep^.magic := ep_magic;
		ep^.teh := TENew(dest, view);
		ep^.dlg := dlg;
		ep^.item := item;
{    data := TEGetText(ep^.teh);}
		p := Ptr(ep);
		AssertValidEPRecord( p );
	end;

	procedure DisposeEnterPassword (var p: Ptr);
		var
			ep: EPRecordPtr;
	begin
		AssertValidEPRecord( p );
		ep := EPRecordPtr(p);
		TEDispose(ep^.teh);
		ep^.magic := '????';
		MDisposePtr( ep );
		p := nil;
	end;

	procedure SetEnterPassword (var p: Ptr; password: Str255);
		var
			ep: EPRecordPtr;
			i: integer;
	begin
		AssertValidEPRecord( p );
		ep := EPRecordPtr(p);
		TESetText(@password[1], length(password), ep^.teh);
		for i := 1 to length(password) do begin
			password[i] := my_bullet;
		end;
		SetItemText(ep^.dlg, ep^.item, password);
	end;

	procedure GetEnterPassword (p: Ptr; var password: Str255);
		var
			ep: EPRecordPtr;
			data: CharsHandle;
	begin
		AssertValidEPRecord( p );
		ep := EPRecordPtr(p);
		data := TEGetText(ep^.teh);
		HandleToString(data, password);
	end;

	procedure HandleEnterPassword (p: Ptr; var er: EventRecord);
		var
			ep: EPRecordPtr;
			ch: Char;
			selitem, selstart, selfin: integer;
	begin
		AssertValidEPRecord( p );
		ep := EPRecordPtr(p);
		if (ep^.dlg = FrontWindow) & (SelectedTextItem(ep^.dlg) = ep^.item) then begin
			if EventIsKeyDown( er ) & not EventHasCommandKey( er ) then begin
				ch := EventChar( er );
				case ch of
					cr, esc, enter, tab, leftArrow, rightArrow, upArrow, downArrow: 
						;
					otherwise begin
						if (ch <> bs) & (ch < spc) then begin
							SysBeep(1);
							er.what := nullEvent;
						end else begin
							GetDialogTextSelection( ep^.dlg, selitem, selstart, selfin );
							Assert( selitem = ep^.item );
							TESetSelect( selstart, selfin, ep^.teh );
							TEKey( ch, ep^.teh );
							if (ch <> bs) & DirtyKey(ch) then begin
								er.message := ord(my_bullet);
							end;
						end;
					end;
				end;
			end;
		end;
	end;

	procedure PasteEnterPassword (p: Ptr; data: Handle);
		var
			ep: EPRecordPtr;
			len: longint;
			i: integer;
			s: Str255;
			bad: Boolean;
			selitem, selstart, selfin: integer;
	begin
		AssertValidEPRecord( p );
		ep := EPRecordPtr(p);
		if (SelectedTextItem(ep^.dlg) = ep^.item) then begin
			GetItemText( ep^.dlg, ep^.item, s );
			GetDialogTextSelection( ep^.dlg, selitem, selstart, selfin );
			Assert( selitem = ep^.item );
			if data <> nil then begin
				len := MGetHandleSize(data);
			end else begin
				len := 0;
			end;
			bad := length(s) - (selfin - selstart) + len > 255;
			if not bad then begin
				for i := 0 to len - 1 do begin
					if BAND(AddPtrLong(data^, i)^, $FF) < 32 then begin
						bad := true;
						leave;
					end;
				end;
			end;
			if bad then begin
				SysBeep(1);
			end else begin
				TESetSelect( selstart, selfin, ep^.teh );
				TEDelete(ep^.teh);
				if data <> nil then begin
					MHLock(data);
					TEInsert(data^, len, ep^.teh);
					MHUnlock( data );
				end;
				s[0] := chr(length(s) - (selfin - selstart) + len);
				for i := 1 to length(s) do begin
					s[i] := my_bullet;
				end;
				selfin := selstart + len;
				SetItemText( ep^.dlg, ep^.item, s );
				SelectDialogItemText( ep^.dlg, ep^.item, selstart, selfin );
			end;
		end;
	end;


	procedure DeleteEnterPassword (p: Ptr);
	begin
		PasteEnterPassword( p, nil );
	end;

end.