unit MyEthernet;

interface

	uses
		MacOS;

	type
		EthernetAddress = packed array[0..5] of Byte;

	function GetEthernetAddressDriver( var addr: EthernetAddress ): OSStatus;
	function GetEthernetAddressOT( var addr : EthernetAddress ): OSStatus;
{  Assumes InitOpenTransport has been called. }

	function GetEthernetAddress( var addr : EthernetAddress; opencloseot: boolean ): OSStatus;

implementation

	uses
		Slots, ROMDefs, ENET, OpenTptLinks, 
//		Devices, ENET, Slots, ROMDefs, GestaltEqu, OpenTransport, OpenTptLinks,
		MyCStrings, MyOTUtils;

	function OpenEthernetDriver (var ern: integer): OSStatus;
		var
			sb: SpBlock;
			pb: ParamBlockRec;
			err, err2: OSStatus;
			found: boolean;
			EnetStr: Str15;
	begin
		found := false;                   {assume no sResource found}

		{set up the SpBlock}
		sb.spCategory := catNetwork;
		sb.spCType := typeEtherNet;
		sb.spDrvrSW := 0;
		sb.spDrvrHW := 0;
		sb.spTBMask := 3;		{match only Category and CType }
		sb.spSlot := 0;				{start search from here}
		sb.spID := 0;					{start search from here}
		sb.spExtDev := 0;			{ID of the external device}

		err := SNextTypeSRsrc(@sb);
		if err = noErr then begin			{found an sResource match; }
			EnetStr := '.ENET';
			pb.ioCompletion := nil;			{call made synchronously}
			pb.ioNamePtr := @EnetStr;
			pb.ioPermssn := fsCurPerm;
			pb.ioSFlags := 0; 	{reserved for driver use}
			pb.ioSlot := sb.spSlot;			{slot of Ethernet card to open}
			pb.ioID := sb.spID;	{sResource ID for slot}
			err := OpenSlotSync( @pb );
			if err = noErr then begin
				ern := pb.ioRefNum;
			end;
		end;
		if err <> noErr then begin
			err := OpenDriver( '.ENET', ern );
		end;
		if err <> noErr then begin
			err2 := OpenDriver( '.ENET0', ern );	{ check for SCSI Ethernet devices }
			if err2 = noErr then begin
				err := noErr;
			end;
		end;
		OpenEthernetDriver := err;
	end;

	function GetEthernetAddressDriver (var addr: EthernetAddress): OSStatus;
		var
			err: OSStatus;
			buffer: packed array[0..77] of Byte;
			pb: EParamBlock;
			ern: integer;
	begin
		err := OpenEthernetDriver(ern);
		if err = noErr then begin
			pb.ioRefNum := ern;
			pb.csCode := ENetGetInfo;
			pb.ePointer := @buffer;
			pb.eBuffSize := sizeof(buffer);
			err := EGetInfo(@pb, false);
			if err = noErr then begin
				BlockMoveData(@buffer, @addr, 6);
			end;
		end;
		GetEthernetAddressDriver := err;
	end;

	function GetEthernetAddressOT( var addr : EthernetAddress) : OSStatus;
		var
			ep : EndpointRef ;
			err : OSStatus;
			bind_request, bind_result : TBind;
			theReturnAddr : T8022Address;
			simple_addr : T8022Address;
			i : integer;
			myEnetName : Str255;
			myOTCP : OTConfigurationPtr;
	begin
		myEnetName := kEnetName;
		P2C(@myEnetName);
		with simple_addr do begin
			fAddrFamily:= AF_8022;			{ OTAddressType for 802 provider }
			for i := 0 to 5 do begin
				fHWAddr[i] := $00;			{ hardware address, use zeros for bind requests }
			end;
			fSAP := $8888;
	{ SAP / protocol type, value > 1500 implies ethernet protocol with no SNAP }
			for i := 0 to 4 do begin
				fSNAP[i] := $00;
			end;
		end;
		myOTCP := OTCreateConfiguration(@myEnetName);
		ep := OTOpenEndpoint( myOTCP, 0, nil, OSStatus(err));
		if err = kOTNoError then begin
			// finish bind request
			bind_request.addr.buf := @simple_addr;
			bind_request.addr.len := k8022BasicAddressLength; // family type + Ethernet + type field
			bind_request.addr.maxlen := 0;
			bind_request.qlen := 0;
			// setup bind result
			bind_result.addr.buf := @theReturnAddr;
			bind_result.addr.len := 0;
			bind_result.addr.maxlen := sizeof(theReturnAddr);
			bind_result.qlen := 0;
			// call bind
			err := OTBind(ep, @bind_request, @bind_result);
			if (err = noErr) then begin
				{ Should be:
						BlockMove( Ptr(@theReturnAddr.fHWAddr), Ptr(@addr), 6);
				  ...but this failed, and I don't know why...}
				for i := 0 to 5 do begin
					addr[i] := theReturnAddr.fHWAddr[i];
				end;
			  err := OTUnbind(ep);  // clean up
			end;
			err := OTCloseProvider(ep);
		end;
		GetEthernetAddressOT := err;
	end;

	function GetEthernetAddress( var addr : EthernetAddress; opencloseot: boolean ): OSStatus;
		var
			err: OSStatus;
	begin
		if HasOT then begin
			if opencloseot then begin
				err := InitOpenTransport();
			end else begin
				err := noErr;
			end;
			if err = noErr then begin
				err := GetEthernetAddressOT( addr );
				if opencloseot then begin
					CloseOpenTransport();
				end;
			end;
		end else begin
			err := -1;
		end;
		if err <> noErr then begin
			err := GetEthernetAddressDriver( addr );
		end;
		GetEthernetAddress := err;
	end;

end.

(*

	function OpenEthernetDriver (var ern: integer): OSErr;
		var
			err: OSErr;
	begin
		err := OpenDriver('.ENET', ern);
		if err <> noErr then begin
			err := OpenDriver('.ENET0', ern);		{
check for SCSI Ethernet devices }
		end; { if }
		OpenEthernetDriver := err;
	end;

*)
