unit MyFileSystemUtils;

interface

	uses
		MacOS;

	type
		ScanProc = function(const fs:FSSpec; folder:Boolean; path:Str255; var pb:CInfoPBRec):Boolean;
{ for folders, return true to scan contents }
{ for files return true if you delete the file - other changes to the file system would be bad... }

	const
		kApplicationFolderType = 'Appl';
	
	const
		kZapDF = true;
		kDontZapDF = false;
		kZapRF = true;
		kDontZapRF = false;
		
	procedure MyResolveAliasFile (var fs: FSSpec);
	function MyGetCatInfo (vrn: integer; dirID: longint; var name: Str63; index: integer; var pb: CInfoPBRec): OSErr;
	function FSpGetCatInfo (const fs: FSSpec; var pb: CInfoPBRec): OSErr;
	function FSpGetIndCatInfo (var fs: FSSpec; index: integer; var pb: CInfoPBRec): OSErr;
	function FSpSetCatInfo (const spec: FSSpec; var pb: CInfoPBRec): OSErr;
	function GetParentID( vrn: integer; dirID: longint; var parID: longint ): OSErr;
	function GetVolName( vrn: integer; var name: Str63 ): OSErr;
	function GetDirFSSpec( vrn: integer; dirID: longint; var spec: FSSpec ): OSErr;
	function FSpGetParID( const spec: FSSpec; var dirID: longint ): OSErr;
	function PBIsFolder( const pb: CInfoPBRec ): Boolean;
	function FSpIsFolder( const spec: FSSpec; var isfolder: Boolean ): OSErr;
	function HGetDirID( vrn: integer; parID: longint; const s: string; var dirID: longint ): OSErr;
	function FSpGetDirID( const spec: FSSpec; var dirID: longint ): OSErr;
	procedure MyGetModDate (const spec: FSSpec; var moddate: UInt32);
	function FSpCatMoveAndRename( const source: FSSpec; const dest: FSSpec ): OSErr;
	procedure FSpZapForks( const spec: FSSpec; zapdf, zaprf: Boolean );
	procedure FSpRecreate( const spec: FSSpec; creator: OSType; fileType: OSType; zapdf, zaprf: Boolean );
	procedure FSpRecreateResFile( const spec: FSSpec; creator: OSType; fileType: OSType; zapdf, zaprf: Boolean );
	function TrashFile( const spec: FSSpec): OSErr;
	function DuplicateFile (const org, newfs: FSSpec): OSErr;
	function CopyData (src, dst: integer; len: longint): OSErr;
	function TouchDir (fs: FSSpec): OSErr;
	function TouchFolder (vrn: integer; dirID: longint): OSErr;
	function UniqueNameCameFromOriginal( const unique, org: string ): Boolean;
	function CreateUniqueFile (var fs: FSSpec; creator, ftype: OSType): OSErr;
	function CreateUniqueFolder (var fs: FSSpec; var dirID: longint ): OSErr;
	function MyPBReadEOL( refnum: integer; buffer: Ptr; maxlen: longint; var len: longint; eol: char ): OSErr;
	function MyPBReadEOLAt( refnum: integer; buffer: Ptr; maxlen: longint; var len: longint; eol: char; var pos: longint ): OSErr;
	function MyFSReadLineEOL (refnum: integer; eol: char; var s: Str255): OSErr;
	function MyFSReadLineEOLAt(refnum: integer; var pos: longint; eol: char; var s: Str255): OSErr;
	function MyFSReadLineBothAt(refnum: integer; var pos: longint; var s: Str255): OSErr;
	function MyFSReadLine (refnum: integer; var s: Str255): OSErr;
	function MyFSReadLineAt (refnum: integer; pos: longint; var s: Str255): OSErr;
	function MyFSRead(refnum: integer; len: longint; p: Ptr): OSErr;
	function MyFSWriteString( refnum: integer; const s: string ): OSErr;
	function MyFSWrite (refnum: integer; len: longint; p: Ptr): OSErr;
	function MyFSWriteAt (refnum: integer; mode: integer; pos, len: longint; p: Ptr): OSErr;
	function MyFSReadAt (refnum: integer; pos, len: longint; p: Ptr): OSErr;
	function MyFSReadFile( const spec: FSSpec; var data: Handle ): OSErr;
	function MyFSWriteFile( const spec: FSSpec; data_fork: Boolean; data: Handle ): OSErr; // You must create the file first
	function FSSpecToFullPath (fs: FSSpec; var path: Str255): OSErr;
	function DiskFreeSpace (vrn: integer): longint; { result in k }
	function DiskSize (vrn: integer): longint; { result in k }
	function BlessSystemFolderDirectory( vrn: integer; dirID: longint ): OSErr;
	function BlessSystemFolder( const spec: FSSpec ): OSErr;
	function SameFSSpec (const fs1, fs2: FSSpec): Boolean;
	function IsRemoteVolume( vRefNum: integer ): Boolean;
	function CreateTemporaryFile (var fs: FSSpec): OSErr;
	function GetDirID (wdrn: integer; var vrn: integer; var dirID: longint): OSErr;
	function GetVolInfo (var name: Str63; var vrn: integer; index: integer; var CrDate: longint): OSErr;
	function GetVolumeAddrBlock(vrn: integer; index: integer; var addr: AddrBlock): OSErr;
	function ScanDirectory (fs: FSSpec; doit: ScanProc): OSErr;
	function RemoveResourceFork( const spec: FSSpec ): OSErr;
	function FindTemporaryFolder( vrn: integer; var resultvrn: integer; var resultparID: longint ): OSErr;
	function PseudoMakeFSSpec( pseudopath: Str255; createFolder: Boolean; var spec: FSSpec ): OSErr;
		// pseudopath is four characters FindFolder type followed by file/folder name
	function PseudoMakeFolderSpec( folderType: OSType; createFolder: Boolean; var spec: FSSpec ): OSErr;
	function MyFindFolder( folderType: OSType; createFolder: Boolean; var foundVRN: integer; var foundDirID: longint ): OSErr;
		// uses STR# 7700 to add new folder types
	procedure AutoRename( strh_id: integer );
	function CountFiles( vrn: integer; dirID: longint; var count: longint ): OSErr;
	function AliasToFullpath( alias: AliasHandle; var path: Str255 ): OSErr;
	
implementation

	uses
		OpenTptAppleTalk, 
		MyTypes, MyStrings, MyMemory, MyMathUtils, MyUtils, MySystemGlobals, MyLowLevel, MyResources;

	function CreateTemporaryFile (var fs: FSSpec): OSErr;
		var
			err: OSErr;
	begin
		err := FindTemporaryFolder( kOnSystemDisk, fs.vRefNum, fs.parID );
		if err = noErr then begin
			err := CreateUniqueFile( fs, 'trsh', 'trsh' );
		end;
		CreateTemporaryFile := err;
	end;

	function FSSpecToFullPath (fs: FSSpec; var path: Str255): OSErr;
		var
			err: OSErr;
			pb: CInfoPBRec;
			s: Str63;
	begin
		s := fs.name;
		err := FSMakeFSSpec(fs.vRefNum, fs.parID, s, fs);
		if err = fnfErr then begin
			err := noErr;
		end;
		if err = noErr then begin
			if fs.parID = 1 then begin
				path := concat(fs.name, ':');
			end else begin
				path := fs.name;
				while (err = noErr) & (fs.parID <> 1) do begin
					err := FSpGetIndCatInfo(fs, -1, pb);
					path := concat(fs.name, ':', path);
					fs.parID := pb.ioFlParID;
				end;
			end;
		end;
		FSSpecToFullPath := err;
	end;

	function TouchDir (fs: FSSpec): OSErr;
		var
			pb: CInfoPBRec;
			err: OSErr;
	begin
		if fs.name = '' then begin
			TouchDir := TouchFolder(fs.vRefNum, fs.parID);
		end else begin
			pb.ioVRefNum := fs.vRefNum;
			pb.ioDirID := fs.parID;
			pb.ioNamePtr := @fs.name;
			pb.ioFDirIndex := 0;
			err := PBGetCatInfoSync(@pb);
			if err = noErr then begin
				pb.ioNamePtr := nil;
				GetDateTime(pb.ioDrMdDat);
				err := PBSetCatInfoSync(@pb);
			end;
			TouchDir := err;
		end;
	end;

	function TouchFolder (vrn: integer; dirID: longint): OSErr;
		var
			pb: CInfoPBRec;
			err: OSErr;
	begin
		pb.ioVRefNum := vrn;
		pb.ioDirID := dirID;
		pb.ioNamePtr := nil;
		pb.ioFDirIndex := -1;
		err := PBGetCatInfoSync(@pb);
		if err = noErr then begin
			pb.ioVRefNum := vrn;
			pb.ioDirID := dirID;
			pb.ioNamePtr := nil;
			GetDateTime(pb.ioDrMdDat);
			err := PBSetCatInfoSync(@pb);
		end;
		TouchFolder := err;
	end;

	const
		number_seperator = '#';
		truncate_seperator = '';

	function HardUniqueNameCameFromOriginal (const unique, org: string): Boolean;
		function M (u, o: integer): Boolean;
			var
				state: (searching, failed, success);
		begin
			state := searching;
			while state = searching do begin
				case ord(u <= length(unique)) * 2 + ord(o <= length(org)) of
					0:  begin // both finished
						state := success;
					end;
					1:  begin // only org remains
						state := failed;
					end;
					2:  begin // only unique remains
						while (u < length(unique)) & (unique[u] = number_seperator) & IsDigit( unique[u+1] ) do begin
							Inc(u);
							while (u <= length(unique)) & IsDigit(unique[u]) do begin
								Inc(u);
							end;
						end;
						if u <= length(unique) then begin
							state := failed;
						end else begin
							state := success;
						end;
					end;
					3:  begin // both remain
						case unique[u] of
							number_seperator:  begin
								if (org[o] = number_seperator) & M(u+1, o+1) then begin
									state := success;
								end else if (u < length(unique)) & IsDigit(unique[u+1]) then begin
									state := failed;
									Inc(u);
									while (u <= length(unique)) & IsDigit(unique[u]) do begin
										Inc(u);
										if M(u,o) then begin
											state := success;
											leave;
										end;
									end;
								end else begin
									state := failed;
								end;
							end;
							truncate_seperator:  begin
								Inc(u);
								Inc(o);
								if (u > length(unique)) | M(u, o) then begin { short circuit the * at the end case }
									state := success;
								end else begin
									state := failed;
									while o <= length(org) do begin
										if M(u, o) then begin
											state := success;
											leave;
										end;
										Inc(o);
									end;
								end;
							end;
							otherwise begin
								if org[o] <> unique[u] then begin
									state := failed;
								end else begin
									Inc(o);
									Inc(u);
								end;
							end;
						end;
					end;
				end;
			end;
			M := state = success;
		end;
	begin
		HardUniqueNameCameFromOriginal := M(1, 1);
	end;

	function UniqueNameCameFromOriginal( const unique, org: string ): Boolean;
		// truncate_seperator matches .+
		// #\d+ matches itself of the empty string
		var
			match: Boolean;
			p: integer;
	begin
		if unique = org then begin // short circuit simple xcase
			match := true;
		end else begin
			p := Pos( number_seperator, unique );
			if (p > 0) & IsPrefix( org, copy( unique, 1, p - 1 ) ) & IsExtension( unique, TPcopy( org, p, 255 ) ) then begin
				match := true;
			end else begin
				match := HardUniqueNameCameFromOriginal( unique, org );
			end;
		end;
		UniqueNameCameFromOriginal := match;
	end;
	
	function CreateUniqueObject( var spec: FSSpec; folder: boolean; creator, ftype: OSType; var dirID: longint ): OSErr;
		function Create: OSErr;
		begin
			if folder then begin
				Create := FSpDirCreate( spec, 0, dirID );
			end else begin
				Create := FSpCreate( spec, creator, ftype, 0 );
			end;
		end;
		
		var
			err: OSErr;
			left, right, ni: Str255;
			i: integer;
	begin
		LimitStringLength( spec.name, 31, truncate_seperator );
		err := Create;
		if err = dupFNErr then begin
			left := spec.name;
			if Pos( '.', spec.name ) > 0 then begin
				SplitRightBy( spec.name, '.', left, right );
				if IsAllInSet( right, ['a'..'z','A'..'Z','0'..'9'] ) then begin
					right := '.' + right;
				end else begin
					left := spec.name;
					right := '';
				end;
			end else begin
				left := spec.name;
				right := '';
			end;
			i := 1;
			while err = dupFNErr do begin
				ni := number_seperator + NumToStr( i );
				LimitStringLength( left, Max( 1, 31 - length(right) - length( ni ) ), truncate_seperator );
				LimitStringLength( right, Max( 1, 31 - length(left) - length( ni ) ), truncate_seperator );
				spec.name := left + ni + right;
				Inc(i);
				err := Create;
			end;
		end;
		CreateUniqueObject := err;
	end;

	function CreateUniqueFile (var spec: FSSpec; creator, ftype: OSType): OSErr;
		var
			junk_dirID: longint;
	begin
		CreateUniqueFile := CreateUniqueObject ( spec, false, creator, ftype, junk_dirID );
	end;

	function CreateUniqueFolder (var spec: FSSpec; var dirID: longint ): OSErr;
	begin
		CreateUniqueFolder := CreateUniqueObject ( spec, true, '????', '????', dirID );
	end;

	function MyFSReadAt (refnum: integer; pos, len: longint; p: Ptr): OSErr;
		var
			pb: ParamBlockRec;
			oe: OSErr;
	begin
		pb.ioRefNum := refnum;
		pb.ioBuffer := p;
		pb.ioReqCount := len;
		pb.ioPosMode := fsFromStart;
		pb.ioPosOffset := pos;
		oe := PBReadSync(@pb);
		if (oe = noErr) & (pb.ioActCount <> len) then begin
			oe := -17012;
		end;
		MyFSReadAt := oe;
	end;

	function MyPBReadEOL( refnum: integer; buffer: Ptr; maxlen: longint; var len: longint; eol: char ): OSErr;
		var
			pb: ParamBlockRec;
			err: OSErr;
	begin
		pb.ioRefNum := refnum;
		pb.ioBuffer := buffer;
		pb.ioReqCount := maxlen;
		pb.ioPosMode := fsFromMark + fsNewLine + BSL(ord(eol), 8);
		pb.ioPosOffset := 0;
		err := PBReadSync( @pb );
		if (err = eofErr) & (pb.ioActCount > 0) then begin
			err := noErr;
		end;
		if err = noErr then begin
			len := pb.ioActCount;
			if chr( GetUnsignedByte( buffer, len - 1 ) ) = eol then begin
				Dec(len);
			end;
		end;
		MyPBReadEOL := err;
	end;

	function MyPBReadEOLAt( refnum: integer; buffer: Ptr; maxlen: longint; var len: longint; eol: char; var pos: longint ): OSErr;
		var
			pb: ParamBlockRec;
			err: OSErr;
	begin
		pb.ioRefNum := refnum;
		pb.ioBuffer := buffer;
		pb.ioReqCount := maxlen;
		pb.ioPosMode := fsFromStart + fsNewLine + BSL(ord(eol), 8);
		pb.ioPosOffset := pos;
		err := PBReadSync( @pb );
		if (err = eofErr) & (pb.ioActCount > 0) then begin
			err := noErr;
		end;
		if err = noErr then begin
			len := pb.ioActCount;
			pos := pos + len;
			if chr( GetUnsignedByte( buffer, len - 1 ) ) = eol then begin
				Dec(len);
			end;
		end;
		MyPBReadEOLAt := err;
	end;

	function MyFSReadLineEOL (refnum: integer; eol: char; var s: Str255): OSErr;
		var
			err: OSErr;
			len: longint;
	begin
		err := MyPBReadEOL( refnum, @s[1], SizeOf(s)-1, len, eol );
		if err <> noErr then begin
			len := 0;
		end;
		s[0] := chr(len);
		MyFSReadLineEOL := err;
	end;
	
	function MyFSReadLineEOLAt(refnum: integer; var pos: longint; eol: char; var s: Str255): OSErr;
		var
			err: OSErr;
			len: longint;
	begin
		err := MyPBReadEOLAt( refnum, @s[1], SizeOf(s)-1, len, eol, pos );
		if err <> noErr then begin
			len := 0;
		end;
		s[0] := chr(len);
		MyFSReadLineEOLAt := err;
	end;

	function MyFSReadLine (refnum: integer; var s: Str255): OSErr;
	begin
		MyFSReadLine := MyFSReadLineEOL( refnum, cr, s );
	end;

	function MyFSReadLineAt (refnum: integer; pos: longint; var s: Str255): OSErr;
	begin
		MyFSReadLineAt := MyFSReadLineEOLAt( refnum, pos, cr, s);
	end;

	function MyFSReadLineBothAt(refnum: integer; var pos: longint; var s: Str255): OSErr;
		var
			err: OSErr;
			errcr, errlf: OSErr;
			poscr, poslf: longint;
			scr, slf: Str255;
	begin
		poscr := pos;
		poslf := pos;
		errcr := MyFSReadLineEOLAt( refnum, poscr, cr, scr );
		errlf := MyFSReadLineEOLAt( refnum, poslf, lf, slf );
		err := errcr;
		if err = noErr then begin
			err := errlf;
		end;
		if (err = noErr) then begin
			if length( scr ) < length( slf ) then begin
				if (length( scr ) = length( slf ) - 1) & (slf[length( slf )] = cr) then begin // crlf
					pos := poslf;
					s := scr;
				end else begin // cr
					pos := poscr;
					s := scr;
				end;
			end else begin // lf or none
				pos := poslf;
				s := slf;
			end;
		end;
		MyFSReadLineBothAt := err;
	end;

	function MyFSRead(refnum: integer; len: longint; p: Ptr): OSErr;
		var
			err: OSErr;
			count: longint;
	begin
		err := noErr;
		if len > 0 then begin
			count := len;
			err := FSRead(refnum, count, p);
			if (err = noErr) & (count <> len) then begin
				err := -17013;
			end;
		end;
		MyFSRead := err;
	end;

	function MyFSWriteString( refnum: integer; const s: string ): OSErr;
	begin
		MyFSWriteString := MyFSWrite( refnum, length(s), @s[1] );
	end;

	function MyFSWrite (refnum: integer; len: longint; p: Ptr): OSErr;
		var
			oe: OSErr;
			count: longint;
	begin
		oe := noErr;
		if len > 0 then begin
			count := len;
			oe := FSWrite(refnum, count, p);
			if (oe = noErr) & (count <> len) then begin
				oe := -17014;
			end;
		end;
		MyFSWrite := oe;
	end;

	function MyFSReadFile( const spec: FSSpec; var data: Handle ): OSErr;
		var
			err, junk: OSErr;
			rn: integer;
			filelen: longint;
	begin
		data := nil;
		err := FSpOpenDF( spec, fsRdPerm, rn );
		if err = noErr then begin
			err := GetEOF( rn,  filelen );
			if err = noErr then begin
				err := MNewHandle( data, filelen );
				if err = noErr then begin
					MHLock( data );
					err := MyFSRead( rn, filelen, data^ );
					MHUnlock( data );
				end;
			end;
			junk := FSClose( rn );
		end;
		if err <> noErr then begin
			MDisposeHandle( data );
		end;
		MyFSReadFile := err;
	end;

	function MyFSWriteFile( const spec: FSSpec; data_fork: Boolean; data: Handle ): OSErr;
		var
			err: OSErr;
			rn: integer;
			waslocked: boolean;
	begin
		if data_fork then begin
			err := FSpOpenDF( spec, fsWrPerm, rn );
		end else begin
			err := FSpOpenRF( spec, fsWrPerm, rn );
		end;
		if err = noErr then begin
			err := SetEOF( rn,  GetHandleSize( data ) );
			HLockState( data, waslocked );
			err := MyFSWriteAt( rn, fsFromStart, 0, GetHandleSize( data ), data^ );
			HRestoreLockState( data, waslocked );
			AddOSErr( err, FSClose( rn ) );
		end;
		MyFSWriteFile := err;
	end;
	
	procedure MyResolveAliasFile (var fs: FSSpec);
		var
			isfolder, wasalias: Boolean;
			temp: FSSpec;
			gv: longint;
			oe: OSErr;
	begin
		if (Gestalt(gestaltAliasMgrAttr, gv) = noErr) & (BTST(gv, gestaltAliasMgrPresent)) then begin
			temp := fs;
			oe := ResolveAliasFile(fs, true, isfolder, wasalias);
			if oe <> noErr then begin
				fs := temp;
			end;
		end;
	end;

	function MyGetCatInfo (vrn: integer; dirID: longint; var name: Str63; index: integer; var pb: CInfoPBRec): OSErr;
	begin
		pb.ioVRefNum := vrn;
		pb.ioDirID := dirID;
		pb.ioNamePtr := @name;
		pb.ioFDirIndex := index;
		MyGetCatInfo := PBGetCatInfoSync(@pb);
	end;
	
	function CountFiles( vrn: integer; dirID: longint; var count: longint ): OSErr;
		var
			err: OSErr;
			pb: CInfoPBRec;
			name: Str63;
	begin
		count := 0;
		name := '';
		err := MyGetCatInfo( vrn, dirID, name, -1, pb );
		if (err = noErr) & not PBIsFolder( pb ) then begin
			err := dirNFErr;
		end;
		if err = noErr then begin
			count := pb.ioDrNmFls;
		end;
		CountFiles := err;
	end;
	
	function GetParentID( vrn: integer; dirID: longint; var parID: longint ): OSErr;
		var
			err: OSErr;
			pb: CInfoPBRec;
			name: Str63;
	begin
		name := '';
		err := MyGetCatInfo( vrn, dirID, name, -1, pb );
		if err = noErr then begin
			parID := pb.ioDrParID;
		end;
		GetParentID := err;
	end;
	
	function GetDirFSSpec( vrn: integer; dirID: longint; var spec: FSSpec ): OSErr;
		var
			err: OSErr;
			pb: CInfoPBRec;
	begin
		spec.name := '';
		err := MyGetCatInfo( vrn, dirID, spec.name, -1, pb );
		if err = noErr then begin
			spec.vRefNum := vrn;
			spec.parID := pb.ioDrParID;
		end;
		GetDirFSSpec := err;
	end;

	function GetVolName( vrn: integer; var name: Str63 ): OSErr;
		var
			err: OSErr;
			pb: CInfoPBRec;
	begin
		name := '';
		err := MyGetCatInfo( vrn, 2, name, -1, pb );
		GetVolName := err;
	end;

	function FSpGetParID( const spec: FSSpec; var dirID: longint ): OSErr;
		var
			err: OSErr;
			pb: CInfoPBRec;
	begin
		err := FSpGetCatInfo( spec, pb );
		if err = noErr then begin
			dirID := pb.ioDrParID;
		end;
		FSpGetParID := err;
	end;
	
	function PBIsFolder( const pb: CInfoPBRec ): Boolean;
	begin
		PBIsFolder := pb.ioFlAttrib and ioDirMask <> 0;
	end;
	
	function FSpIsFolder( const spec: FSSpec; var isfolder: Boolean ): OSErr;
		var
			err: OSErr;
			pb: CInfoPBRec;
	begin
		isfolder := false;
		err := FSpGetCatInfo( spec, pb );
		if err = noErr then begin
			isfolder := PBIsFolder( pb );
		end;
		FSpIsFolder := err;
	end;

	function HGetDirID( vrn: integer; parID: longint; const s: string; var dirID: longint ): OSErr;
		var
			err: OSErr;
			pb: CInfoPBRec;
			name: Str63;
	begin
		name := s;
		err := MyGetCatInfo( vrn, parID, name, 0, pb );
		if err = noErr then begin
			if PBIsFolder( pb ) then begin
				dirID := pb.ioDrDirID;
			end else begin
				err := fnfErr;
			end;
		end;
		HGetDirID := err;
	end;

	function FSpGetDirID( const spec: FSSpec; var dirID: longint ): OSErr;
		var
			err: OSErr;
			pb: CInfoPBRec;
	begin
		err := FSpGetCatInfo( spec, pb );
		if err = noErr then begin
			if PBIsFolder( pb ) then begin
				dirID := pb.ioDrDirID;
			end else begin
				err := fnfErr;
			end;
		end;
		FSpGetDirID := err;
	end;

	function FSpGetCatInfo (const fs: FSSpec; var pb: CInfoPBRec): OSErr;
	begin
		MZero( @pb, SizeOf(pb) );
		pb.ioVRefNum := fs.vRefNum;
		pb.ioDirID := fs.parID;
		pb.ioNamePtr := @fs.name;
		pb.ioFDirIndex := 0;
		FSpGetCatInfo := PBGetCatInfoSync(@pb);
	end;

	function FSpGetIndCatInfo (var fs: FSSpec; index: integer; var pb: CInfoPBRec): OSErr;
	begin
		pb.ioVRefNum := fs.vRefNum;
		pb.ioDirID := fs.parID;
		pb.ioNamePtr := @fs.name;
		pb.ioFDirIndex := index;
		FSpGetIndCatInfo := PBGetCatInfoSync(@pb);
	end;

	function FSpSetCatInfo (const spec: FSSpec; var pb: CInfoPBRec): OSErr;
	begin
		pb.ioVRefNum := spec.vRefNum;
		pb.ioDirID := spec.parID;
		pb.ioNamePtr := @spec.name;
		FSpSetCatInfo := PBSetCatInfoSync(@pb);
	end;

	procedure MyGetModDate (const spec: FSSpec; var moddate: UInt32);
		var
			err: OSErr;
			pb: CInfoPBRec;
	begin
		err := FSpGetCatInfo( spec, pb );
		if err = noErr then begin
			moddate := pb.ioFlMdDat
		end else begin
			moddate := $80000000;
		end;
	end;

	function CopyData (src, dst: integer; len: longint): OSErr;
		const
			buffer_len = 4096;
		var
			buffer: array[1..buffer_len] of SignedByte;
			l: longint;
			err: OSErr;
	begin
		err := noErr;
		while (len > 0) & (err = noErr) do begin
			if len > SizeOf(buffer) then begin
				l := SizeOf(buffer);
			end else begin
				l := len;
			end;
			err := FSRead(src, l, @buffer);
			if (l = 0) & (err = noErr) then begin
				err := -17015;
			end;
			if err = noErr then begin
				err := MyFSWrite(dst, l, @buffer);
			end;
			len := len - l;
		end;
		CopyData := err;
	end;

	function FSpExists( const spec: FSSpec ): Boolean;
		var
			pb: CInfoPBRec;
	begin
		FSpExists := FSpGetCatInfo( spec, pb ) = noErr;
	end;

	function FSpCatMoveAndRename( const source: FSSpec; const dest: FSSpec ): OSErr;
		var
			err: OSErr;
			now: UnsignedWide;
			pb: CMovePBRec;
			temp_name: Str255;
			follow_source: FSSpec;
	begin
		if FSpExists( dest ) then begin
			err := dupFNErr;
		end else if source.vRefNum <> dest.vRefNum then begin
			err := nsvErr;
		end else begin
			follow_source := source;
			while true do begin
				pb.ioNamePtr := @follow_source.name;
				pb.ioVRefNum := follow_source.vRefNum;
				pb.ioNewName := nil;
				pb.ioNewDirID := dest.parID;
				pb.ioDirID := follow_source.parID;
				err := PBCatMoveSync( @pb );
				if err = noErr then begin
					follow_source.parID := dest.parID;
				end;
				if err <> dupFNErr then begin
					leave;
				end;
				Microseconds( now );
				NumToString( now.lo, temp_name );
				err := FSpRename( follow_source, temp_name );
				if err = noErr then begin
					follow_source.name := temp_name;
				end;
				if (err <> noErr) & (err <> dupFNErr) then begin
					leave;
				end;
			end;
			if err = noErr then begin
				err := FSpRename( follow_source, dest.name );
			end;
		end;
		FSpCatMoveAndRename := err;
	end;

	function TrashFile( const spec: FSSpec): OSErr;
		var
			err: OSErr;
			dest: FSSpec;
	begin
		err := FindFolder( spec.vRefNum, kTrashFolderType, true, dest.vRefNum, dest.parID );
		if err = noErr then begin
			dest.name := spec.name;
			err := CreateUniqueFile( dest, 'trsh', 'trsh' );
			if err = noErr then begin
				err := FSpDelete( dest );
				err := FSpCatMoveAndRename( spec, dest );
			end;
		end;
		TrashFile := err;
	end;

	procedure FSpZapForks( const spec: FSSpec; zapdf, zaprf: Boolean );
		var
			err: OSErr;
			refnum: integer;
	begin
		if zapdf then begin
			err := FSpOpenDF( spec, fsRdWrPerm, refnum );
			if err = noErr then begin
				err := SetEOF( refnum, 0 );
				err := FSClose( refnum );
			end;
		end;
		if zaprf then begin
			err := FSpOpenRF( spec, fsRdWrPerm, refnum );
			if err = noErr then begin
				err := SetEOF( refnum, 0 );
				err := FSClose( refnum );
			end;
		end;
	end;

	procedure FSpRecreate( const spec: FSSpec; creator: OSType; fileType: OSType; zapdf, zaprf: Boolean );
		var
			err: OSErr;
			fi: FInfo;
	begin
		FSpZapForks( spec, zapdf, zaprf );
		err := FSpCreate( spec, creator, fileType, 0 );
		if err = dupFNErr then begin
			if FSpGetFInfo( spec, fi ) = noErr then begin
				fi.fdCreator := creator;
				fi.fdType := fileType;
				err := FSpSetFInfo( spec, fi );
			end;
		end;
	end;

	procedure FSpRecreateResFile( const spec: FSSpec; creator: OSType; fileType: OSType; zapdf, zaprf: Boolean );
	begin
		FSpZapForks( spec, zapdf, zaprf );
		FSpCreateResFile( spec, creator, fileType, 0 );
	end;

	function DuplicateFile (const org, newfs: FSSpec): OSErr;
		const
			fdInited = $0100;
		var
			oe, junk: OSErr;
			fi: FInfo;
			pb: CInfoPBRec;
			orn, nrn: integer;
			rlen, dlen: longint;
	begin
		oe := FSpGetFInfo(org, fi);
		if oe = noErr then begin
			FSpRecreate(newfs, fi.fdCreator, fi.fdType, kZapDF, kZapRF );
			fi.fdFlags := band(fi.fdFlags, GoodBNOT(fdInited));
			oe := FSpSetFInfo(newfs, fi);
		end;
		if oe = noErr then begin
			oe := FSpGetCatInfo(org, pb);
			if oe = noErr then begin
				dlen := pb.ioFlLgLen;
				rlen := pb.ioFlRLgLen;
				oe := FSpSetCatInfo( newfs, pb);
			end;

			if oe = noErr then begin
				oe := FSpOpenDF(org, fsRdPerm, orn);
				if oe = noErr then begin
					oe := FSpOpenDF(newfs, fsWrPerm, nrn);
					if oe = noErr then begin
						oe := CopyData(orn, nrn, dlen);
						AddOSErr( oe, FSClose(nrn) );
					end;
					junk := FSClose(orn);
				end;
			end;

			if oe = noErr then begin
				oe := FSpOpenRF(org, fsRdPerm, orn);
				if oe = noErr then begin
					oe := FSpOpenRF(newfs, fsWrPerm, nrn);
					if oe = noErr then begin
						oe := CopyData(orn, nrn, rlen);
						AddOSErr( oe, FSClose(nrn) );
					end;
					junk := FSClose(orn);
				end;
			end;

			if oe <> noErr then begin
				junk := FSpDelete(newfs);
			end;
		end;
		DuplicateFile := oe;
	end;

	function MyFSWriteAt (refnum: integer; mode: integer; pos, len: longint; p: Ptr): OSErr;
		var
			pb: ParamBlockRec;
			oe: OSErr;
	begin
		pb.ioRefNum := refnum;
		pb.ioBuffer := p;
		pb.ioReqCount := len;
		pb.ioPosMode := mode;
		pb.ioPosOffset := pos;
		oe := PBWriteSync(@pb);
		if (oe = noErr) & (pb.ioActCount <> len) then begin
			oe := -17016;
		end;
		MyFSWriteAt := oe;
	end;

	const
		maxk = $70000000 div 1024;

	function MultiplyAllocation (blocks, blocksize: longint): longint; { result in k }
		var
			size: longint;
	begin
		blocks := BAND(BSR(blocks, 1), $00007FFF); { div 2 }
		blocksize := BAND(BSR(blocksize, 9), $007FFFFF); { div 512 }
		if (blocksize > 256) & (blocks > 256) then begin
			size := (blocksize div 16) * (blocks div 16);
			if size > maxk div 256 then begin
				size := maxk div 256;
			end;
			size := size * 256;
		end else begin
			size := blocksize * blocks; { in k }
			if size > maxk then begin
				size := maxk;
			end;
		end;
		MultiplyAllocation := size;
	end;

	function OldDiskFreeSpace (vrn: integer): longint; { result in k }
		var
			err: OSErr;
			pb: HParamBlockRec;
			free: longint;
	begin
		free := maxk;
		pb.ioNamePtr := nil;
		pb.ioVRefNum := vrn;
		pb.ioVolIndex := 0;
		err := PBHGetVInfoSync(@pb);
		if err = noErr then begin
			free := MultiplyAllocation(pb.ioVFrBlk, pb.ioVAlBlkSiz);
		end;
		OldDiskFreeSpace := free;
	end;

	function DiskFreeSpace (vrn: integer): longint; { result in k }
		var
			err: OSErr;
			free: longint;
	begin
		err := GetVInfo(vrn, nil, vrn, free);
		if err <> noErr then begin
			free := maxk;
		end else begin
			if free < 0 then begin
				free := maxk;
			end else begin
				free := free div 1024;
				if free > maxk then begin
					free := maxk;
				end;
			end;
		end;
		DiskFreeSpace := free;
	end;

	function DiskSize (vrn: integer): longint; { result in k }
		var
			err: OSErr;
			pb: HParamBlockRec;
			size: longint;
	begin
		size := 0;
		pb.ioNamePtr := nil;
		pb.ioVRefNum := vrn;
		pb.ioVolIndex := 0;
		err := PBHGetVInfoSync(@pb);
		if err = noErr then begin
			size := MultiplyAllocation(pb.ioVNmAlBlks, pb.ioVAlBlkSiz);
		end;
		DiskSize := size;
	end;

	function IsRemoteVolume( vRefNum: integer ): Boolean;
		var
			hpb: HParamBlockRec;
			vol_parms: GetVolParmsInfoBuffer;
	begin
		hpb.ioNamePtr := nil;
		hpb.ioVRefNum := vRefNum;
		hpb.ioBuffer := @vol_parms;
		hpb.ioReqCount := SizeOf(vol_parms);
		IsRemoteVolume := (PBHGetVolParmsSync(@hpb) = noErr)
			& (hpb.ioActCount >= SizeOf(vol_parms))
			& (vol_parms.vMServerAdr <>0);
	end;
	
{$PUSH}
{$ALIGN MAC68K}

	type
		BootBlocksRecord = record           {boot block header}
		  case boolean of 
				true: (
		  		data: packed array[0..1023] of Byte;
		  	)
		  	false: (
					bbID:             Integer; {boot blocks signature}
					bbEntry:          LongInt; {entry point to boot code}
					bbVersion:        Integer; {boot blocks version number}
					bbPageFlags:      Integer; {used internally}
					bbSysName:        Str15;   {System filename}
					bbShellName:      Str15;   {Finder filename}
					bbDbg1Name:       Str15;   {debugger filename}
					bbDbg2Name:       Str15;   {debugger filename}
					bbScreenName:     Str15;   {name of startup screen}
					bbHelloName:      Str15;   {name of startup program}
					bbScrapName:      Str15;   {name of system scrap file}
					bbCntFCBs:        Integer; {number of FCBs to allocate}
					bbCntEvts:        Integer; {number of event queue elements}
					bb128KSHeap:      LongInt; {system heap size on 128K Mac}
					bb256KSHeap:      LongInt; {used internally}
					bbSysHeapSize:    LongInt; {system heap size on all machines}
					filler:           Integer; {reserved}
					bbSysHeapExtra:   LongInt; {additional system heap space}
					bbSysHeapFract:   LongInt; {fraction of RAM for system heap}
        )
			end;
			BootBlocksPtr = ^BootBlocksRecord;
			
{$ALIGN RESET}
{$POP}
        
	function BlessSystemFolderDirectory( vrn: integer; dirID: longint): OSErr;
		var
			err: OSErr;
			pb: HParamBlockRec;
	begin
		MZero( @pb, SizeOf(pb) );
		pb.ioNamePtr := nil;
		pb.ioVRefNum := vrn;
		pb.ioVolIndex := 0;
		err := PBHGetVInfoSync(@pb);
		if err = noErr then begin
			pb.ioVFndrInfo[1] := dirID;  { ARGHHHHHHH! }
			err := PBSetVInfoSync(@pb);
		end;
		BlessSystemFolderDirectory := err;
	end;		

	function BlessSystemFolder (const spec: FSSpec): OSErr;

		function IsCurrentSystem( const spec: FSSpec ): boolean;
			var
				err: OSErr;
				vrn: integer;
				dirID: longint;
		begin
			err := FindFolder( kOnSystemDisk, kSystemFolderType, false, vrn, dirID );
			IsCurrentSystem := (err = noErr) & (spec.vRefNum = vrn) & (spec.parID = dirID);
		end;
	
		function SetupBootFSParamBlock( vrn: integer; var block: BootBlocksRecord; var fpb: ParamBlockRec ): OSErr;
			var
				err: OSErr;
				pb: HParamBlockRec;
		begin
			MZero( @pb, SizeOf(pb) );
			pb.ioNamePtr := nil;
			pb.ioVRefNum := vrn;
			pb.ioVolIndex := 0;
			err := PBHGetVInfoSync(@pb);
			MZero( @fpb, SizeOf(fpb) );
			fpb.ioRefNum := pb.ioVDRefNum;
			fpb.ioVRefNum := pb.ioVDrvInfo;
			fpb.ioBuffer := @block;
			fpb.ioReqCount := SizeOf(block);
			fpb.ioPosMode := fsFromStart;
			fpb.ioPosOffset := 0;
			SetupBootFSParamBlock := err;
		end;
		
		function GetBoot1( const spec: FSSpec; var block: BootBlocksRecord ): OSErr;
			var
				err: OSErr;
				savedresfile: integer;
				boot1: Handle;
		begin
			err := noErr;
			boot1 := nil;
			savedresfile := CurResFile;
			if IsCurrentSystem( spec ) then begin
				boot1 := GetResource( 'boot', 1 );
				if boot1 <> nil then begin
					MDetachResource( boot1 );
				end;
			end else begin
				err := FSpGetResource( spec, 'boot', 1, boot1 );
			end;
			UseResFile( savedresfile );
			if err = noErr then begin
				if boot1 = nil then begin
					GetBoot1 := resNotFound;
				end else if MGetHandleSize( boot1 ) <> SizeOf(block) then begin
					err := -17018;
				end else begin
					BlockMoveData( boot1^, @block, SizeOf(block) );
				end;
			end;
			MDisposeHandle( boot1 );
			GetBoot1 := err;
		end;
		
		var
			err: OSErr;
			fpb: ParamBlockRec;
			block: BootBlocksRecord;
	begin
		err := BlessSystemFolderDirectory( spec.vRefNum, spec.parID );
		if err = noErr then begin
			err := SetupBootFSParamBlock( spec.vRefNum, block, fpb );
		end;
		if err = noErr then begin
			err := PBReadSync( @fpb );
			if (err = noErr) & (fpb.ioReqCount <> fpb.ioActCount) then begin
				err := -2;
			end;
		end;
		if (err = noErr) & (block.bbID = 0) then begin
			err := GetBoot1( spec, block );
			if err = noErr then begin
				err := SetupBootFSParamBlock( spec.vRefNum, block, fpb );
			end;
			if err = noErr then begin
				err := PBWriteSync( @fpb );
				if (err = noErr) & (fpb.ioReqCount <> fpb.ioActCount) then begin
					err := -3;
				end;
			end;
		end;
		if err = noErr then begin
			err := BlessSystemFolderDirectory( spec.vRefNum, spec.parID );
		end;
		BlessSystemFolder := err;
	end;

	function SameFSSpec (const fs1, fs2: FSSpec): Boolean;
	begin
		SameFSSpec := (fs1.vRefNum = fs2.vRefNum) & (fs1.parID = fs2.parID) & (IUEqualString(fs1.name, fs2.name) = 0);
	end;

	function GetDirID (wdrn: integer; var vrn: integer; var dirID: longint): OSErr;
		var
			procID: longint;
			oe: OSErr;
	begin
		oe := GetWDInfo(wdrn, vrn, dirID, procID);
		if oe <> noErr then begin
			vrn := wdrn;
			dirID := 0;
		end;
		GetDirID := oe;
	end;

	function GetVolInfo (var name: Str63; var vrn: integer; index: integer; var CrDate: longint): OSErr;
		var
			pb: ParamBlockRec;
			oe: OSErr;
	begin
		if (name <> '') & (name[length(name)] <> ':') then begin
			name := concat(name, ':');
		end;
		pb.ioNamePtr := @name;
		pb.ioVRefNum := vrn;
		pb.ioVolIndex := index;
		oe := PBGetVInfoSync(@pb);
		if oe = noErr then begin
			vrn := pb.ioVRefNum;
			CrDate := pb.ioVCrDate;
		end;
		GetVolInfo := oe;
	end;

	function GetVolumeAddrBlock(vrn: integer; index: integer; var addr: AddrBlock): OSErr;
		var
			err: OSErr;
			vinfopb: HParamBlockRec;
			vparamspb: HParamBlockRec;
			volparams: GetVolParmsInfoBuffer;
			gv: longint;
			afpRefNum: integer;
			dap: DDPAddressPtr;
	begin
		longint(addr) := 0;
		vinfopb.ioNamePtr := nil;
		vinfopb.ioVRefNum := vrn;
		vinfopb.ioVolIndex := index;
		err := PBHGetVInfoSync(@vinfopb);
		if err = noErr then begin
			vparamspb := vinfopb;
			vparamspb.ioNamePtr := nil;
			vparamspb.ioBuffer := @volparams;
			vparamspb.ioReqCount := SizeOf(volparams);
			err := PBHGetVolParmsSync(@vparamspb);
		end;
		if err = noErr then begin
			if (volparams.vMServerAdr <> 0) & (vinfopb.ioVDrvInfo <> 0) then begin // network volume
				if (OpenDriver( ".AFPTranslator", afpRefNum) = noErr) &  (vinfopb.ioVDRefNum = afpRefNum) then begin // AppleShare volume
					addr := AddrBlock(volparams.vMServerAdr);
					if (Gestalt( 'afps', gv ) = noErr) & (band(gv, $0FFFF) >= 6) then begin // AppleShare Client 3.7
						// Well, volparams.vMServerAdr is probably an OTAddressPtr...
						dap := DDPAddressPtr(volparams.vMServerAdr);
						if band(longint(dap),$01) = 0 then begin // At least it is even, sigh
							if (dap^.fAddressType = AF_ATALK_DDP) | (dap^.fAddressType = AF_ATALK_DDPNBP) | (dap^.fAddressType = AF_ATALK_MNODE) then begin
								// AppleTalk Address
								addr.aNet := dap^.fNetwork;
								addr.aNode := dap^.fNodeID;
								addr.aSocket := dap^.fSocket;
							end;
						end;
					end;
				end;
			end;
		end;
		GetVolumeAddrBlock := err;
	end;

	function ScanDirectory (fs: FSSpec; doit: ScanProc): OSErr;
		var
			pb: CInfoPBRec;
			ret, folder: Boolean;
			path: Str255;
		procedure Scan (dirID: longint);
			var
				index, len: integer;
				oe: OSErr;
		begin
			index := 1;
			repeat
				with pb do begin
					oe := MyGetCatInfo(fs.vRefNum, dirID, fs.name, index, pb);
					index := index + 1;
					if oe = noErr then begin
						fs.parID := dirID;
						folder := PBIsFolder( pb );
						ret := doit(fs, folder, path, pb);
						if folder and ret then begin
							len := length(path);
							path := concat(path, fs.name, ':');
							Scan(pb.ioDirID);
							path[0] := chr(len);
						end else if not folder and ret then begin
							index := index - 1;
						end;
					end;
				end;
			until oe <> noErr;
		end;
		var
			err: OSErr;
			dummy: Boolean;
	begin
		path := ':';
		if fs.name <> '' then begin
			err := FSpGetCatInfo(fs, pb);
			if err = noErr then begin
				if PBIsFolder( pb ) then begin
					Scan(pb.ioDirID);
				end else begin
					dummy := doit(fs, false, path, pb);
				end;
			end;
		end else begin
			Scan(fs.parID);
			err := noErr;
		end;
		ScanDirectory := err;
	end;

	function RemoveResourceFork( const spec: FSSpec ): OSErr;
		var
			err, err2: OSErr;
			refnum: integer;
	begin
		err:=FSpOpenRF( spec, fsRdWrPerm, refnum );
		if err = noErr then begin
			err := SetEOF( refnum, 0 );
			err2 := FSClose( refnum );
			if err = noErr then begin
				err := err2;
			end;
		end;
		RemoveResourceFork := err;
	end;
	
	function FindTemporaryFolder( vrn: integer; var resultvrn: integer; var resultparID: longint ): OSErr;
		var
			err: OSErr;
	begin
		err := FindFolder( vrn, kChewableItemsFolderType, true, resultvrn, resultparID );
		if err <> noErr then begin
			err := FindFolder( vrn, kTemporaryFolderType, true, resultvrn, resultparID );
		end;
(*
		err := FSMakeFSSpec( 0, 0, 'Zany:Temp Files:xxx', info_spec );
		if err = fnfErr then err := noErr;
*)
		FindTemporaryFolder := err;
	end;

	function PseudoMakeFSSpec( pseudopath: Str255; createFolder: Boolean; var spec: FSSpec ): OSErr;
		// pseudopath is four characters FindFolder type followed by file/folder name
		var
			err: OSErr;
			our_folder_type: OSType;
	begin
		StripOSTypeFromString( pseudopath, our_folder_type );
		err := MyFindFolder( our_folder_type, createFolder, spec.vRefNum, spec.parID );
		if err = noErr then begin
			err := FSMakeFSSpec( spec.vRefNum, spec.parID, pseudopath, spec );
		end else if err = fnfErr then begin
			err := dirNFErr;
		end;
		PseudoMakeFSSpec := err;
	end;

	const
		my_find_folder_strh_id = 7700;

	function PseudoMakeFolderSpec( folderType: OSType; createFolder: Boolean; var spec: FSSpec ): OSErr;
		var
			err: OSErr;
			s: Str255;
			our_folder_type: OSType;
			index: integer;
	begin
		index := 1;
		while true do begin
			GetIndString( s, my_find_folder_strh_id, index );
			Inc(index);
			if s = '' then begin
				leave;
			end;
			StripOSTypeFromString( s, our_folder_type );
			if our_folder_type = folderType then begin
				err := PseudoMakeFSSpec( s, createFolder, spec );
				leave;
			end;
		end;
		if s = '' then begin
			err := MyFindFolder( folderType, createFolder, spec.vRefNum, spec.parID );
			if err = noErr then begin
				err := GetDirFSSpec( spec.vRefNum, spec.parID, spec );
			end;
			if err = fnfErr then begin
				err := dirNFErr;
			end;
		end;
		PseudoMakeFolderSpec := err;
	end;
	
	function MyFindFolder( folderType: OSType; createFolder: Boolean; var foundVRN: integer; var foundDirID: longint ): OSErr;
		var
			err: OSErr;
			s: Str255;
			index: integer;
			our_folder_type: OSType;
			spec: FSSpec;
			isfolder, wasalias: Boolean;
	begin
		Assert( has_FindFolder );
		if folderType = kApplicationFolderType then begin
			foundVRN := app_fs.vRefNum;
			foundDirID := app_fs.parID;
			err := noErr;
		end else if folderType = kTemporaryFolderType then begin
			err := FindTemporaryFolder( kOnSystemDisk, foundVRN, foundDirID );
		end else begin
			err := FindFolder( kOnSystemDisk, folderType, createFolder, foundVRN, foundDirID );
		end;
		if err <> noErr then begin
			index := 1;
			while true do begin
				GetIndString( s, my_find_folder_strh_id, index );
				Inc(index);
				if s = '' then begin
					leave;
				end;
				StripOSTypeFromString( s, our_folder_type );
				if our_folder_type = folderType then begin
					err := PseudoMakeFSSpec( s, createFolder, spec );
					if err = fnfErr then begin
						if createFolder then begin
							err := FSpDirCreate( spec, 0, foundDirID );
						end else begin
							err := dirNFErr;
							leave;
						end;
					end else if err = noErr then begin
						err := ResolveAliasFile( spec, true, isfolder, wasalias );
						err := FSpGetDirID( spec, foundDirID );
					end;
					foundVRN := spec.vRefNum;
					leave;
				end;
			end;			
		end;
		if err <> noErr then begin
			foundVRN := -1;
			foundDirID := 2;
		end;
		MyFindFolder := err;
	end;

	procedure AutoRename( strh_id: integer );
		var
			err: OSErr;
			i: integer;
			s, newname: Str255;
			creator, typ: OSType;
			spec, dest: FSSpec;
			fi: FInfo;
	begin
		i := 1;
		while true do begin
			GetIndString( s, strh_id, i );
			Inc(i);
			if s = '' then begin
				leave;
			end;
			StripOSTypeFromString( s, creator );
			StripOSTypeFromString( s, typ );
			SplitBy( s, ':', s, newname );
			if PseudoMakeFSSpec( s, kDontCreateFolder, spec ) = noErr then begin
				if FSpGetFInfo( spec, fi ) = noErr then begin
					if ((creator = '****') | (fi.fdCreator = creator)) & ((typ = '****') | (fi.fdType = typ)) then begin
						if newname <> '' then begin
							err := PseudoMakeFSSpec( newname, kCreateFolder, dest );
							if err = fnfErr then begin
								err := FSpCatMoveAndRename( spec, dest );
							end;
						end else begin
							err := -17019;
						end;
						if err <> noErr then begin
							err := FSpDelete( spec );
						end;
						if err <> noErr then begin
							err := TrashFile( spec );
						end;
					end;
				end;
			end;
		end;
	end;

	function AliasToFullpath( alias: AliasHandle; var path: Str255 ): OSErr;
		var
			err: OSErr;
			part: Str255;
			i: integer;
	begin
		path := '';
		i := asiAliasName;
		err := noErr;
		while true do begin
			err := GetAliasInfo(alias, i, part);
			Inc(i);
			if (err <> noErr) | (part = '') then begin
				leave;
			end;
			path := part + ':' + path;
		end;
		if err = noErr then begin
			err := GetAliasInfo(alias, asiVolumeName, part);
		end;
		if err = noErr then begin
			if (i <> 2) | (part + ':' <> path) then begin // HACK HACK HACK - For volumes, asiVolumeName = asiAliasName!
				path := part + ':' + path;
			end;
		end;
		if Pos( ':', path ) < length(path) then begin // Dont delete trailing : from "volume:"
			DeleteExtension( path, ':' );
		end;
		if err <> noErr then begin
			path := '';
		end;
		AliasToFullpath := err;
	end;

end.
(*

		
		err := GetAliasInfo(alias, asiAliasName, part);
		i := asiParentName;
		path := '';
		while (err = noErr) & (part <> '') do begin
			path := concat(part, ':', path);
			err := GetAliasInfo(alias, i, part);
			i := i + 1;
		end;
		if err = noErr then begin
			Delete(path, length(path), 1);
			err := GetAliasInfo(alias, asiVolumeName, part);
		end;
		if err = noErr then begin
			path :=  concat(part, ':', path);
		end;
	procedure SafeFindFolder (vRefNum: integer; folderType: OSType; var foundVRefNum: integer; var foundDirID: longint);
		var
			theWorld: SysEnvRec;
			gv: longint;
	begin
		foundVRefNum := -1;
		foundDirID := 2;
		if (Gestalt(gestaltFindFolderAttr, gv) <> noErr) | (not BTST(gv, gestaltFindFolderPresent)) | (FindFolder(vRefNum, folderType, true, foundVRefNum, foundDirID) <> noErr) then begin
			if SysEnvirons(1, theWorld) = noErr then begin
				foundVRefNum := theWorld.sysVRefNum;
				foundDirID := 0;
			end else begin
				foundVRefNum := -1;
				foundDirID := 2;
			end;
		end;
	end;

	procedure SafeFindFolder (vRefNum: integer; folderType: OSType; var foundVRefNum: integer; var foundDirID: longint);
	function MyFSMakeFSSpec (vrn: integer; dirID: longint; name: Str255; var fs: FSSpec): OSErr;
		var
			pb: CInfoPBRec;
			oe: OSErr;
			gv: longint;
	begin
		if (Gestalt(gestaltFSAttr, gv) = noErr) & (BTST(gv, gestaltHasFSSpecCalls)) then begin
			oe := FSMakeFSSpec(vrn, dirID, name, fs);
		end else begin
			oe := MyGetCatInfo(vrn, dirID, name, 0, pb);
			if (oe = noErr) then begin
				fs.vRefNum := pb.ioVRefNum;
				fs.parID := pb.ioFlParID;
				fs.name := name;
			end;
		end;
		MyFSMakeFSSpec := oe;
	end;

	function BlessSystemFolder (vrn: integer; dirID: longint): OSErr;
		var
			err: OSErr;
			pb: HParamBlockRec;
	begin
		pb.ioNamePtr := nil;
		pb.ioVRefNum := vrn;
		pb.ioVolIndex := 0;
		err := PBHGetVInfoSync(@pb);
		if err = noErr then begin
			pb.ioVFndrInfo[1] := dirID;  { ARGHHHHHHH! }
			err := PBSetVInfoSync(@pb);
		end;
		BlessSystemFolder := err;
	end;

{$PUSH}
{$ALIGN MAC68K}
	type
		VolParamsRecord = packed record
					version: integer;
					attrib: longint;
					localhand: Handle;
					address: AddrBlock;
				end;
{$ALIGN RESET}
{$POP}

	function GetVolumeAddrBlock(vrn: integer; index: integer; var addr: AddrBlock): OSErr;
		var
			err: OSErr;
			pb: HParamBlockRec;
			volparams: VolParamsRecord;
	begin
		longint(addr) := 0;
		pb.ioNamePtr := nil;
		pb.ioVRefNum := vrn;
		pb.ioVolIndex := index;
		err := PBHGetVInfoSync(@pb);
		if err = noErr then begin
			pb.ioNamePtr := nil;
			pb.ioBuffer := @volparams;
			pb.ioReqCount := SizeOf(volparams);
			err := PBHGetVolParmsSync(@pb);
		end;
		if err = noErr then begin
			addr := volparams.address;
		end;
		GetVolumeAddrBlock := err;
	end;

*)
