unit MyHandles;

interface

	uses
		MacOS;

	type
		HandleArray = ^Ptr;

	type
		HandleArrayBeforeProc = function ( hhhh: HandleArray; refcon: longint; index1, index2: longint ): Boolean;
		
	function HandleArrayCreate (var hhhh: HandleArray; size: longint): OSErr;
	procedure HandleArrayDestroy (var hhhh: HandleArray);
	function HandleArrayCount (hhhh: HandleArray): longint;
	function HandleArrayGetSize (hhhh: HandleArray): longint;
	function HandleArrayAppend (hhhh: HandleArray; data: Ptr): OSErr;
	function HandleArrayInsert (hhhh: HandleArray; index: longint; data: Ptr): OSErr;
	procedure HandleArrayGet (hhhh: HandleArray; index: longint; data: Ptr);
	procedure HandleArraySet (hhhh: HandleArray; index: longint; data: Ptr);
	procedure  HandleArrayDelete(hhhh: HandleArray; index: longint);
	procedure HandleArrayReset (hhhh: HandleArray);
	procedure HandleArraySwap (hhhh: HandleArray; index1, index2: longint);
	procedure HandleArraySort(hhhh: HandleArray; refcon: longint; IsBefore: HandleArrayBeforeProc );

implementation

	uses
		MyLowLevel, MyAssertions, MyTypes, MyMemory;

	function HandleArrayCreate (var hhhh: HandleArray; size: longint): OSErr;
		var
			err: OSErr;
	begin
		err := MNewHandle(hhhh, SizeOf(longint));
		if err = noErr then begin
			longintH(hhhh)^^ := size;
		end;
		HandleArrayCreate := err;
	end;

	procedure HandleArrayDestroy (var hhhh: HandleArray);
	begin
		MDisposeHandle( hhhh );
	end;

	procedure HandleArrayReset (hhhh: HandleArray);
	begin
		Assert(hhhh <> nil);
		MShrinkHandleSize( hhhh, SizeOf(longint) );
	end;

	function HandleArrayCount (hhhh: HandleArray): longint;
	begin
		Assert(hhhh <> nil);
		HandleArrayCount := (MGetHandleSize( hhhh ) - SizeOf(longint)) div longintH(hhhh)^^;
	end;

	function HandleArrayGetSize (hhhh: HandleArray): longint;
	begin
		Assert(hhhh <> nil);
		if MGetHandleSize( hhhh ) > SizeOf(longint) then begin
			HandleArrayGetSize := longintH(hhhh)^^;
		end else begin
			HandleArrayGetSize := -1;
		end;
	end;

	function HandleArrayAppend (hhhh: HandleArray; data: Ptr): OSErr;
	begin
		Assert(hhhh <> nil);
		HandleArrayAppend := MAppendToHandle( hhhh, data, longintH(hhhh)^^);
	end;

	function HandleArrayInsert (hhhh: HandleArray; index: longint; data: Ptr): OSErr;
	begin
		Assert((hhhh <> nil) & (0 < index) & (index <= HandleArrayCount(hhhh) + 1));
		HandleArrayInsert := MMungerInsert( hhhh, SizeOf(longint) + (index - 1) * longintH(hhhh)^^, data, longintH(hhhh)^^);
	end;

	procedure HandleArrayGet (hhhh: HandleArray; index: longint; data: Ptr);
	begin
		Assert((hhhh <> nil) & (0 < index) & (index <= HandleArrayCount(hhhh)));
		BlockMoveData(Ptr(ord(hhhh^) + SizeOf(longint) + (index - 1) * longintH(hhhh)^^), data, longintH(hhhh)^^);
	end;

	procedure HandleArraySet (hhhh: HandleArray; index: longint; data: Ptr);
	begin
		Assert((hhhh <> nil) & (0 < index) & (index <= HandleArrayCount(hhhh)));
		BlockMoveData(data, Ptr(ord(hhhh^) + SizeOf(longint) + (index - 1) * longintH(hhhh)^^), longintH(hhhh)^^);
	end;

	procedure HandleArraySwap (hhhh: HandleArray; index1, index2: longint);
		var
			i, size: longint;
			p1, p2: Ptr;
			temp: SignedByte;
	begin
		Assert((hhhh <> nil) & (0 < index1) & (index2 <= HandleArrayCount(hhhh)) & (0 < index2) & (index2 <= HandleArrayCount(hhhh)));
		if index1 <> index2 then begin
			size := longintH(hhhh)^^;
			p1 := AddPtrLong( hhhh^, SizeOf(longint) + (index1 - 1) * size );
			p2 := AddPtrLong( hhhh^, SizeOf(longint) + (index2 - 1) * size );
			for i := 0 to size - 1 do begin
				temp := p1^;
				p1^ := p2^;
				p2^ := temp;
				OffsetPtr( p1, 1 );
				OffsetPtr( p2, 1 );
			end;
		end;
	end;

	procedure HandleArraySort(hhhh: HandleArray; refcon: longint; IsBefore: HandleArrayBeforeProc );
		var
			i, j, count, smallest : integer;
	begin
		count := HandleArrayCount(hhhh);
		for i := 1 to count-1 do begin
			smallest := i;
			for j := i+1 to count do  begin
				if IsBefore( hhhh, refcon, j, smallest ) then begin
					smallest := j;
				end;
			end;
			HandleArraySwap(hhhh, i, smallest);
		end;

{$ifc do_debug}
		{ Just check the sorting.  Can't be too careful. }
		for i := 2 to HandleArrayCount(hhhh) do begin
			Assert( not IsBefore(hhhh, refcon, i, i-1) );
		end;
{$endc}
	end;

	procedure HandleArrayDelete (hhhh: HandleArray; index: longint);
	begin
		Assert((hhhh <> nil) & (0 < index) & (index <= HandleArrayCount(hhhh)));
		MMungerDelete( hhhh, SizeOf(longint) + (index - 1) * longintH(hhhh)^^, longintH(hhhh)^^);
	end;

end.