unit MyInitialWindowPositions;

interface

	uses
		MacOS;

	const
		kStagger = true;
		kDontStagger = false;
		
	procedure SaveInitialWindowPosition( window:WindowPtr; stagger: Boolean );
	procedure SetInitialWindowPosition( window:WindowPtr );
	procedure SetInitialDialogPosition( window:WindowPtr ); // Call before ShowWindow
	procedure SetNoInitialWindowPosition;

	procedure SaveWindowPosition( save_location: OSType; window:WindowPtr );
	procedure RestoreWindowPosition( save_location: OSType; window:WindowPtr );
	procedure DefaultWindowPosition( save_location: OSType );
	
	procedure SaveWindowRect( save_location: OSType; window:WindowPtr );
	procedure RestoreWindowRect( save_location: OSType; window:WindowPtr );

implementation

	uses
		MyWindows, MyAssertions, MyNewPreferences;

	const
		stagger_offset: Point = (v:20,h:20);
//		stagger_offset: Point = Point($00140014);
		
	var
		next_window_position:Point;
	
	procedure SaveWindowPosition( save_location: OSType; window:WindowPtr );
		var
			loc:Point;
	begin
		Assert( window <> nil );
		if (save_location<>OSType(0)) & (window<>nil) then begin
			GetWindowPosition( window, loc );
			prefs.SetTagLong( save_location, loc );
		end;
	end;
	
	procedure RestoreWindowPosition( save_location: OSType; window:WindowPtr );
	begin
		Assert( window <> nil );
		if window<>nil then begin
			if save_location<>OSType(0) then begin
				prefs.GetTagLong( save_location, next_window_position );
			end;
			SetInitialWindowPosition( window );
		end;
	end;
	
	procedure DefaultWindowPosition( save_location: OSType );
	begin
		if not prefs.ExistsTag(save_location) then begin
			prefs.SetTagLong( save_location, 0 );
		end;
	end;

	procedure SetNoInitialWindowPosition;
	begin
		next_window_position.h:=0;
		next_window_position.v:=0;
	end;
	
	procedure SaveInitialWindowPosition( window:WindowPtr; stagger: Boolean );
	begin
		GetWindowPosition( window, next_window_position );
		if stagger then begin
			AddPt( stagger_offset, next_window_position );
		end;
	end;
	
	procedure SetInitialWindowPosition(window:WindowPtr);
	begin
		if (next_window_position.h<>0) | (next_window_position.v<>0) then begin
			SetWindowPosition(window,next_window_position);
			NudgeOntoScreen( window, document_titlebar_height );
			next_window_position.h:=0;
			next_window_position.v:=0;
		end else begin
			StaggerWindow( window );
		end;
		if not TitleBarOnScreen( window ) then begin
			MoveWindow( window, 40, 40, false );
		end;
	end;
	
	procedure SetInitialDialogPosition( window:WindowPtr ); // Call before ShowWindow
		var
			fw: WindowPtr;
			pos: Point;
	begin
		fw := FrontWindow;
		if fw <> nil then begin
			GetWindowPosition( fw, pos );
			AddPt( stagger_offset, pos );
			SetWindowPosition( window, pos );
			NudgeOntoScreen( window, document_titlebar_height );
		end;
	end;
	
	procedure SaveWindowRect( save_location: OSType; window:WindowPtr );
		var
			frame: Rect;
	begin
		GetWindowRect( window, frame );
		prefs.SetTagData( save_location, @frame, SizeOf(frame) );
	end;
	
	procedure RestoreWindowRect( save_location: OSType; window:WindowPtr );
		var
			frame: Rect;
			len: longint;
	begin
		if prefs.InfoTag( save_location, len ) & (len = SizeOf(frame)) then begin
			prefs.GetTagData( save_location, @frame, SizeOf(frame) );
			SetWindowRect( window, frame );
		end else begin
			StaggerWindow( window );
		end;
		if not TitleBarOnScreen( window ) then begin
			MoveWindow( window, 40, 40, false );
		end;
	end;
	
end.
