unit MyInternetConfig;

interface

	uses
		InternetConfig;

	var
		has_IC: Boolean;
		internet_config_instance: ICInstance;

	procedure StartupInternetConfig;
	procedure ConfigureInternetConfig(creator: OSType; required: Boolean);
	
	procedure DestroyCachedMappings;
	
	function GetFilenameInfo (name: Str255; var entry: ICMapEntry; istext: Boolean): Boolean;
{ NOTE: entry will always be valid as a "best guess"}
	function GetCreatorTypeInfo (fcreator, ftype: OSType; name: Str255; var entry: ICMapEntry): Boolean;
{ NOTE: entry will always be valid as a "best guess"}
	function GetMimeInfo( const mime, name: Str255; var entry: ICMapEntry ): Boolean;
{ NOTE: entry will always be valid as a "best guess"}
	function GetTextCreator: OSType; { DONT USE FOR EDITOR HELPER! }
	procedure GetBinaryCreator;
	function GetTextHelper: OSType;
	function GetWebHelper: OSType;
	function GetHelper (name: Str255; var app: ICAppSpec): OSErr;
	procedure GetICDownloadFolder (var vrn: integer; var dirID: longint);

	function SameMimeType( mime1, mime2: Str255 ): Boolean;

	function GetICString (const key: Str255; var data: Str255): OSErr;
	function GetICStr (const key: Str255): Str255;
	function SetICString (const key: Str255; data: Str255): OSErr;
	function GetICBoolean( const key: Str255; var result: Boolean ): OSErr;
	function GetICBool( const key: Str255 ): Boolean;
	function SetICBoolean( const key: Str255; result: Boolean ): OSErr;
	function MyICBegin (perm: ICPerm): OSErr;
	function MyICEnd: OSErr;
	function LaunchInternetConfig:OSErr;
	function CheckICUsageVersion(component_version:longint): OSErr;
	function MyLaunchURL(hint,url:Str255): OSErr;
	function SafeICGetPrefHandle (inst: ICInstance; const key: Str255; var attr: ICAttr; var prefh: Handle): ICError;
	procedure GetFontSize (key1,key2:Str255; deffont,defsize:integer; var font, size: integer);
	procedure GetScreenFontSize (var font, size: integer);
	procedure GetListFontSize (var font, size: integer);
	procedure ScrambleICPassword( var pass: string );

	function MyGetICMapHandle (var hhhh: Handle): Boolean;

	function GetICSeed: longint;

implementation

	uses
		MyMemory, MySystemGlobals, MyUtils, MyStrings, MyProcesses, MyStartup, MyFileSystemUtils;

	const
		IC_first_entry_pos = 0;

{$ifc do_debug}
	var
		startup_check: integer;
{$endc}

	var
		gMapHandle: Handle;
		ic_seed: longint;
		text_creator: OSType;
		binary_creator: OSType;
		binary_type: OSType;
		binary_creator_app_name:Str63;
		last_checked:longint;
		g_creator: OSType;
		g_required: Boolean;

	function SafeICGetPrefHandle (inst: ICInstance; const key: Str255; var attr: ICAttr; var prefh: Handle): ICError;
		var
			err, junk:ICError;
			size: longint;
			did_begin: Boolean;
	begin
		AssertDidStartup( startup_check );
		if not has_IC then begin
			err := unimpErr;
			prefh := nil;
		end else begin
			err := MNewHandle( prefh, 0 );
			if err = noErr then begin
				err := ICFindPrefHandle(inst, key, attr, prefh);
			end;
			if err <> noErr then begin
				MDisposeHandle( prefh );
				did_begin := ICBegin(inst, icReadOnlyPerm) = noErr;
				err := ICGetPref(inst, key, attr, nil, size);
				if err = noErr then begin
					err := MNewHandle( prefh, size );
					if err = noErr then begin
						MHLock(prefh);
						err := ICGetPref(inst, key, attr, prefh^, size);
						MHUnlock(prefh);
					end;
				end;
				if did_begin then begin
					junk := ICEnd(inst);
				end;
			end;
			if err <> noErr then begin
				MDisposeHandle( prefh );
			end;
		end;
		SafeICGetPrefHandle := err;
	end;

	function ICMapErr (icerr: ICError): OSErr;
	begin
		AssertDidStartup( startup_check );
		if (icerr < -32768) or (icerr > 32767) then begin
			icerr := icInternalErr;
		end; (* if *)
		ICMapErr := icerr;
	end; (* ICMapErr *)

	function MyLaunchURL(hint,url:Str255): OSErr;
		var
			start,fin:longint;
	begin
		AssertDidStartup( startup_check );
		MyLaunchURL := icPrefNotFoundErr;
		if has_IC then begin
			start := 0;
			fin := length(url);
			MyLaunchURL := ICMapErr(ICLaunchURL (internet_config_instance,hint, @url[1],length(url), start,fin));
		end;
	end;

	function CheckICUsageVersion(component_version:longint): OSErr;
		var
			err: OSErr;
			component_instance: ComponentInstance;
	begin
		if not has_IC then begin
			err := unimpErr;
		end else begin
			err := ICMapErr(ICGetComponentInstance(internet_config_instance, component_instance));
			if err = noErr then begin
				if BAND(GetComponentVersion(component_instance), $FFFF0000) < BAND(component_version, $FFFF0000) then begin
					err := unimpErr;
				end;
			end else begin
				err := noErr; { we work fine without a component, we just can't deal with an old component }
			end;
		end;
		CheckICUsageVersion := err;
	end;

	function LaunchInternetConfig:OSErr;
		var
			err: OSErr;
	begin
		if not has_IC then begin
			err := unimpErr;
		end else begin
			err := ICMapErr(ICEditPreferences (internet_config_instance,''));
		end;
		LaunchInternetConfig:= err;
	end;

	procedure DestroyCachedMappings;
	begin
		AssertDidStartup( startup_check );
		MDisposeHandle( gMapHandle );
	end;
	
	function MyGetICMapHandle (var hhhh: Handle): Boolean;
		var
			seed: longint;
			junk: ICError;
			junk_attr: longint;
	begin
		AssertDidStartup( startup_check );
		if has_IC then begin
			if (gMapHandle <> nil) then begin
				if last_checked<TickCount-120 then begin
					if (ICGetSeed(internet_config_instance, seed) <> noErr) | (seed <> ic_seed) then begin
						DestroyCachedMappings;
					end else begin
						last_checked:=TickCount;
					end;
				end;
			end;

			if (gMapHandle = nil) then begin
				junk := SafeICGetPrefHandle( internet_config_instance, kICMapping, junk_attr, gMapHandle);
				if (gMapHandle <> nil) & (ICGetSeed(internet_config_instance, seed) = noErr) then begin
					text_creator := OSType(0);
					binary_creator := OSType(0);
					ic_seed := seed;
				end;
			end;
		end;
		hhhh := gMapHandle;
		MyGetICMapHandle := hhhh <> nil;
	end;

	function GetHelper (name: Str255; var app: ICAppSpec): OSErr;
		var
			err: OSErr;
			attr: ICAttr;
			prefsize: longint;
	begin
		if has_IC then begin
			prefsize := SizeOf(ICAppSpec);
			err := ICMapErr(ICGetPref(internet_config_instance, concat(kICHelper, name), attr, @app, prefsize));
			if (err = noErr) & ((prefsize < 5) | (prefsize < 5 + length(app.name))) then begin
				err := badExtResource;
			end;
		end else begin
			err := unimpErr;
		end;
		GetHelper := err;
	end;

	function GetTextHelper: OSType;
		var
			app:ICAppSpec;
	begin
		if GetHelper('editor',app) = noErr then begin
			GetTextHelper := app.fCreator;
		end else begin
			GetTextHelper := 'ttxt';
		end;
	end;

	function GetWebHelper: OSType;
		var
			app:ICAppSpec;
	begin
		if GetHelper( 'http',app ) = noErr then begin
			GetWebHelper := app.fCreator;
		end else begin
			GetWebHelper := 'MSIE';
		end;
	end;

	function GetTextCreator: OSType;
		var
			dummy: Boolean;
			entry: ICMapEntry;
			map:Handle;
	begin
		dummy:= MyGetICMapHandle (map); { reset text_creator if the Handle has changed }
		if text_creator = OSType(0) then begin
			dummy := GetCreatorTypeInfo( 'ttxt', 'TEXT', 'file.txt', entry );
			text_creator := entry.file_creator;
		end;
		GetTextCreator := text_creator;
	end;

	procedure GetBinaryCreator; { WARNING: GetBinaryCreator calls GetFilenameInfo calls GetBinaryCreator }
		var
			dummy: Boolean;
			entry: ICMapEntry;
			map:Handle;
	begin
		dummy:= MyGetICMapHandle (map); { reset binary_creator if the Handle has changed }
		if binary_creator = OSType(0) then begin
			binary_creator := 'hDmp'; { WARNING: we must set binary_creator to avoid unending recursion! }
			binary_type := 'BINA';
			binary_creator_app_name := 'HexEdit';
			if GetFilenameInfo('file.binary', entry, false) then begin
				binary_creator := entry.file_creator;
				binary_type := entry.file_type;
				binary_creator_app_name := entry.creator_app_name;
			end;
		end;
	end;

	function GetCreatorTypeInfo (fcreator, ftype: OSType; name: Str255; var entry: ICMapEntry): Boolean;
		var
			found:Boolean;
			entries: Handle;
//			temp_name: Str255;
	begin
//		SplitBy( name, '#', name, temp_name );
		found := false;
		if MyGetICMapHandle(entries) then begin
			found := ICMapEntriesTypeCreator(internet_config_instance, entries, ftype, fcreator, name, entry)= noErr;
		end;
		if not found then begin
			entry.file_type := ftype;
			entry.file_creator := fcreator;
			entry.creator_app_name := OSTypeToString(fcreator);
			if ftype = 'TEXT' then begin
				entry.flags := ICmap_data_fork_mask;
				entry.MIME_type := 'text/plain';
				entry.entry_name := 'Text File';
			end else begin
				entry.flags := ICmap_binary_mask + ICmap_data_fork_mask;
				entry.MIME_type := 'application/octet-stream';
				entry.entry_name := 'Binary File';
			end;
			entry.post_creator := OSType(0);
			entry.extension := '';
			entry.post_app_name := '';
		end;
		GetCreatorTypeInfo := found;
	end;

	function GetFilenameInfo (name: Str255; var entry: ICMapEntry; istext: Boolean): Boolean;
		var
			found: Boolean;
			entries: Handle;
	begin
		Assert( name <> '' );
		DeleteExtension( name, '~' );
		if name = '' then begin
			if istext then begin
				name := 'file.txt';
			end else begin
				name := 'file.binary';
			end;
		end;
		found := false;
		if MyGetICMapHandle(entries) then begin
			found:= ICMapEntriesFilename(internet_config_instance, entries, name, entry) = noErr;
		end;
		if not found then begin
			if IsExtension(name, '.txt') or istext then begin
				entry.file_type := 'TEXT';
				entry.file_creator := GetTextCreator;
				entry.flags := ICmap_data_fork_mask;
				entry.creator_app_name := 'SimpleText';
				entry.MIME_type := 'text/plain';
				entry.entry_name := 'Text File';
			end else begin
				GetBinaryCreator;
				entry.file_type := binary_type;
				entry.file_creator := binary_creator;
				entry.creator_app_name := binary_creator_app_name;
				entry.flags := ICmap_binary_mask + ICmap_data_fork_mask;
				entry.MIME_type := 'application/octet-stream';
				entry.entry_name := 'Binary File';
			end;
			entry.post_creator := OSType(0);
			entry.extension := '';
			entry.post_app_name := '';
		end;
		GetFilenameInfo := found;
	end;
	
	function SameMimeType( mime1, mime2: Str255 ): Boolean;
		procedure CleanMimeType( var mime: Str255 );
			var
				p: integer;
		begin
			LowerCaseString( mime1 );
			p := Pos( '/x-', mime );
			if p > 0 then begin
				Delete( mime, p+1, 2 );
			end;
		end;
	begin
		CleanMimeType( mime1 );
		CleanMimeType( mime2 );
		SameMimeType := mime1 = mime2;
	end;
	
	function LookupMimeInfo( const mime, name: Str255; var entry: ICMapEntry ): Boolean;
		var
			err: OSErr;
			best: integer;
			bestentry: ICMapEntry;
			entries: Handle;
			i, count, pos: longint;
	begin
		best := -1;
		if (mime <> '') & MyGetICMapHandle(entries) then begin
			err := ICCountMapEntries( internet_config_instance, entries, count );
			if err = noErr then begin
				pos := 0;
				for i := 1 to count do begin
					err := ICGetMapEntry( internet_config_instance, entries, pos, entry );
					if err <> noErr then begin
						leave;
					end;
					if SameMimeType( mime, entry.MIME_type ) & not btst(entry.flags, ICmap_not_incoming_bit) then begin
						if (length(entry.extension) > best) & IsExtension( name, entry.extension ) then begin
							best := length(entry.extension);
							bestentry := entry;
						end else if best < 0 then begin
							best := 0;
							bestentry := entry;
						end;
					end;
					pos := pos + entry.total_length;
				end;
			end;
		end;
		entry := bestentry;
		LookupMimeInfo := best >= 0;
	end;
	
	function GetMimeInfo( const mime, name: Str255; var entry: ICMapEntry ): Boolean;
		var
			dummy: Boolean;
			found: Boolean;
	begin
		found := LookupMimeInfo( mime, name, entry );
		if not found then begin
			dummy := GetFilenameInfo( name, entry, true );
			if not dummy then begin
				if IsPrefix( 'text/', mime ) then begin
					dummy := GetFilenameInfo( 'blah.txt', entry, true );
				end else begin
					dummy := GetFilenameInfo( 'blah.binary', entry, false );
				end;
			end;
			entry.MIME_type := mime;
		end;
		GetMimeInfo := found;
	end;

	function GetICString (const key: Str255; var data: Str255): OSErr;
		var
			err:OSErr;
			size: longint;
			junk_attr: ICAttr;
	begin
		AssertDidStartup( startup_check );
		if has_IC then begin
			size := 256;
			err := ICMapErr(ICGetPref(internet_config_instance, key, junk_attr, @data, size));
		end else begin
			err := unimpErr;
		end;
		if err <> noErr then begin
			data:='';
		end;
		GetICString:=err;
	end;

	function GetICStr (const key: Str255): Str255;
		var
			data: Str255;
			junk: OSErr;
	begin
		junk := GetICString(key, data);
		GetICStr := data;
	end;

	function SetICString (const key: Str255; data: Str255): OSErr;
	begin
		if has_IC then begin
			SetICString := ICMapErr(ICSetPref(internet_config_instance, key, ICattr_no_change, @data, length(data) + 1));
		end else begin
			SetICString := unimpErr;
		end;
	end;

	function SetICBoolean( const key: Str255; data: Boolean ): OSErr;
		var
			n: UInt16;
	begin
		if has_IC then begin
			if data then begin
				n := $0101;
			end else begin
				n := 0;
			end;
			SetICBoolean := ICMapErr(ICSetPref(internet_config_instance, key, ICattr_no_change, @n, 1));
		end else begin
			SetICBoolean := unimpErr;
		end;
	end;

	function GetICBoolean( const key: Str255; var result: Boolean ): OSErr;
		var
			data: integer;
			err: OSErr;
			junk_attr: ICAttr;
			size: longint;
	begin
		AssertDidStartup( startup_check );
		err := unimpErr;
		if has_IC then begin
			data := 0;
			size := 1;
			err := ICMapErr( ICGetPref (internet_config_instance, key, junk_attr, @data, size ) );
			if (err = noErr) & (size <> 1) then begin
				err := -17020;
			end;
		end;
		if err = noErr then begin
			result := data <> 0;
		end else begin
			result := false;
		end;
		GetICBoolean := err;
	end;

	function GetICBool( const key: Str255 ): Boolean;
		var
			junk: OSErr;
			result: Boolean;
	begin
		junk := GetICBoolean( key, result );
		GetICBool := result;
	end;

	procedure GetFontSize (key1,key2:Str255; deffont,defsize:integer; var font, size: integer);
		var
			junk_attr: ICAttr;
			fr: ICFontRecord;
			frsize: longint;
			err:ICError;
	begin
		AssertDidStartup( startup_check );
		if has_IC then begin
			frsize := SizeOf(fr);
			err:=ICGetPref(internet_config_instance, key1, junk_attr, @fr, frsize);
			if err<>noErr then begin
				frsize := SizeOf(fr);
				err:=ICGetPref(internet_config_instance, key2, junk_attr, @fr, frsize);
			end;
		end else begin
			err := unimpErr;
		end;
		if err=noErr then begin
			GetFNum(fr.font, font);
			size := fr.size;
		end else begin
			font := deffont;
			size := defsize;
		end;
	end;

	procedure GetScreenFontSize (var font, size: integer);
	begin
		GetMyFonts(MFT_Monaco9, font, size);
		GetFontSize(kICScreenFont, kICListFont, font, size, font, size);
	end;

	procedure GetListFontSize (var font, size: integer);
	begin
		GetMyFonts(MFT_Geneva9, font, size);
		GetFontSize(kICListFont,kICScreenFont, font, size, font, size);
	end;

	function MyICBegin (perm: ICPerm): OSErr;
	begin
		Assert( has_IC );
		MyICBegin := ICMapErr(ICBegin(internet_config_instance, perm));
	end;

	function MyICEnd: OSErr;
	begin
		Assert( has_IC );
		MyICEnd := ICEnd(internet_config_instance);
	end;

	procedure GetICDownloadFolder (var vrn: integer; var dirID: longint);
		var
			alias: AliasHandle;
			fs: FSSpec;
			waschanged: Boolean;
			err, junk: OSErr;
			pb: CInfoPBRec;
			junk_attr: ICAttr;
			prefh: Handle;
			icfs: ICFileSpec;
			junk_long: longint;
	begin
		junk := MyFindFolder( kDesktopFolderType, kDontCreateFolder, vrn, dirID);
		alias := nil;
		err := SafeICGetPrefHandle(internet_config_instance, kICDownloadFolder, junk_attr, prefh);
		if err = noErr then begin
			if MGetHandleSize(prefh) >= SizeOf(ICFileSpec) then begin
				BlockMoveData(prefh^, @icfs, SizeOf(icfs));
				if icfs.alias.aliasSize > 0 then begin
					junk_long := Munger(prefh, 0, nil, SizeOf(ICFileSpec) - SizeOf(AliasRecord), @junk_long, 0);
					err := ResolveAlias(nil, AliasHandle(prefh), fs, waschanged);
				end;
				if err <> noErr then begin
					err := FSMakeFSSpec(0, 0, concat(icfs.vol_name, ':'), fs);
					if err = noErr then begin
						err := FSMakeFSSpec(fs.vRefNum, icfs.fss.parID, icfs.fss.name, fs);
					end;
				end;
				if err = noErr then begin
					err := FSpGetCatInfo(fs, pb);
					if (err = noErr) & not PBIsFolder( pb ) then begin
						err := -2;
					end;
					if err = noErr then begin
						vrn := pb.ioVRefNum;
						dirID := pb.ioDirID;
					end;
				end;
			end;
			MDisposeHandle( prefh );
		end;
	end;
	
	function GetICSeed: longint;
		var
			seed: longint;
	begin
		if not has_IC | (ICGetSeed(internet_config_instance, seed) <> noErr) then begin
			seed := 0;
		end;
		GetICSeed := seed;
	end;

	function InitMyInternetConfig(var msg: integer): OSStatus;
		var
			junk, err: ICError;
			folders: ICDirSpec;
	begin
{$unused(msg)}
		DidStartup( startup_check );
		last_checked := TickCount;
		gMapHandle := nil;
		ic_seed := 0;
		text_creator := OSType(0);
		binary_creator := OSType(0);
		err := noErr;
{$ifc GENERATINGPOWERPC}
		if (longint(@ICStart) = kUnresolvedCFragSymbolAddress) then begin
			err := unimpErr;
		end;
{$endc}
		if err = noErr then begin
			err := ICStart( internet_config_instance, g_creator );
		end;
		has_IC := err = noErr;
		if has_IC then begin
			folders.vRefNum := app_fs.vRefNum;
			folders.dirID := app_fs.parID;
			junk := ICFindConfigFile(internet_config_instance, 1, @folders);
		end;
		if not g_required then begin
			err := noErr;
		end;
		InitMyInternetConfig := err;
	end;

	procedure FinishMyInternetConfig;
		var
			junk: ICError;
	begin
		MDisposeHandle(gMapHandle);
		if has_IC then begin
			junk := ICStop(internet_config_instance);
		end;
	end;

	procedure ConfigureInternetConfig(creator: OSType; required: Boolean);
	begin
		DidStartup( startup_check );
		StartupInternetConfig;
		g_creator := creator;
		g_required := required;
	end;

	procedure StartupInternetConfig;
	begin
		SetStartup(InitMyInternetConfig, nil, 0, FinishMyInternetConfig);
	end;

{$D-}

	procedure ScrambleICPassword( var pass: string );
		var
			i: integer;
	begin
		for i := 1 to length(pass) do begin
			pass[i] := chr(ord(pass[i]) xor ($55 + i));
		end;
	end;
	
end.
			if font = 0 then begin
				font := deffont;
			end;
			if size = 0 then begin
				size := defsize;
			end;
