unit MyKeychain;

interface

	uses
		MacOS;

	function HasKeychainManager: Boolean;
	function MKCIsKeychainUnlocked: Boolean;
	function MKCLockKeychain: OSStatus;
	function MKCUnlockKeychain: OSStatus;
	function MKCSetKeychainLockStatus( locked: Boolean ): OSStatus;
	function MKCGetKeychainName( var s: Str255 ): OSStatus;
	function MKCAddInternetPassword( protocol: OSType; standard_port: UInt16; const host, user: Str255; port: UInt16; const password: Str255 ): OSStatus;
	function MKCFindInternetPassword( protocol: OSType; standard_port: UInt16; const host, user: Str255; port: UInt16; var password: Str255 ): OSStatus;
	
implementation

	uses
		Keychain, 
		MyServices, MyUtils;

	function HasKeychainManager: Boolean;
	begin
{$ifc GENERATINGCFM}
		HasKeychainManager := longint(@KCGetKeychainManagerVersion) <> kUnresolvedCFragSymbolAddress;
{$elsec}
// KeychainManagerAvailable crashes is _CodeFragmentDispatch is not implemented
//		HasKeychainManager := MyTrapAvailable( _CodeFragmentDispatch ) & KeychainManagerAvailable;
		HasKeychainManager := KeychainManagerAvailable;
{$endc}
	end;
	
	function MKCAddInternetPassword( protocol: OSType; standard_port: UInt16; const host, user: Str255; port: UInt16; const password: Str255 ): OSStatus;
		var
			err: OSStatus;
			acthost: Str255;
	begin
		if HasKeychainManager then begin
			if port = 0 then begin
				port := standard_port;
			end;
			SetHostPort( host, port, acthost, port );
			err := KCAddInternetPassword( @acthost, nil, @user, port, 
				protocol, kKCAuthTypeDefault, 
				length(password), @password[1], nil );
		end else begin
			err := errKCNoActiveKeychain;
		end;
		MKCAddInternetPassword := err;
	end;
	
	function MKCFindInternetPassword( protocol: OSType; standard_port: UInt16; const host, user: Str255; port: UInt16; var password: Str255 ): OSStatus;
		var
			err: OSStatus;
			actualLength: UInt32;
			acthost: Str255;
	begin
		if HasKeychainManager then begin
			if port = 0 then begin
				port := standard_port;
			end;
			SetHostPort( host, port, acthost, port );
			err := KCFindInternetPassword( @acthost, nil, @user, port, 
				protocol, kKCAuthTypeDefault, 
				255, @password[1], actualLength, nil );
			if err = noErr then begin
				password[0] := chr(actualLength);
			end;
		end else begin
			err := errKCNoActiveKeychain;
		end;
		if err <> noErr then begin
			password := '';
		end;
		MKCFindInternetPassword := err;
	end;

	function MKCIsKeychainUnlocked: Boolean;
		var
			result: Boolean;
			err: OSErr;
			keychainStatus: UInt32;
	begin
		result := false;
		if HasKeychainManager then begin
			err := KCGetStatus( nil, keychainStatus );
			result := (err = noErr) & btst(keychainStatus, kLockStateKCStatus);
		end;
		MKCIsKeychainUnlocked := result;
	end;
	
	function MKCLockKeychain: OSStatus;
		var
			err: OSErr;
	begin
		err := errKCNoActiveKeychain;
		if HasKeychainManager then begin
			err := KCLockKeychain( nil );
		end;
		MKCLockKeychain := err;
	end;
	
	function MKCUnlockKeychain: OSStatus;
		var
			err: OSErr;
	begin
		err := errKCNoActiveKeychain;
		if HasKeychainManager then begin
			err := KCUnlockKeychain( nil, nil, nil );
		end;
		MKCUnlockKeychain := err;
	end;
	
	function MKCSetKeychainLockStatus( locked: Boolean ): OSStatus;
		var
			err: OSErr;
	begin
		if locked then begin
			err := MKCLockKeychain;
		end else begin
			err := MKCUnlockKeychain;
		end;
		MKCSetKeychainLockStatus := err;
	end;
	
	function MKCGetKeychainName( var s: Str255 ): OSStatus;
		var
			err: OSStatus;
			keychainspec: KCSpec;
	begin
		s := 'Unknown';
		err := KCGetActiveKeychain( keychainspec );
		if err = noErr then begin
			s := keychainspec.kcFileSpec.name;
		end;
		MKCGetKeychainName := err;
	end;
	
end.
// It would be nice if they had a Pascal interface...
	const
		protocolFTP = 'ftp ';
		protocolFTPAccount = 'ftpa';
		
	const
		authTypeFTP = 'ftpa';
		errKCNoActiveKeychain = -25295;
		kLockState = 1;
		
	type
		KCItemRef = Ptr;
		KCItemRefPtr = ^KCItemRef;
		KCSpec = Ptr;
		KCSpecPtr = ^KCSpec;
		
	function KCGetKeychainManagerVersion (var returnVers: UInt32): OSStatus; external;
{$ifc not GENERATINGCFM}
	function KeychainManagerAvailable: Boolean; external;
{$endc}
	function KCAddInternetPassword( const serverName: Str255; securityDomain: StringPtr;
						const accountName: Str255; port: UInt16; protocol: OSType; 
						authType: OSType; passwordLength: UInt32; passwordData: Ptr;
						itemRef: KCItemRefPtr
						): OSStatus; external;
	function KCFindInternetPassword( serverName, securityDomain, accountName: StringPtr;
						port: UInt16; protocol: OSType; authType: OSType; 
						maxLength: UInt32; passwordData: Ptr; var actualLength: UInt32; 
						itemRef: KCItemRefPtr
						): OSStatus; external;
	function KCGetStatus( keychainSpec: KCSpecPtr; var keychainStatus: UInt32 ): OSStatus; external;
	function KCLockKeychain( keychainSpec: KCSpecPtr ): OSStatus; external;
	function KCUnlockKeychain( keychainSpec: KCSpecPtr ): OSStatus; external;

