unit MyListWindow;

interface

	uses
		MyOOMainLoop;
	
	type
		ListWindowObject = object(WObject)
				list: ListHandle;
				columns: integer;
				vcontrol, hcontrol: ControlHandle;
				rootcontrol: ControlHandle;
				windowfi: FontInfo;
				windowfiheight: integer;
				x_list_offset, list_width: integer;
				list_is_active: boolean;
				window_is_active: boolean;
//				max_display_width: integer;
				
				line_height: integer;
				header_info_height, header_buttons_height: integer;
				
				typed_chars: Str31;
				typed_time: longint;
				never_drag: Boolean;
				use_natural_order: boolean;
				procedure CreateList (font, size: integer; ncolumns: integer; ldefID: integer; hscroll: Boolean);
				procedure SetHeights;

				procedure SetListOffset( listoff: integer );
				function GetListOffset: integer;

				procedure MatchVScrollToListPosition;
				procedure SetListPosition( listpos: longint );
				procedure OffsetListPosition( listposoff: longint );

				procedure GetWindowRects( var header_info, header_buttons, list_frame: Rect );
				procedure Destroy;
				override;
				procedure DoContent ( const er: EventRecord );
				override;
				procedure Resize;
				override;
				procedure DrawGrow;
				override;
				procedure ListActivateDeactivate (activate: Boolean);
				procedure DoActivateDeactivate (activate: Boolean);
				override;
				procedure DoListTab( const er:EventRecord; backwards: Boolean );
				procedure DoTab( const er:EventRecord; backwards: Boolean );
				procedure DoKey (const er:EventRecord; ch: char);
				override;
				function CompareStrings( const s1, s2: string ): integer;
				function Match (c: Cell; const what: Str255): Boolean;
				procedure Find (what: Str255; fromstart, allatonce, backwards: Boolean);
				procedure AdjustHControl;
				procedure AdjustVControl;
				procedure SetListWidth (max: integer);
				procedure UpdateZoomHeight;
				function DontDrag (const er: EventRecord): Boolean;
				function DoLClick (const er: EventRecord): Boolean;
				procedure DoDoubleClick;
				procedure DoDoubleClickCell (c: Cell);
				function GetEntryName (c: Cell): Str255;
				function GetUniqueEntryName (c: Cell): Str255;
				procedure OpenParent;
				procedure DrawHeaderInfo( const lineframe: Rect );
				procedure DrawHeaderButtons( const lineframe: Rect );
				procedure DrawBackground( const frame: Rect );
				procedure DoHeaderInfoClick (const lineframe: Rect; local: Point; const er: EventRecord );
				procedure DoHeaderButtonsClick (const lineframe: Rect; local: Point; const er: EventRecord );
				procedure TrackVControl( local: Point );
				procedure TrackHControl( local: Point );
				procedure DoListClick(const lineframe: Rect; local: Point; const er: EventRecord );
				procedure SetSingleSelection (v: integer);
				function SelectFirstAfter (s: Str255): Boolean;
				function SelectFirstBefore (s: Str255): Boolean;
				function GetFirstSelection (var c: Cell): Boolean;
				function GetLastSelection (var c: Cell): Boolean;
				function CountSelections: integer;
				function IsSelection: Boolean;
				function DoSetupDragCell (c: Cell; dragref: DragReference; dragrgn: RgnHandle): OSErr;
				function DoSetupDrag (dragref: DragReference; dragrgn: RgnHandle): OSErr;
				override;
				procedure RedrawLine( line: integer );
				procedure Draw;
				override;
				procedure AddRows( count, rownum: longint );
				procedure DelRows( count, rownum: longint );
				procedure DeleteSelection;
			end;

	procedure StartupMyListWindow;

implementation

	uses
		MyDialogs, MyAssertions, MyStartup, MyTypes, MyMathUtils, MyMemory, 
		MyWindows, MyEvents, MyAppearance, MyControls, 
		MySystemGlobals, MyListManager, MyCursors, MyCallProc, MyStrings,
		SmartScrollAPI;

{$ifc do_debug}
	var
		startup_check: integer;
{$endc}

	var
		gHActionProc, gVActionProc : UniversalProcPtr;

	procedure ListWindowObject.AddRows( count, rownum: longint );
		var
			junk: integer;
	begin
		junk := LAddRow( count, rownum, list );
		AdjustVControl;
	end;
	
	procedure ListWindowObject.DelRows( count, rownum: longint );
	begin
		LDelRow( count, rownum, list );
		AdjustVControl;
	end;
	
	procedure ListWindowObject.DeleteSelection;
	begin
		LDeleteSelection( list );
		AdjustVControl;
	end;
		
	procedure ListWindowObject.GetWindowRects( var header_info, header_buttons, list_frame: Rect );
	begin
		GetWindowPortRect( window, header_info );
		Dec(header_info.top);
		header_buttons := header_info;
		list_frame := header_info;

		header_info.bottom := header_info.top + header_info_height;
		header_buttons.top := header_info.bottom;
		header_buttons.bottom := header_buttons.top + header_buttons_height;
		list_frame.top := header_buttons.bottom;
	end;
	
	procedure ListWindowObject.Draw;
		var
			header_info, header_buttons, list_frame: Rect;
			view_height: longint;
	begin
// DebugStr( 'Draw Enter' );
		SetPort( window );
		PenNormal;
		GetWindowRects( header_info, header_buttons, list_frame );
		DrawHeaderInfo( header_info );
		DrawHeaderButtons( header_buttons );
		DrawControls( window );
		view_height := longint(list^^.cellSize.v) * RectHeight( list^^.dataBounds );
		if RectHeight( list^^.rView ) > view_height then begin
			list_frame := list^^.rView;
			list_frame.top := list_frame.top + view_height;
			DrawBackground( list_frame );
		end;
		LUpdate( window^.visRgn, list );
		DrawGrow;
// DebugStr( 'Draw Exit' );
	end;

	function ListWindowObject.Match (c: Cell; const what: Str255): Boolean;
	begin
{$unused(c, what)}
		Match := false;
	end;

	procedure ListWindowObject.Find (what: Str255; fromstart, allatonce, backwards: Boolean);
		var
			c: Cell;
			found, found1: Boolean;
	begin
		if allatonce then begin
			found := false;
			c.v := 0;
			c.h := 0;
			while (c.v < LCount( list )) do begin
				found1 := Match(c, what);
				LSetSelect(found1, c, list);
				if found1 then begin
					found := true;
				end;
				c.v := c.v + 1;
			end;
		end else begin
			if backwards then begin
				if fromstart then begin
					c.v := LCount( list )-1;
					c.h := 0;
				end else begin
					c.v := 0;
					c.h := 0;
					if LGetSelect(true, c, list) then begin
						c.v:=c.v-1;
					end else begin
						c.v := LCount( list )-1;
						c.h := 0;
					end;
				end;
				found := false;
				while (c.v >=0) do begin
					found := Match(c, what);
					if found then begin
						leave;
					end;
					c.v := c.v - 1;
				end;
			end else begin
				c.v := 0;
				c.h := 0;
				if not fromstart then begin
					while LGetSelect(true, c, list) do begin
						c.v := c.v + 1;
						c.h := 0;
					end;
				end;
				found := false;
				while (c.v < LCount( list )) do begin
					found := Match(c, what);
					if found then begin
						leave;
					end;
					c.v := c.v + 1;
				end;
			end;
			if found then begin
				SetSingleSelection(c.v);
			end;
		end;
		if not found then begin
			SysBeep(1);
		end;
	end;

	procedure ListWindowObject.DrawHeaderInfo( const lineframe: Rect );
	begin
{$unused(lineframe)}
	end;

	procedure ListWindowObject.DrawBackground( const frame: Rect );
	begin
{$unused(frame)}
		EraseRect( frame );
	end;

	procedure ListWindowObject.DrawHeaderButtons( const lineframe: Rect );
	begin
{$unused(lineframe)}
	end;

	procedure ListWindowObject.DoHeaderInfoClick (const lineframe: Rect; local: Point; const er: EventRecord );
	begin
{$unused(lineframe, local, er)}
	end;

	procedure ListWindowObject.DoHeaderButtonsClick (const lineframe: Rect; local: Point; const er: EventRecord );
	begin
{$unused(lineframe, local, er)}
	end;

	procedure ListWindowObject.SetListWidth (max: integer);
	begin
		list_width := max;
		zoomSize.h := max + 15;
		UpdateZoomHeight;
		AdjustHControl;
	end;

	procedure ListWindowObject.UpdateZoomHeight;
		var
			zv: longint;
	begin
		zv := longint(list^^.cellSize.v) * longint(LCount( list )) + 16 + header_info_height + header_buttons_height;
		if zv > 30000 then begin
			zoomSize.v := 30000;
		end else begin
			zoomSize.v := zv;
		end;
	end;

	procedure ListWindowObject.SetListOffset( listoff: integer );
	begin
		if listoff <> x_list_offset then begin
			x_list_offset := listoff;
			LUpdate( window^.visRgn, list );
		end;
	end;
		
	function ListWindowObject.GetListOffset: integer;
	begin
		GetListOffset := x_list_offset;
	end;
	
	procedure ListWindowObject.AdjustHControl;
{Calculate the new control maximum value and current value }
{max is calculated by comparing the maximum document}
{width to the width of the viewRect. The current values are set by comparing the offset between}
{the view and destination rects. If necessary have the control be re-drawn by}
{calling ShowControl.}
		var
			max: integer;
			oldValue, oldMax: integer;
			oldclip: RgnHandle;
			viswidth: integer;
			new_list_offset: integer;
	begin
		oldValue := GetControlValue(hcontrol);
		oldMax := GetControlMaximum(hcontrol);
//		GetWindowPortRect( window, r );
		max := Max( 0, list_width - RectWidth( list^^.rView ) ); // (r.right - 16 - r.left);
		new_list_offset := Pin(0, oldValue, max);
		SetPort(window);
		ZeroClipRegion( oldclip );
		SetControlMaximum(hcontrol, max);
		RestoreClipRegion( oldclip );
		SetControlValue(hcontrol, new_list_offset);
		viswidth := list^^.rView.right - list^^.rView.left;
		SetSmartScrollInfo( hcontrol, viswidth, max + viswidth );
		if  ((max <> oldMax) or (new_list_offset <> oldValue)) then begin // canRedraw and
			ShowControl(hcontrol);			{check to see if the control can be re-drawn}
		end;
		SetListOffset( new_list_offset );
		SafeSetControlViewSize( hcontrol, RectWidth( list^^.rView ) );
	end;

	procedure ListWindowObject.AdjustVControl;
{Calculate the new control maximum value and current value }
{max is calculated by comparing the maximum document}
{width to the width of the viewRect. The current values are set by comparing the offset between}
{the view and destination rects. If necessary have the control be re-drawn by}
{calling ShowControl.}
		var
			max: integer;
			oldValue, oldMax: integer;
			oldclip: RgnHandle;
			new_list_position: integer;
			curListLines, curDisplayLines: longint;
	begin
		oldValue := GetControlValue(vcontrol);
		oldMax := GetControlMaximum(vcontrol);
//		GetWindowPortRect( window, r );
		curListLines := LCount( list );
		curDisplayLines := RectHeight( list^^.rView ) div list^^.cellSize.v;
		max := Max( 0, curListLines - curDisplayLines ); // (r.right - 16 - r.left);
		new_list_position := Pin(0, oldValue, max);
		SetPort(window);
		ZeroClipRegion( oldclip );
		SetControlMaximum(vcontrol, max);
		RestoreClipRegion( oldclip );
		SetControlValue(vcontrol, new_list_position);
//		viswidth := list^^.rView.right - list^^.rView.left;
		SetSmartScrollInfo( vcontrol, curDisplayLines, curListLines );
		if  ((max <> oldMax) or (new_list_position <> oldValue)) then begin
			ShowControl(vcontrol);
		end;
		SetListPosition( new_list_position );
		SafeSetControlViewSize( vcontrol, curDisplayLines );
	end;
	
	procedure ListWindowObject.MatchVScrollToListPosition;
		var
			pos: integer;
	begin
		pos := list^^.visible.top;
		if GetControlValue( vcontrol ) <> pos then begin
			SetControlValue( vcontrol, pos );
		end;
	end;

	procedure ListWindowObject.SetListPosition( listpos: longint );
		var
			curListLines, curDisplayLines: longint;
	begin
		curListLines := LCount( list );
		curDisplayLines := RectHeight( list^^.rView ) div list^^.cellSize.v;
		if curListLines <= curDisplayLines then begin
			listpos := 0;
			Assert( GetControlMaximum( vcontrol ) = 0 );
		end else begin
			listpos := Pin( 0, listpos, curListLines - curDisplayLines );
			Assert( GetControlMaximum( vcontrol ) = curListLines - curDisplayLines );
		end;
		SetControlValue( vcontrol, listpos );
		LScroll( 0, listpos - list^^.visible.top, list );
//		LUpdate( window^.visRgn, list );
	end;
	
	procedure ListWindowObject.OffsetListPosition( listposoff: longint );
	begin
		SetListPosition( list^^.visible.top + listposoff );
	end;

	procedure ListWindowObject.SetHeights;
	begin
		header_info_height := 0;
		header_buttons_height := 0;
		line_height := windowfiheight + 1;
	end;

	procedure ListWindowObject.CreateList (font, size: integer; ncolumns: integer; ldefID: integer; hscroll: Boolean);
		var
			view, bounds: Rect;
			siz: Point;
			dr: Rect;
			junk: OSErr;
	begin
		AssertDidStartup( startup_check );
		x_list_offset := -1;
		list_is_active := true;
		window_is_active := true;
		SetPort(window);
		if has_Appearance then begin
			junk := CreateRootControl( window, rootcontrol );
		end;
		SafeSetThemeWindowBackground( window, kThemeListViewBackgroundBrush, false );
		columns := ncolumns;
		use_natural_order := true;
		never_drag := true;
		typed_time := 0;
//		max_display_width := maxInt;
		TextFont(font);
		TextSize(size);
		GetFontInfo( windowfi );
		windowfiheight := windowfi.ascent + windowfi.descent + windowfi.leading;
		SetHeights;
		draw_grow_icon := true;
		SetRect( view, 0, 0, 100, 100 );
		SetRect(bounds, 0, 0, 1, 0);
		SetPt(siz, 30000, line_height);
		list := LNew(view, bounds, siz, ldefID, window, true, true, false, false);
		list^^.refCon := longint(gCallLDEFProc);
		list^^.listFlags := list^^.listFlags or lDoVAutoscroll;
		SetRect(dr, 0, 0, 16, 100);
		vcontrol := NewControl(window, dr, '', true, 0, 0, 0, scrollBarProc, 0);
		if hscroll then begin
			SetRect(dr, 0, 0, 100, 16);
			hcontrol := NewControl(window, dr, '', true, 0, 0, 0, scrollBarProc, 0);
		end else begin
			hcontrol := nil;
		end;
		UpdateZoomHeight;
	end;

	procedure ListWindowObject.Destroy;
	begin
		LDispose(list);
		inherited Destroy;
	end;

	procedure ListWindowObject.DoDoubleClickCell (c: Cell);
	begin
{$unused(c)}
	end;

	function ListWindowObject.DoSetupDragCell (c: Cell; dragref: DragReference; dragrgn: RgnHandle): OSErr;
	begin
{$unused(c, dragref, dragrgn)}
		DoSetupDragCell := -17021;
	end;

	function ListWindowObject.DoSetupDrag (dragref: DragReference; dragrgn: RgnHandle): OSErr;
		var
			c: Cell;
			err: OSErr;
	begin
		err := -23;
		c.h := 0;
		c.v := 0;
		while LGetSelect(true, c, list) do begin
			err := DoSetupDragCell(c, dragref, dragrgn);
			if err <> noErr then begin
				leave;
			end;
			c.v := c.v + 1;
			c.h := 0;
		end;
		DoSetupDrag := err;
	end;

	type
		ClickLoopData = record
			first_call: Boolean;
			can_drag: Boolean;
			first_point: Point;
			er: EventRecord;
			wobj: ListWindowObject;
		end; 
		ClickLoopDataPtr = ^ClickLoopData;

	function MyClickLoop( list: ListHandle; data: ClickLoopDataPtr ) : Boolean;
		var
			r, cellRect: Rect;
			cellClicked: Cell;
			curPt: Point;
			dummy: Boolean;
			ret: Boolean;
	begin
		ret := true;
		data^.wobj.MatchVScrollToListPosition;
		if data^.first_call then begin
			data^.first_call := false;
			GetMouse(data^.first_point);
		end else if data^.can_drag then begin
			SetRect(r, data^.first_point.h - 3, data^.first_point.v - 3, data^.first_point.h + 3, data^.first_point.v + 3);
			cellClicked := LLastClick(list);
			LRect(cellRect, cellClicked, list);
			dummy := SectRect(r, cellRect, r);
			GetMouse(curPt);
			if not PtInRect(curPt, r) then begin
				data^.wobj.DoTrackDrag(data^.er);
				ret := false;
			end;
		end;
		MyClickLoop := ret;
	end;

	function ListWindowObject.DontDrag (const er: EventRecord): Boolean;
	begin
{$unused(er)}
		DontDrag := never_drag | (EventHasCommandKey( er ) & not EventHasOptionKey( er )) | EventHasShiftKey( er );
	end;

	function ListWindowObject.DoLClick (const er: EventRecord): Boolean;
		var
			double: Boolean;
			local: Point;
			data: ClickLoopData;
			modifiers: integer;
	begin
		local := er.where;
		GlobalToLocal(local);
		data.first_call := true;
		data.er := er;
		data.wobj := self;
		data.can_drag := has_DragManager & not DontDrag(er);
		modifiers := er.modifiers;
		if data.can_drag & EventHasCommandKey( er ) & EventHasOptionKey( er ) then begin
			modifiers := 0;
		end;
		CursorSetProcessing(false);
		double := LClickSafe(local, modifiers, list, MyClickLoop, @data);
		if double & EventHasCommandKey( er ) then begin
			SetSingleSelection( LLastClick( list ).v );
		end;
		DoLClick := double;
	end;

	procedure ListWindowObject.OpenParent;
	begin
	end;

	procedure ListWindowObject.SetSingleSelection (v: integer);
	begin
		if not EmptyRgn( WindowPeek(window)^.updateRgn ) then begin
			LUpdate(WindowPeek(window)^.updateRgn, list);
		end;
		LSetSingleSelection(list, v);
		LAutoScroll(list);
		SetListPosition( list^^.visible.top );
	end;

	procedure ListWindowObject.DoDoubleClick;
		var
			c: Cell;
	begin
		c.h := 0;
		c.v := 0;
		while LGetSelect(true, c, list) do begin
			DoDoubleClickCell(c);
			c.v := c.v + 1;
			c.h := 0;
		end;
	end;

	var
		action_listobj: ListWindowObject;

	procedure CommonAction (control: ControlHandle; var amount: integer);
		var
			value, max, ovalue: integer;
	begin
		value := GetControlValue(control);
		ovalue := value;
		max := GetControlMaximum(control);
		value := Pin(0, value - amount, max);
		if value <> ovalue then begin
			SetControlValue(control, value);
		end;
		amount := ovalue - value;   { calculate true change }
	end;

{ Determines how much to change the value of the horizontal scrollbar by and how }
{ much to scroll the TE record. }
	procedure HAction (control: ControlHandle; part: integer);
		var
			amount: integer;
			window: WindowPtr;
	begin
		if (part <> 0) then begin
			window := action_listobj.window;
			case part of
				kControlUpButtonPart, kControlDownButtonPart:	begin	{ a few pixels }
					amount := 8;
				end;
				kControlPageUpPart, kControlPageDownPart: begin		{ a page width }
					with action_listobj.list^^.rView do begin
						amount := (right - left);
					end;
				end;
			end;
			if ((part = kControlDownButtonPart) or (part = kControlPageDownPart)) then begin
				amount := -amount;		{ reverse direction }
			end;
			CommonAction(control, amount);
			if amount <> 0 then begin
				action_listobj.SetListOffset( GetControlValue(control) );
				BeginUpdate( action_listobj.window );
				action_listobj.Draw;
				EndUpdate( action_listobj.window );
			end;
		end;
	end;

	procedure VAction (control: ControlHandle; part: integer);
		var
			amount: integer;
			window: WindowPtr;
	begin
		if (part <> 0) then begin
			window := action_listobj.window;
			case part of
				kControlUpButtonPart, kControlDownButtonPart:	begin
					amount := 1;
				end;
				kControlPageUpPart, kControlPageDownPart: begin
					with action_listobj.list^^.visible do begin
						amount := bottom - top -1;
					end;
				end;
			end;
			if ((part = kControlDownButtonPart) or (part = kControlPageDownPart)) then begin
				amount := -amount;	
			end;
			CommonAction(control, amount);
			if amount <> 0 then begin
				action_listobj.SetListPosition( GetControlValue(control) );
				BeginUpdate( action_listobj.window );
				action_listobj.Draw;
				EndUpdate( action_listobj.window );
			end;
		end;
	end;

	function ListWindowObject.GetEntryName (c: Cell): Str255;
	begin
{$unused(c)}
		GetEntryName := '';
	end;

	function ListWindowObject.GetUniqueEntryName (c: Cell): Str255;
	begin
		GetUniqueEntryName := concat(GetEntryName(c), chr(0), chr(c.v div 256), chr(c.v mod 256));
	end;

	function ListWindowObject.SelectFirstAfter (s: Str255): Boolean;
		var
			i, index: integer;
			c: Cell;
			best, n: Str255;
			good: Boolean;
	begin
		good := false;
		best := concat(chr(255), chr(255));
		for i := 0 to LCount( list ) - 1 do begin
			c.h := 0;
			c.v := i;
			n := GetUniqueEntryName(c);
			if (CompareStrings(s, n) < 0) & (CompareStrings(n, best) < 0) then begin
				best := n;
				index := c.v;
				good := true;
			end;
		end;
		if good then begin
			SetSingleSelection(index);
		end;
		SelectFirstAfter := good;
	end;

	function ListWindowObject.SelectFirstBefore (s: Str255): Boolean;
		var
			i, index: integer;
			c: Cell;
			best, n: Str255;
			good: Boolean;
	begin
		good := false;
		index := 0;
		best := '';
		for i := 0 to LCount( list ) - 1 do begin
			c.h := 0;
			c.v := i;
			n := GetUniqueEntryName(c);
			if (CompareStrings(s, n) > 0) & (CompareStrings(n, best) > 0) then begin
				best := n;
				index := c.v;
				good := true;
			end;
		end;
		if good then begin
			SetSingleSelection(index);
		end;
		SelectFirstBefore := good;
	end;

	function ListWindowObject.CompareStrings( const s1, s2: string ): integer;
	begin
		if use_natural_order then begin
			CompareStrings := NaturalOrderCompareString( s1, s2 );
		end else begin
			CompareStrings := CompareString( s1, s2, nil );
		end;
	end;

	function ListWindowObject.GetFirstSelection (var c: Cell): Boolean;
		var
			best, n: Str255;
			index: integer;
	begin
		GetFirstSelection := false;
		c.h := 0;
		c.v := 0;
		best := concat(chr(255), chr(255));
		while LGetSelect(true, c, list) do begin
			GetFirstSelection := true;
			n := GetUniqueEntryName(c);
			if CompareStrings(n, best) < 0 then begin
				index := c.v;
			end;
			c.v := c.v + 1;
		end;
		c.h := 0;
		c.v := index;
	end;

	function ListWindowObject.GetLastSelection (var c: Cell): Boolean;
		var
			best, n: Str255;
			index: integer;
	begin
		GetLastSelection := false;
		c.h := 0;
		c.v := 0;
		best := '';
		while LGetSelect(true, c, list) do begin
			GetLastSelection := true;
			n := GetUniqueEntryName(c);
			if CompareStrings(n, best) > 0 then begin
				index := c.v;
			end;
			c.v := c.v + 1;
		end;
		c.h := 0;
		c.v := index;
	end;
	
	procedure ListWindowObject.DoListTab( const er:EventRecord; backwards: Boolean );
		var
			c: Cell;
			dummy: Boolean;
	begin
{$unused(er)}
		ObscureCursor;
		if backwards then begin
			if not GetFirstSelection(c) | not SelectFirstBefore(GetUniqueEntryName(c)) then begin
				dummy := SelectFirstBefore(chr(255));
			end;
		end else begin
			if not GetLastSelection(c) | not SelectFirstAfter(GetUniqueEntryName(c)) then begin
				dummy := SelectFirstAfter('');
			end;
		end;
	end;

	procedure ListWindowObject.DoTab( const er:EventRecord; backwards: Boolean );
	begin
{$unused(er)}
		DoListTab( er, backwards );
	end;

	procedure ListWindowObject.DoKey (const er:EventRecord; ch: char);
		var
			c: Cell;
			dummy: Boolean;
			index: integer;
			onlyoneselection: Boolean;
	begin
{$unused(er)}
		onlyoneselection := BAND(list^^.selFlags, lOnlyOne) <> 0;
		if ch < ' ' then begin
			typed_time := 0;
		end;
		case ord(ch) of
			tabChar: begin
				DoTab( er, EventHasShiftKey( er ) );
			end;
			downArrowChar:  begin
				if last_event_had_command then begin
					DoDoubleClick;
				end else begin
					ObscureCursor;
					if LCount( list ) > 0 then begin
						if LGetLastSelection(list, c) then begin
							index := c.v + 1;
						end else begin
							index := 0;
						end;
						if index >= LCount( list ) then begin
							index := LCount( list ) - 1;
						end;
						if onlyoneselection or not last_event_had_shift then begin
							SetSingleSelection(index);
						end else begin
							c.v := index;
							LSetSelect(true, c, list);
						end;
					end;
				end;
			end;
			upArrowChar:  begin
				if last_event_had_command then begin
					OpenParent;
				end else begin
					ObscureCursor;
					if LCount( list ) > 0 then begin
						if not LGetFirstSelection(list, c) then begin
							c.v := LCount( list );
						end;
						c.v := c.v - 1;
						if c.v < 0 then begin
							c.v := 0;
						end;
						if onlyoneselection or not last_event_had_shift then begin
							SetSingleSelection(c.v);
						end else begin
							LSetSelect(true, c, list);
						end;
					end;
				end;
			end;
			leftArrowChar: begin
				DoListTab( er, true );
			end;
			rightArrowChar: begin
				DoListTab( er, false );
			end;
			homeChar:  begin
				ObscureCursor;
				SetListPosition( 0 );
			end;
			endChar:  begin
				ObscureCursor;
				SetListPosition( LCount( list ) );
			end;
			pageUpChar:  begin
				ObscureCursor;
				OffsetListPosition( -(RectHeight( list^^.visible ) - 1) );
			end;
			pageDownChar:  begin
				ObscureCursor;
				OffsetListPosition( (RectHeight( list^^.visible ) - 1) );
			end;
			3, 13:  begin
				DoDoubleClick;
			end;
			otherwise begin
				ObscureCursor;
				if ch >= ' ' then begin
					if last_event_time - typed_time > 60 then begin
						typed_chars := '';
					end;
					typed_time := last_event_time;
					typed_chars := concat(typed_chars, ch);
					if not SelectFirstAfter(typed_chars) then begin
						dummy := SelectFirstBefore(chr(255));
					end;
				end;
			end;
		end;
{ WARNING: self may have been destroyed! }
	end;
	
	procedure GetControlThumbSize( control: ControlHandle; var siz: Point );
		var
			junk: longint;
			rgn: RgnHandle;
	begin
		if system_version < $0800 then begin
			SetPt( siz, 16, 16 );
		end else begin
			rgn := NewRgn;
			junk := CallControlDefProc( GetControlVariant(control), control, calcThumbRgn, SInt32(rgn), control^^.contrlDefProc^ );
			siz.h := RectWidth( rgn^^.rgnBBox );
			siz.v := RectHeight( rgn^^.rgnBBox );
			DisposeRgn( rgn );
		end;
	end;
	
	const
		control_arrow_size = 16;
	
	procedure ListWindowObject.TrackVControl( local: Point );
		var
			factor: real;
			min, max: longint;
			initial, value, curvalue: longint;
			curpos: Point;
			thumbsiz: Point;
	begin
		SetPort( window );
		GetControlThumbSize( vcontrol, thumbsiz );
		initial := GetControlValue( vcontrol );
		curvalue := initial;
		min := GetControlMinimum( vcontrol );
		max := GetControlMaximum( vcontrol );
		factor := (max - min + 1) / (RectHeight( vcontrol^^.contrlRect ) - control_arrow_size * 2 - thumbsiz.v );
		while StillDown do begin
			GetMouse( curpos );
			value := Pin( min, initial + Trunc( (curpos.v - local.v) * factor + 0.5),max );
			if value <> curvalue then begin
				curvalue := value;
				SetListPosition( value );
				BeginUpdate( window );
				Draw;
				EndUpdate( window );
			end;
		end;
	end;
	
	procedure ListWindowObject.TrackHControl( local: Point );
		var
			factor: real;
			min, max: longint;
			initial, value, curvalue: longint;
			curpos: Point;
			thumbsiz: Point;
	begin
		SetPort( window );
		GetControlThumbSize( hcontrol, thumbsiz );
		initial := GetControlValue( hcontrol );
		curvalue := initial;
		min := GetControlMinimum( hcontrol );
		max := GetControlMaximum( hcontrol );
		factor := (max - min + 1) / (RectWidth( hcontrol^^.contrlRect ) - control_arrow_size * 2 - thumbsiz.h );
		while StillDown do begin
			GetMouse( curpos );
			value := Pin( min, initial + Trunc( (curpos.h - local.h) * factor + 0.5),max );
			if value <> curvalue then begin
				curvalue := value;
				SetControlValue( hcontrol, value );
				SetListOffset( value );
				BeginUpdate( window );
				Draw;
				EndUpdate( window );
			end;
		end;
	end;
	
	procedure ListWindowObject.DoListClick(const lineframe: Rect; local: Point; const er: EventRecord );
		var
			didit: Boolean;
			ctl: ControlHandle;
			part, value: integer;
	begin
{$unused(lineframe)}
		didit := false;
		if (hcontrol <> nil) | (vcontrol <> nil)  then begin
			part := FindControl(local, window, ctl);
			if ctl <> nil then begin
				if ctl = hcontrol then begin
					didit := true;
					if part = kControlIndicatorPart then begin
						TrackHControl( local );
//						value := GetControlValue(hcontrol);
//						part := SafeHandleControlIndicatorClick(hcontrol, local, er.modifiers, HIndicatorAction);
					end else begin
						action_listobj := self;
						value := TrackControl(hcontrol, local, gHActionProc);
					end;
					SetListOffset( GetControlValue(hcontrol) );
				end;
				if ctl = vcontrol then begin
					didit := true;
					if part = kControlIndicatorPart then begin
						TrackVControl( local );
//						value := GetControlValue(vcontrol);
//						part := SafeHandleControlIndicatorClick(vcontrol, local, er.modifiers, VIndicatorAction);
					end else begin
						action_listobj := self;
						value := TrackControl(vcontrol, local, gVActionProc);
					end;
					SetListPosition( GetControlValue(vcontrol) );
				end;
			end;
		end;
		if not didit & DoLClick(er) then begin
			DoDoubleClick;
		end;
	end;

	procedure ListWindowObject.DoContent( const er: EventRecord );
		var
			local: Point;
			header_info, header_buttons, list_frame: Rect;
	begin
		SetPort(window);
		local := er.where;
		GlobalToLocal(local);
		GetWindowRects( header_info, header_buttons, list_frame );
		if PtInRect( local, header_info ) then begin
			DoHeaderInfoClick( header_info, local, er );
		end else if PtInRect( local, header_buttons ) then begin
			DoHeaderButtonsClick( header_buttons, local, er );
		end else begin
			DoListClick( list_frame, local, er );
		end;
	end;
	
	procedure ListWindowObject.ListActivateDeactivate (activate: Boolean);
	begin
		list_is_active := activate;
		LActivate(window_is_active and list_is_active, list);
	end;
	
	procedure ListWindowObject.DoActivateDeactivate (activate: Boolean);
	begin
		window_is_active := activate;
		LActivate(activate & list_is_active, list);
		if activate then begin
			ShowControl( vcontrol );
			ShowControl( hcontrol );
		end else begin
			HideControl( vcontrol );
			HideControl( hcontrol );
		end;
//		SetAllControlEnables( activate );
		inherited DoActivateDeactivate(activate);
	end;

	procedure ListWindowObject.Resize;
		var
//			r: Rect;
			nheight, lineheight: integer;
			header_info, header_buttons, list_frame: Rect;
//			oldclip: RgnHandle;
	begin
// DebugStr( 'Resize Enter' );
		SetPort(window);
		lineheight := list^^.cellSize.v;
		GetWindowRects( header_info, header_buttons, list_frame );
		nheight := (RectHeight( list_frame ) - 15) mod lineheight;
		if nheight <> 0 then begin
			list_frame.bottom := list_frame.bottom - nheight;
			SizeWindow(window, RectWidth( list_frame ), list_frame.bottom, false);
		end;
//		ZeroClipRegion( oldclip );  // doesnt zoom out very nicely
		growRect.top := (50 + lineheight - 1) div lineheight * lineheight + list_frame.top + 16;
		list^^.rView.topLeft := list_frame.topLeft; { LMove???? }
		LSize( RectWidth( list_frame ) - 15, RectHeight( list_frame ) - 15, list);
		if hcontrol <> nil then begin
			hcontrol^^.contrlVis := kControlIsInvisible;
			CleanMoveControl(hcontrol, list_frame.left - 1, list_frame.bottom - 15);
			CleanSizeControl(hcontrol, RectWidth( list_frame ) - 13, 16);
			AdjustHControl;
			hcontrol^^.contrlVis := kControlIsVisible;
		end;
		if vcontrol <> nil then begin
			vcontrol^^.contrlVis := kControlIsInvisible;
			CleanMoveControl(vcontrol, list_frame.right - 15, list_frame.top - 1);
			CleanSizeControl(vcontrol, 16, RectHeight( list_frame ) - 13);
			AdjustVControl;
			vcontrol^^.contrlVis := kControlIsVisible;
		end;
//		RestoreClipRegion( oldclip );
		UpdateZoomHeight;
		InvalRect(window^.portRect);
		inherited Resize;
// DebugStr( 'Resize Exit' );
	end;

	procedure ListWindowObject.DrawGrow;
		var
			header_info, header_buttons, list_frame: Rect;
	begin
		GetWindowRects( header_info, header_buttons, list_frame );
		DrawTheFriggingGrowIcon(window, list_frame);
	end;

	function ListWindowObject.CountSelections: integer;
	begin
		CountSelections := LCountSelections(list);
	end;

	function ListWindowObject.IsSelection: Boolean;
	begin
		IsSelection := LHasSelection(list);
	end;

	procedure ListWindowObject.RedrawLine( line: integer );
		var
			c: Cell;
	begin
		c.v := line;
		c.h := 0;
		LDraw( c, list );
	end;

	function InitMyListWindow(var msg: integer): OSStatus;
	begin
{$unused(msg)}
		DidStartup( startup_check );
		DidStartup( startup_check );
		gHActionProc := NewControlActionProc(HAction);
		gVActionProc := NewControlActionProc(VAction);
		InitMyListWindow := noErr;
	end;

	procedure StartupMyListWindow;
	begin
		StartupDialogs;
		StartupMainLoop;
		StartupAppearance;
		SetStartup( InitMyListWindow, nil, 0, nil );
	end;

end.
(*
	function MyClickLoop: Boolean; { returns the bloody equal flag for gods sake! }
	begin
		MyClickLoop := MyClickLoop2; { BE VERY CAREFUL!  Returns the equal flag! }
	end;
//						if part <> 0 then begin
//							if value <> GetListOffset then begin
//								InvalRect(window^.portRect);
//							end;
//							SetListOffset( GetControlValue(hcontrol) );
//						end;
	procedure HIndicatorAction (control: ControlHandle; part: integer);
		var
			amount: integer;
			window: WindowPtr;
	begin
		if (part <> 0) then begin
			window := action_listobj.window;
		end;
	end;

	procedure VIndicatorAction (control: ControlHandle; part: integer);
		var
			amount: integer;
			window: WindowPtr;
	begin
		if (part <> 0) then begin
			window := action_listobj.window;
		end;
	end;

*)
