unit MyMacBinary;

interface

	uses
		MacOS;
		
	const
		mbin_standard_signature = 'mBIN';
		
	type
		MBpacket = packed array[1..128] of Byte;
		
	const
		sizeof_MBIIStartHeader = 101;
		sizeof_MBpacket = 128;
		sizeof_MBIIHeader = 128;

{$PUSH}
{$ALIGN MAC68K}

	type
		MBIIStartHeader = packed record
				name: Str63;
				ftype: OSType;
				fcreator: OSType;
				nflags_high: Byte; // changed to nflags to ensure anyone using this code considers script & xflags
				zero1: Byte;
				flocation: Point;
				windowID: integer; {? - ignore }
				protected: Byte; { low order bit - ignore}
				zero2: Byte;
				dlen: longint;
				rlen: longint;
				create_date: UInt32;
				mod_date: UInt32;
				clen: integer;
				nflags_low: Byte;
			end;
		MBIIHeader = packed record
				version: SignedByte;
				MBIIStart: SignedByte;
				space: packed array[2..101] of Byte;
				mbin3_signature: OSType;
				script: SInt8;
				xflags: SInt8;
				pad108: longint;
				pad112: longint;
				total_unpack_len: longint;{ignore}
				second_header_len: integer;{ignore}
				versionII: Byte;
				minversionII: Byte;
				crc: integer;
				processorID: integer; {ignore}
			end;
		MBIIHeaderPtr = ^MBIIHeader;
{$ALIGN RESET}
{$POP}

	function ValidateMBHeader (var header: MBIIHeader): Boolean;
	procedure CatInfo2MBHeader (var pb: CInfoPBRec; var header: MBIIHeader; dtdbr: integer; var comment: Str255);
	procedure CalcCRCForMacBinaryHeader( var header: MBIIHeader );

implementation

	uses
		MyMemory, MyDesktopDB, MyAssertions, MyFileSystemUtils, CalcCRC;

	function ValidateMBHeader (var header: MBIIHeader): Boolean;
		var
			ocrc: integer;
			result: boolean;
			start: MBIIStartHeader;
			i: integer;
	begin
		Assert( sizeof_MBIIStartHeader = SizeOf(MBIIStartHeader) - 1 );
		Assert( sizeof_MBIIHeader = SizeOf(MBIIHeader) );
		BlockMoveData(@header.MBIIStart, @start, SizeOf(start));
		result := false;

		if (header.version = 0) & (1 <= length(start.name)) & (length(start.name) <= 63) then begin
			ocrc := 0;
			CalcMBCRCBlock(@header, 124, ocrc);
			if ocrc = MBIIHeader(header).crc then begin
				result := true;
			end else if (length(start.name) <= 31) & (MBpacket(header)[75] = 0) then begin { Assume its a valid MacBinary I file }
				MBpacket(header)[101] := 0; { Zero out the flags low_byte }
				header.total_unpack_len := 0;
				header.second_header_len := 0;
				header.versionII := 129;
				header.minversionII := 129;
				header.crc := 0;
				header.processorID := 0;
				header.mbin3_signature := OSType(0);
				result := true;
			end;
			if result & (header.mbin3_signature <> mbin_standard_signature) then begin
				header.mbin3_signature := mbin_standard_signature;
				header.versionII := 130;
				header.script := 0;
				header.xflags := 0;
			end;
		end;

		if result then begin
			result := false;
			if (0 <= start.clen) & (start.clen <= 200)
				& (0 <= start.dlen) & (0 <= start.rlen)
				& (0 < length(start.name)) & (length(start.name) <= 63) then begin
				result := true;
				for i := 1 to length(start.name) do begin
					if (start.name[i] = chr(0)) | (start.name[i] = ':') then begin
						result := false;
						leave;
					end;
				end;
			end;
		end;
		ValidateMBHeader := result;
	end;
	
	procedure CalcCRCForMacBinaryHeader( var header: MBIIHeader );
		var
			ocrc: integer;
	begin
		ocrc := 0;
		CalcMBCRCBlock(@header, 124, ocrc);
		header.crc := ocrc;
	end;

	procedure CatInfo2MBHeader (var pb: CInfoPBRec; var header: MBIIHeader; dtdbr: integer; var comment: Str255);
		var
			start: MBIIStartHeader;
			fs: FSSpec;
	begin
		Assert( sizeof_MBIIStartHeader = SizeOf(MBIIStartHeader) - 1 );
		Assert( sizeof_MBIIHeader = SizeOf(MBIIHeader) );
		Assert( not PBIsFolder( pb ) );
		MZero( @header, SizeOf(header) );
		MZero( @start, SizeOf(start) );
		header.version := 0;
		header.minversionII := 129;
		if (header.script <> 0) | (header.xflags <> 0) then begin // MB3
			header.versionII := 130;
			header.mbin3_signature := mbin_standard_signature;
			SInt8(header.script) := SInt8(pb.ioFlXFndrInfo.fdScript);
			SInt8(header.xflags) := SInt8(pb.ioFlXFndrInfo.fdXFlags);
		end else begin // MB2
			header.versionII := 129;
		end;
		
		start.name := pb.ioNamePtr^;
		start.nflags_high := BAND(BSR(pb.ioFlFndrInfo.fdFlags, 8), $0FF);
		start.nflags_low := BAND(pb.ioFlFndrInfo.fdFlags, $0FF);
		start.flocation := pb.ioFlFndrInfo.fdLocation;
		start.windowID := pb.ioFlFndrInfo.fdFldr;
		start.create_date := pb.ioFlCrDat;
		start.mod_date := pb.ioFlMdDat;

		start.ftype := pb.ioFlFndrInfo.fdType;
		start.fcreator := pb.ioFlFndrInfo.fdCreator;
		start.dlen := pb.ioFlLgLen;
		start.rlen := pb.ioFlRLgLen;

		fs.vRefNum := pb.ioVRefNum;
		fs.parID := pb.ioFlParID;
		fs.name := pb.ioNamePtr^;
		GetDTDBComment(dtdbr, fs, comment);
		start.clen := length(comment);
		BlockMoveData(@start, @header.MBIIStart, sizeof_MBIIStartHeader);
		CalcCRCForMacBinaryHeader( header );
	end;

end.
