unit MyMemory;

interface

	uses
		MyAssertions;

	const
		trash_byte = $E5; { odd, big, negative, easily recognizable }

{ WARNING: MTrash et al only do anything in debugging mode! }

{$ifc not do_debug}
{$definec MTrash(p,s)}
{$definec MTrashPtr(p)}
{$definec MTrashHandle(data)}
{$elsec}
{$definec MTrash(p,s) MFill(p,s,trash_byte)}
{$definec MTrashPtr(p) MFill(p,GetPtrSize(p),trash_byte)}
{$definec MTrashHandle(data) MFill(data^,MGetHandleSize(data),trash_byte)}
{$endc}

{$ifc undefined do_debug_memory}		{ controls tracking of Handle/Ptr allocation/deallocation }
{$setc do_debug_memory := 0}				{ defaults to off if not specified }
{$endc}

{$ifc not do_debug_memory}
{$definec MDumpMemoryDebug}
{$definec MFinishMemoryDebug(fileIDX)}
{$definec MNextMemoryEpoch}
{$definec MNoteNewHandle(data)}
{$definec MNoteNewPtr(data)}
{$definec MForgetMemory(data)}
{$elsec}
{$definec MDumpMemoryDebug MDumpMemoryDebugCode}
{$definec MFinishMemoryDebug(fileIDX) MFinishMemoryDebugCode(fileIDX)}
{$definec MNextMemoryEpoch MNextMemoryEpochCode}
{$definec MNoteNewHandle(data) MNoteNewHandleCode(data)}
{$definec MNoteNewPtr(data) MNoteNewPtrCode(data)}
{$definec MForgetMemory(data) MForgetMemoryCode(data)}
{$endc}

{$ifc do_debug_memory}
	procedure MNoteNewHandleCode( data: univ Handle );
	procedure MNoteNewPtrCode( data: univ Ptr );
	procedure MForgetMemoryCode( data: univ Ptr );
{$endc}
	
	function MNewPtr( var p: univ Ptr; size: longint ): OSErr;
	function MNewPtrClear( var p: univ Ptr; size: longint ): OSErr;
	procedure MDisposePtr ( var p: univ Ptr );
	function MGetPtrSize( data: univ Ptr ): longint;
	function MSetPtrSize ( p: univ Ptr; size: longint ): OSErr;

	function MNewHandle( var data: univ Handle; size: longint ): OSErr;
	function MNewHandleClear( var data: univ Handle; size: longint ): OSErr;
	function MNewTempHandle( var data: univ Handle; size: longint ): OSErr;

	function MNewHandleWith( var data: univ Handle; contents: Ptr; size: longint ): OSErr;
	function MNewHandleWithString( var data: univ Handle; const contents: string ): OSErr;

	procedure MDisposeHandle ( var data: univ Handle );

	function MGetHandleSize( data: univ Handle ): longint;

	function MSetHandleSize ( data: univ Handle; size: longint ): OSErr;
	function MGrowHandleSize ( data: univ Handle; size: longint ): OSErr;
	procedure MShrinkHandleSize( data: univ Handle; size: longint );

	function MNewRgn: RgnHandle;
	procedure MDisposeRgn( var rgn: RgnHandle );
	
	procedure MDetachResource( data: univ Handle );
	procedure MRemoveResource( data: univ Handle );
	procedure MAddResource( data: univ Handle; theType: ResType; theID: integer; const name: Str255 );
	
	function MMungerFindString( data: univ Handle; offset: longint; const s: string ): longint;
	function MMungerFind( data: univ Handle; offset: longint; ptr1: univ Ptr; len1: longint ): longint;
	function MMungerInsert( data: univ Handle; offset: longint; ptr2: univ Ptr; len2: longint ): OSErr;
	function MMungerInsertString( data: univ Handle; offset: longint; const s: string ): OSErr;
	procedure MMungerDelete( data: univ Handle; offset: longint; len1: longint );
	function MHandleToHandle( source, dest: univ Handle ): OSErr;
	function MAppendToHandle( data: univ Handle; p: univ Ptr; len: longint ): OSErr;
	function MAppendString( data: univ Handle; const s: string ): OSErr;
	
	function MNewIconSuite( var iconsuite: Handle ): OSErr;
	function MAddIconToSuite( var data: Handle; iconsuite: Handle; typ: OSType ): OSErr;
	function MAddIconDataToSuite( data: Ptr; siz: integer; iconsuite: Handle; typ: OSType ): OSErr;
	procedure MDisposeIconSuite( var iconsuite: Handle );

	procedure MZero ( p: univ Ptr; size: longint );
	procedure MFill ( p: univ Ptr; size: longint; val: integer );
	procedure MFillLong ( p: univ Ptr; size: longint; val: longint );
{ Ptr and size must be long alligned }

	procedure MLockHigh ( data: univ Handle );
	procedure HLockState( data: univ Handle; var waslocked: boolean );
	procedure HRestoreLockState( data: univ Handle; waslocked: boolean );
	procedure HNoPurgeState ( data: univ Handle; var waspurgeable: boolean );
	procedure HRestorePurgeState( data: univ Handle; waspurgeable: boolean );

	procedure MHLock( data: univ Handle );
	procedure MHUnlock( data: univ Handle );
	procedure MHPurge( data: univ Handle );
	procedure MHNoPurge( data: univ Handle );

	function CheckPointer ( p: Ptr ): Boolean;
	function CheckPtr ( p: Ptr ): Boolean;
	function CheckHandle ( data: Handle ): Boolean;
	function CheckObject( data: univ Handle ): Boolean;
	
{$definec MNewObject(obj) new(obj); MNewObjectCode(obj)}
{$definec MDisposeObject(obj) MDisposeObjectCode(obj); dispose(obj)}

	procedure MNewObjectCode( obj: univ Handle );
	procedure MDisposeObjectCode( obj: univ Handle );

// Depricated, unnecessary if MNewObject/MDisposeObject  is used
{$ifc option(ptrbasedobject)}
{$definec MLockObjHigh(obj)}
{$elsec}
{$definec MLockObjHigh(obj) MLockHigh(obj)}
{$endc}

{$ifc do_debug_memory}
	procedure MDumpMemoryDebugCode;
	procedure MFinishMemoryDebugCode(fileIDX: integer);
	function MNextMemoryEpochCode: integer;
{$endc}

implementation

	uses
{$ifc do_debug_memory}
		MyFileSystemUtils, MyUtils, MyTypes, 
{$endc}
		MyLowLevel;

{$ifc do_debug_memory}

	const
		initedFlag = $13579;
		activate_debug_memory = true;
		stopOnUnknownBlock = true;

	type
		memEntry = record
			next: memEntryP;
			prev: memEntryP;
			porh: LongInt;
			siz: LongInt;
			epoch: integer;
			kind: char;
			extra: char;
		end;
		memEntryP = ^memEntry;
	
	var
		memHead: memEntryP;
		currentEpoch: integer;
		initFlag: LongInt;
		
	function GetObjectSize( data: univ Handle ): longint;
	begin
{$ifc option(ptrbasedobject)}
		GetObjectSize := MGetPtrSize( data );
{$elsec}
		GetObjectSize := MGetHandleSize( data );
{$endc}
	end;
	
	procedure MInitMemoryDebug;
	begin
		if (initFlag <> initedFlag) then begin
			currentEpoch := 0;
			memHead := nil;
			initFlag := initedFlag;
		end;
	end;
	
	procedure MDumpMemoryDebugCode;
		var
			err: OSErr;
			spec: FSSpec;
			refnum: integer;
			s, t: Str255;
	begin
		s := LMGetCurApName^;
		t := ' Memory Debug';
		if length(s) >= 31 - length(t) then begin
			s[0] := chr(31 - length(t));
		end;
		s := s + t;
		err := FSMakeFSSpec( -1, 2, s, spec );
		if err = fnfErr then begin
			err := noErr;
		end;
		if err = noErr then begin
			FSpRecreate( spec, 'R*ch', 'TEXT', kZapDF, kZapRF );
			err := FSpOpenDF( spec, fsRdWrPerm, refnum );
		end;
		if err = noErr then begin
			MFinishMemoryDebug( refnum );
			err := FSClose( refnum );
		end;
	end;

	procedure MFinishMemoryDebugCode(fileIDX: integer);
		var
			prev, mep: memEntryP;
			err: OSErr;
			sz: LongInt;
			szs: Str15;
	
		function xNumToStr(x: LongInt): Str15;
			var
				s: Str15;
		begin
			s := NumToStr(x);
			while (length(s) < 8) do begin
				s := concat(' ', s);
			end;
			xNumToStr := s;
		end;
		
		var
			line: Str255;

	begin
		DebugStr( 'Check for leaks;g' );
		if activate_debug_memory then begin
			MInitMemoryDebug;
			mep := memHead;
			if (fileIDX <> 0) then begin
				err := MyFSWriteString(fileIDX, concat('*  Mem Blk       OrgSize     Size       Epoch', chr(12)));
			end;
			while (mep <> nil) do begin
				if (fileIDX <> 0) then begin
					if mep^.porh <> 0 then begin
						case mep^.kind of
							'H': begin
								if CheckHandle( Handle(mep^.porh) ) then begin
									sz := GetHandleSize( Handle(mep^.porh) );
								end;
							end;
							'P': begin
								if CheckPtr( Ptr(mep^.porh) ) then begin
									sz := GetPtrSize(Ptr(mep^.porh));
								end;
							end;
							'O': begin
								if CheckObject( Handle(mep^.porh) ) then begin
									sz := GetObjectSize( Handle(mep^.porh) );
								end;
							end;
							otherwise begin
								Assert( false );
							end;
						end;
					end;
					if sz >= 0 then begin
						szs := xNumToStr(sz);
					end else begin
						szs := 'Invalid ';
					end;
					line := concat(mep^.kind, mep^.extra, ' $', HexNN(mep^.porh, 8), '    ', 
															xNumToStr(mep^.siz), '    ', szs, '    ',
														  xNumToStr(mep^.epoch), cr);
					err := MyFSWriteString( fileIDX, line );
					DebugStr( line );
				end;
				prev := mep;
				mep := mep^.next;
				DisposePtr(Ptr(prev));
			end;
			if memHead <> nil then begin
				DebugStr( 'Some Leaks!  See debug file' );
			end;
			memHead := nil;
		end else begin
			DebugStr( 'Leak Detection Inactive' );
		end;
	end;
	
	function MNextMemoryEpochCode: integer;
	begin
		currentEpoch := currentEpoch + 1;
		MNextMemoryEpochCode := currentEpoch;
	end;
	
	procedure debugAddPorH( thing: univ longint; siz: LongInt; kind: char; extra: char);
		var
			mep: memEntryP;
			err: OSErr;
	begin
(*
if true & (siz = 52) & (kind = 'O') & (extra = 'N') then begin
	DebugStr( 'debugAddPorH ' + HexNN(thing,8) + ' ' + NumToStr(siz) + kind + extra + ';sc;g' );
end;
*)
		if activate_debug_memory then begin
			MInitMemoryDebug;
			mep := memEntryP(NewPtr(SizeOf(memEntry)));
			err := MemError;
			Assert (err = noErr);
			mep^.epoch := currentEpoch;
			mep^.prev := nil;
			mep^.porh := thing;
			mep^.siz := siz;
			mep^.kind := kind;
			mep^.extra := extra;
			mep^.next := memHead;
			if (memHead <> nil) then begin
				memHead^.prev := mep;
			end;
			memHead := mep;
//			if mep^.porh = $03D1D1E4 then begin
//				DebugStr( 'got it' );
//			end;
		end;
	end;
	
	procedure debugDelPorH( thing: univ LongInt );
		var
			mep: memEntryP;
	begin
		if activate_debug_memory then begin
			MInitMemoryDebug;
			mep := memHead;
			while (mep <> nil) do begin
				if (mep^.porh = thing) then begin
					if (mep^.prev <> nil) then begin
						mep^.prev^.next := mep^.next;
					end else begin
						memHead := mep^.next;
					end;
					if (mep^.next <> nil) then begin
						mep^.next^.prev := mep^.prev;
					end;
					DisposePtr(Ptr(mep));
					exit(debugDelPorH);
				end else begin
					mep := mep^.next;
				end;
			end;
			if stopOnUnknownBlock then begin
				DebugStr('Deallocating unknown block;sc;hc');
			end;
		end;
	end;
{$endc}

	function CheckPointer ( p: Ptr ): Boolean;
	begin
		Assert( p <> nil );
		CheckPointer := p <> nil;
	end;

	function CheckPtr ( p: Ptr ): Boolean;
	begin
		Assert( (p <> nil) & (GetPtrSize( p ) >= 0) & (MemError = noErr) );
		CheckPtr := p <> nil;
	end;

	function CheckHandle ( data: Handle ): Boolean;
	begin
		Assert( (data <> nil) & (GetHandleSize( data ) >= 0) & (MemError = noErr) );
		CheckHandle := data <> nil;
	end;

	function CheckObject( data: univ Handle ): Boolean;
	begin
{$ifc option(ptrbasedobject)}
		CheckObject := CheckPtr( Ptr(data) );
{$elsec}
		CheckObject := CheckHandle( data );
{$endc}
	end;
	
	procedure MNewObjectCode( obj: univ Handle );
	begin
		if CheckObject( obj ) then begin
{$ifc not option(ptrbasedobject)}
			MLockHigh(obj);
{$endc}
{$ifc do_debug_memory}
			debugAddPorH( obj, GetObjectSize( obj ), 'O', 'N' );
{$endc}
		end;
	end;
	
	procedure MDisposeObjectCode( obj: univ Handle );
	begin
		if CheckObject( obj ) then begin
{$ifc do_debug_memory}
			debugDelPorH( obj );
{$endc}
		end;
	end;

	function MNewPtr ( var p: univ Ptr; size: longint ): OSErr;
		var
			err: OSErr;
	begin
		Assert( size >= 0 );
		p := NewPtr(size);
		err := MemError;
		if err = noErr then begin
			MTrashPtr( p );
{$ifc do_debug_memory}
			debugAddPorH( p, size, 'P', 'N' );
{$endc}
		end;
		MNewPtr := err;
	end;

	function MNewPtrClear( var p: univ Ptr; size: longint ): OSErr;
		var
			err: OSErr;
	begin
		Assert( size >= 0 );
		p := NewPtrClear( size );
		err := MemError;
		if err = noErr then begin
{$ifc do_debug_memory}
			debugAddPorH( p, size, 'P', 'C' );
{$endc}
		end;
		MNewPtrClear := err;
	end;

	function MNewRgn: RgnHandle;
		var
			rgn: RgnHandle;
	begin
		rgn := NewRgn;
		Assert( rgn <> nil );
{$ifc do_debug_memory}
		debugAddPorH( rgn, GetHandleSize(Handle(rgn)), 'H', 'G' );
{$endc}
		MNewRgn := rgn;
	end;
	
	procedure MDisposeRgn( var rgn: RgnHandle );
	begin
		if CheckHandle( Handle(rgn) ) then begin
{$ifc do_debug_memory}
			debugDelPorH( rgn );
{$endc}
			DisposeRgn( rgn );
		end;
		rgn := nil;
	end;

	function MNewTempHandle( var data: univ Handle; size: longint ): OSErr;
		var
			err: OSErr;
	begin
		Assert( size >= 0 );
		data := TempNewHandle( size, err );
		if err = noErr then begin
			MTrashHandle( data ); 
{$ifc do_debug_memory}
			debugAddPorH( data, size, 'H', 'T' );
{$endc}
		end;
		MNewTempHandle := err;
	end;

	function MNewHandle ( var data: univ Handle; size: longint ): OSErr;
		var
			err: OSErr;
	begin
		Assert( size >= 0 );
		data := NewHandle( size );
		err := MemError;
		if err = noErr then begin
			MTrashHandle( data ); 
{$ifc do_debug_memory}
			debugAddPorH( data, size, 'H', 'N' );
{$endc}
		end;
		MNewHandle := err;
	end;

	function MNewHandleWith( var data: univ Handle; contents: Ptr; size: longint ): OSErr;
		var
			err: OSErr;
	begin
		Assert( size >= 0 );
		Assert( contents <> nil );
		err := PtrToHand( contents, data, size );
		if err = noErr then begin
{$ifc do_debug_memory}
			debugAddPorH( data, size, 'H', 'W' );
{$endc}
		end;
		MNewHandleWith := err;
	end;
	
	function MNewHandleWithString( var data: univ Handle; const contents: string ): OSErr;
	begin
		MNewHandleWithString := MNewHandleWith( data, @contents[1], length(contents) );
	end;
	
	function MNewHandleClear( var data: univ Handle; size: longint ): OSErr;
		var
			err: OSErr;
	begin
		Assert( size >= 0 );
		data := NewHandleClear( size );
		err := MemError;
		if err = noErr then begin
{$ifc do_debug_memory}
			debugAddPorH( data, size, 'H', 'C' );
{$endc}
		end;
		MNewHandleClear := err;
	end;

{$ifc do_debug_memory}
	procedure MNoteNewHandleCode( data: univ Handle );
	begin
		if CheckHandle( data ) then begin
			debugAddPorH( data, GetHandleSize( data ), 'H', 'X' );
		end;
	end;
	
	procedure MNoteNewPtrCode( data: univ Ptr );
	begin
		if CheckPtr( data ) then begin
			debugAddPorH( data, GetPtrSize( data ), 'P', 'X' );
		end;
	end;

	procedure MForgetMemoryCode( data: univ Ptr );
	begin
		debugDelPorH( data );
	end;
{$endc}

	procedure MDetachResource( data: univ Handle );
	begin
		if CheckHandle( data ) then begin
			DetachResource( data );
{$ifc do_debug_memory}
			debugAddPorH( data, GetHandleSize( data ), 'H', 'D' );
{$endc}
		end;
	end;
	
	procedure MRemoveResource( data: univ Handle );
	begin
		if CheckHandle( data ) then begin
			RemoveResource( data );
{$ifc do_debug_memory}
			debugAddPorH( data, GetHandleSize( data ), 'H', 'R' );
{$endc}
		end;
	end;
	
	procedure MAddResource( data: univ Handle; theType: ResType; theID: integer; const name: Str255 );
	begin
		if CheckHandle( data ) then begin
			AddResource( data, theType, theID, name );
{$ifc do_debug_memory}
			debugDelPorH( data );
{$endc}
		end;
	end;

	function MSetPtrSize ( p: univ Ptr; size: longint ): OSErr;
{$ifc do_debug}
		var
			oldsize: longint;
{$endc}
	begin
{$ifc do_debug}
		Assert( p <> nil );
		Assert( size >= 0 );
		oldsize := GetPtrSize( p );
		if oldsize < size then begin
			SetPtrSize( p, size );
			if MemError = noErr then begin
				MTrash( AddPtrLong( p, oldsize ), size - oldsize );
			end;
		end else if oldsize > size then begin
			MTrash( AddPtrLong( p, size ), oldsize - size );
		end;
{$endc}
		if CheckPtr( p ) then begin
			SetPtrSize( p, size );
			MSetPtrSize := MemError;
		end else begin
			MSetPtrSize := -17022;
		end;
	end;

	function MSetHandleSize ( data: univ Handle; size: longint ): OSErr;
{$ifc do_debug}
		var
			oldsize: longint;
{$endc}
	begin
{$ifc do_debug}
		Assert( data <> nil );
		Assert( size >= 0 );
		oldsize := MGetHandleSize( data );
		Assert( MemError = noErr );
		if oldsize < size then begin
			SetHandleSize( data, size );
			if MemError = noErr then begin
				MTrash( AddPtrLong( data^, oldsize ), size - oldsize );
			end;
		end else if oldsize > size then begin
			MTrash( AddPtrLong( data^, size ), oldsize - size );
		end;
{$endc}
		if CheckHandle( data ) then begin
			SetHandleSize( data, size );
			MSetHandleSize := MemError;
		end else begin
			MSetHandleSize := -17023;
		end;
	end;

	function MGrowHandleSize ( data: univ Handle; size: longint ): OSErr;
{$ifc do_debug}
		var
			oldsize: longint;
{$endc}
	begin
{$ifc do_debug}
		Assert( data <> nil );
		Assert( size >= 0 );
		oldsize := MGetHandleSize( data );
		Assert( MemError = noErr );
		Assert( size >= oldsize );
{$endc}
		MGrowHandleSize := MSetHandleSize( data, size );
	end;
	
	procedure MShrinkHandleSize( data: univ Handle; size: longint );
{$ifc do_debug}
		var
			oldsize: longint;
{$endc}
		var
			junk: OSErr;
	begin
{$ifc do_debug}
		Assert( data <> nil );
		Assert( size >= 0 );
		oldsize := MGetHandleSize( data );
		Assert( MemError = noErr );
		Assert( size <= oldsize );
{$endc}
		junk := MSetHandleSize( data, size );
		Assert( junk = noErr );
	end;
	
	procedure MDisposePtr ( var p: univ Ptr );
	begin
		if (p <> nil) & CheckPtr( p ) then begin
			MTrashPtr( p );
			DisposePtr(p);
{$ifc do_debug_memory}
			debugDelPorH( p );
{$endc}
			p := nil;
		end;
	end;

	procedure MDisposeHandle ( var data: univ Handle );
	begin
		if (data <> nil) & CheckHandle( data ) then begin
			MTrashHandle( data );
			DisposeHandle( data );
{$ifc do_debug_memory}
			debugDelPorH( data );
{$endc}
			data := nil;
		end;
	end;

	procedure MZero (p: univ Ptr; size: longint);
	begin
		MFill( p, size, 0 );
	end;
	
	procedure MFill (p: univ Ptr; size: longint; val: integer);
		var
			i: UInt32;
	begin
		Assert( size >= 0 );
		if CheckPointer(p) then begin
			if size > 0 then begin { since i is unsigned, size-1 must be >= 0 }
				for i := 0 to size - 1 do begin
					AddPtrLong(p, i)^ := SignedByte(val);
				end;
			end;
		end;
	end;

	procedure MFillLong (p: univ Ptr; size: longint; val: longint);
{ Ptr and size must be long alligned }
		type
			longPtr = ^longint;
		var
			i: longint;
	begin
		Assert( size >= 0 );
		if CheckPointer(p) then begin
			Assert( (band(ord4(p), 3) = 0) & (band(size, 3) = 0) );
			i := longint(p);
			while size > 3 do begin
				longPtr(i)^ := val;
				i := i + 4;
				size := size - 4;
			end;
		end;
	end;

	procedure MLockHigh ( data: univ Handle );
	begin
		if CheckHandle( data ) then begin
			HLockHi( data );
		end;
	end;

	procedure HLockState( data: univ Handle; var waslocked: boolean );
	begin
		if CheckHandle( data ) then begin
			waslocked := BTST( HGetState(data), 7 );
			HLock(data);
		end;
	end;
	
	procedure HRestoreLockState( data: univ Handle; waslocked: boolean );
	begin
		if CheckHandle( data ) then begin
			if not waslocked then begin
				HUnlock( data );
			end;
		end;
	end;

	procedure HNoPurgeState ( data: univ Handle; var waspurgeable: boolean );
	begin
		if CheckHandle( data ) then begin
			waspurgeable := BTST( HGetState(data), 6 );
			HNoPurge(data);
		end;
	end;

	procedure HRestorePurgeState( data: univ Handle; waspurgeable: boolean );
	begin
		if CheckHandle( data ) then begin
			if waspurgeable then begin
				HPurge( data );
			end;
		end;
	end;
	
	procedure MHLock( data: univ Handle );
	begin
		if CheckHandle( data ) then begin
			HLock( data );
		end;
	end;

	procedure MHUnlock( data: univ Handle );
	begin
		if CheckHandle( data ) then begin
			HUnlock( data );
		end;
	end;

	procedure MHPurge( data: univ Handle );
	begin
		if CheckHandle( data ) then begin
			HPurge( data );
		end;
	end;

	procedure MHNoPurge( data: univ Handle );
	begin
		if CheckHandle( data ) then begin
			HNoPurge( data );
		end;
	end;

	function MGetHandleSize( data: univ Handle ): longint;
	begin
		MGetHandleSize := 0;
		if CheckHandle( data ) then begin
			MGetHandleSize := GetHandleSize( data );
		end;
	end;

	function MGetPtrSize( data: univ Ptr ): longint;
	begin
		MGetPtrSize := 0;
		if CheckPtr( data ) then begin
			MGetPtrSize := GetPtrSize( data );
		end;
	end;

	function MMungerFind( data: univ Handle; offset: longint; ptr1: univ Ptr; len1: longint ): longint;
	begin
		if CheckHandle( data ) then begin
			Assert( (len1 > 0) & (0 <= offset) & (offset <= MGetHandleSize( data ) ) );
			MMungerFind := Munger(data, offset, ptr1, len1, nil, 0);
		end else begin
			MMungerFind := -17024;
		end;
	end;
	
	function MMungerFindString( data: univ Handle; offset: longint; const s: string ): longint;
	begin
		MMungerFindString := MMungerFind( data, offset, @s[1], length(s) );
	end;
	
	function MMungerInsert( data: univ Handle; offset: longint; ptr2: univ Ptr; len2: longint ): OSErr;
		var
			junk_long: longint;
	begin
		if CheckHandle( data ) then begin
			Assert( (len2 >= 0) & (0 <= offset) & (offset <= MGetHandleSize( data ) ) );
			junk_long := Munger(data, offset, nil, 0, ptr2, len2);
			MMungerInsert := MemError;
		end else begin
			MMungerInsert := -17025;
		end;
	end;
	
	function MMungerInsertString( data: univ Handle; offset: longint; const s: string ): OSErr;
	begin
		MMungerInsertString := MMungerInsert( data, offset, @s[1], length(s) );
	end;
	
	procedure MMungerDelete( data: univ Handle; offset: longint; len1: longint);
		var
			junk_long: longint;
	begin
		if CheckHandle( data ) then begin
			Assert( (len1 >= 0) & (0 <= offset) & (offset + len1 <= MGetHandleSize( data ) ) );
			junk_long := Munger(data, offset, nil, len1, @junk_long, 0);
		end;
	end;
	
	function MHandleToHandle( source, dest: univ Handle ): OSErr;
		var
			err: OSErr;
			size: longint;
	begin
		err := -9986;
		if CheckHandle( source ) & CheckHandle( dest ) then begin
			size := MGetHandleSize( source );
			err := MSetHandleSize( dest, size );
			if err = noErr then begin
				BlockMoveData( source^, dest^, size );
			end;
		end;
		MHandleToHandle := err;
	end;
	
	function MAppendToHandle( data: univ Handle; p: univ Ptr; len: longint ): OSErr;
	begin
		Assert( (len >= 0) );
		MAppendToHandle := -9987;
		if CheckHandle( data ) & CheckPointer( p ) then begin
			MAppendToHandle := PtrAndHand( p, data, len );
		end;
	end;
	
	function MAppendString( data: univ Handle; const s: string ): OSErr;
	begin
		MAppendString := MAppendToHandle( data, @s[1], length(s) );
	end;
	
	function MNewIconSuite( var iconsuite: Handle ): OSErr;
		var
			err: OSErr;
	begin
		err := NewIconSuite( iconsuite );
		if err = noErr then begin
			Assert( iconsuite <> nil );
			MNoteNewHandle( iconsuite );
		end else begin
			iconsuite := nil;
		end;
		MNewIconSuite := err;
	end;
	
	function MAddIconToSuite( var data: Handle; iconsuite: Handle; typ: OSType ): OSErr;
		var
			err: OSErr;
	begin
		err := -9988;
		if CheckHandle( data ) then begin
			if CheckHandle( iconsuite ) then begin
				err := AddIconToSuite( data, iconsuite, typ );
				if err = noErr then begin
					MForgetMemory( data );
					data := nil;
				end;
			end;
			if err <> noErr then begin
				MDisposeHandle( data );
			end;
		end;
		MAddIconToSuite := err;
	end;
	
	function MAddIconDataToSuite( data: Ptr; siz: integer; iconsuite: Handle; typ: OSType ): OSErr;
		var
			err: OSErr;
			temphand: Handle;
	begin
		err := MNewHandleWith( temphand, data, siz );
		if err = noErr then begin
			err := MAddIconToSuite( temphand, iconsuite, typ );
		end;
		MAddIconDataToSuite := err;
	end;
	
	procedure MDisposeIconSuite( var iconsuite: Handle );
		var
			err: OSErr;
	begin
		if iconsuite <> nil then begin
			MForgetMemory( iconsuite );
			err := DisposeIconSuite( iconsuite, true );
			Assert( err = noErr );
			iconsuite := nil;
		end;
	end;
	
end.
	procedure HUnlockState ( data: univ Handle; var state: SignedByte );
	begin
		if CheckHandle( data ) then begin
			state := HGetState(data);
			HUnlock(data);
		end;
	end;

	procedure HPurgeState ( data: univ Handle; var state: SignedByte );
	begin
		if CheckHandle( data ) then begin
			state := HGetState(data);
			HPurge(data);
		end;
	end;

//	procedure HUnlockState ( data: univ Handle; var state: SignedByte );
//	procedure HPurgeState ( data: univ Handle; var state: SignedByte );
//	procedure HNoPurgeState ( data: univ Handle; var state: SignedByte );
//	procedure HRestoreState(hhhh: univ Handle; state: SignedByte);

