unit MyMenus;

interface

	uses
		MacOS;
		
	procedure SetIDItem (menu, item: integer; s: Str255);
	function GetIDItem (menu, item: integer): Str255;
	procedure SetItemEnable (mh: MenuHandle; item: integer; enable: Boolean);
	function GetItemEnable (mh: MenuHandle; item: integer): Boolean;
	procedure SetIDItemEnable (menu, item: integer; enable: Boolean);
	function GetIDItemEnable (menu, item: integer): Boolean;
	function AnyItemEnabled( mh: MenuHandle ): Boolean;
	function AnyIDItemEnabled( menu: integer ): Boolean;
	procedure DotItem (mh: MenuHandle; item: integer; dotted: Boolean);
	procedure ClearMenuHandle (mh: MenuHandle);
	procedure SafeAppendMenu (mh: MenuHandle; item: Str255);

implementation

	uses
		Balloons, 
		MyAssertions;

{$ifc not do_debug}
{$definec AssertValidMenuID(m)}
{$elsec}
{$definec AssertValidMenuID(m) AssertValidMenuIDCode(m)}
{$endc}

{$ifc do_debug}
	procedure AssertValidMenuIDCode( menu: integer );
	begin
		Assert( (menu = kHMHelpMenuID) | (menu > 0) );
	end;
{$endc}

	procedure ClearMenuHandle (mh: MenuHandle);
	begin
		Assert( mh <> nil );
		while (CountMItems(mh) > 0) do begin
			DeleteMenuItem(mh, 1);
		end;
	end;

	procedure SafeAppendMenu (mh: MenuHandle; item: Str255);
	begin
		Assert( mh <> nil );
		Assert( item <> '' );
		AppendMenu(mh, 'fred');
		SetMenuItemText(mh, CountMItems(mh), item);
	end;

	procedure SetIDItem (menu, item: integer; s: Str255);
	begin
		AssertValidMenuID( menu );
		Assert( (item > 0) & (s <> '') );
		SetMenuItemText(GetMenuHandle(menu), item, s);
	end;

	function GetIDItem (menu, item: integer): Str255;
		var
			s: Str255;
	begin
		AssertValidMenuID( menu );
		Assert( item > 0 );
		GetMenuItemText(GetMenuHandle(menu), item, s);
		GetIDItem := s;
	end;

	procedure SetItemEnable (mh: MenuHandle; item: integer; enable: Boolean);
	begin
		Assert( (mh <> nil) & (item >= 0) );
		if enable then begin
			EnableItem(mh, item);
		end else begin
			DisableItem(mh, item);
		end;
	end;

	procedure SetIDItemEnable (menu, item: integer; enable: Boolean);
	begin
		AssertValidMenuID( menu );
		Assert( item >= 0 );
		SetItemEnable(GetMenuHandle(menu), item, enable);
	end;

	function GetItemEnable (mh: MenuHandle; item: integer): Boolean;
	begin
		Assert( (mh <> nil) & (item >= 0) );
		GetItemEnable := (item > 31) | btst(mh^^.enableFlags, item);
	end;

	function GetIDItemEnable (menu, item: integer): Boolean;
	begin
		AssertValidMenuID( menu );
		Assert( item >= 0 );
		GetIDItemEnable := GetItemEnable(GetMenuHandle(menu), item);
	end;
	
	function AnyItemEnabled( mh: MenuHandle ): Boolean;
		var
			flags, count: longint;
	begin
		Assert( (mh <> nil) );
		flags := band( mh^^.enableFlags shr 1, $7FFFFFFF );
		count := CountMItems( mh );
		if count < 31 then begin
			flags := flags shl (32-count);
		end;
		AnyItemEnabled := (count > 31) | (flags <> 0);
	end;

	function AnyIDItemEnabled( menu: integer ): Boolean;
	begin
		AssertValidMenuID( menu );
		AnyIDItemEnabled := AnyItemEnabled( GetMenuHandle(menu) );
	end;

	procedure DotItem (mh: MenuHandle; item: integer; dotted: Boolean);
	begin
		Assert( (mh <> nil) & (item > 0) );
		if dotted then begin
			SetItemMark(mh, item, '');
		end else begin
			SetItemMark(mh, item, chr(0));
		end;
	end;

end.