 unit MyMovableModal;

{ This unit implements a MovableModalDialog routine similar to }
{ the Toolbox routine ModalDialog, to be used for movable modal dialogs }

{ Based on code by Marco Piovanelli <piovanel@dsi.unimi.it> }

interface

	uses
		MacOS;

{$ifc not defined do_movable_modal_passwords}
{$setc do_movable_modal_passwords := true}
{$endc}

	procedure EnterMovableModal( var movable_state: Ptr );
{$ifc do_movable_modal_passwords}
	procedure AddMovableEnterPassword( movable_state: Ptr; password_data: Ptr );
{$endc}
	procedure MovableModalDialog( movable_state: Ptr; filterProc: ProcPtr; var itemHit: integer );
	procedure ExitMovableModal( var movable_state: Ptr );

{ Call DisableMenuBar after the modal dialog is shown }
{
	GetNewDialog
	set up dialog
	ShowWindow(dialog);
	EnterMovableModal
	optionally AddMovableEnterPassword
	while not finished
		MovableModalDialog
	end-while
	ExitMovableModal
	DisposeDialog
}

implementation

	uses
		MyTypes, MyDialogs, MyCursors, MySystemGlobals, MyMenus, MyUtils, MyEvents, 
{$ifc do_movable_modal_passwords}
		MyEnterPassword,
{$endc}
		MyMemory, MyAssertions;

	const
		kSystemMenuThreshold = -16000;		{ menu IDs <= than this are used by the system }
		kMovableModalEventMask = mDownMask + mUpMask + keyDownMask + keyUpMask + autoKeyMask + updateMask + activMask + osMask;

{$PUSH}
{$ALIGN MAC68K}

	type
		MenuEntry = record
				hMenu: MenuHandle;
				leftEdge: Integer;
			end;

		MenuList = record
				offsetToLastMenu: Integer;
				rightmostEdge: Integer;
				unused: Integer;
				theMenus: array[0..1000] of MenuEntry;
			end;
		MenuListPtr = ^MenuList;
		MenuListHandle = ^MenuListPtr;

{$ALIGN RESET}
{$POP}

	const
		mm_magic = 'MMmg';
{$ifc do_movable_modal_passwords}
		max_passwords = 10;
{$endc}
		
	type
		MovableModalState = record
				magic: OSType;
				mbsBarEnable: LongInt;
				mbsEditEnable: LongInt;
{$ifc do_movable_modal_passwords}
				password_count: integer;
				passwords: array[1..max_passwords] of Ptr;
{$endc}
			end;
		MovableModalStatePtr = ^MovableModalState;

{$ifc not do_debug}
{$definec AssertValidMovableModalState(p)}
{$elsec}
{$definec AssertValidMovableModalState(p) AssertValidMovableModalStateCode(p)}
{$endc}

{$ifc do_debug}
	procedure AssertValidMovableModalStateCode( p: univ MovableModalStatePtr);
	begin
		Assert( (p <> nil) & (p^.magic = mm_magic) );
{$ifc do_movable_modal_passwords}
		Assert( (p^.password_count >= 0) & (p^.password_count <= max_passwords) );
{$endc}
	end;
{$endc}

	procedure SetEditMenu( theDialog: DialogPtr );
		var
			item, start, fin: integer;
	begin
		if theDialog <> nil then begin
			GetDialogTextSelection( theDialog, item, start, fin );
			if item > 0 then begin
				SetIDItemEnable( M_Edit, EMcut, start < fin );
				SetIDItemEnable( M_Edit, EMcopy, start < fin );
				SetIDItemEnable( M_Edit, EMpaste, true );
				SetIDItemEnable( M_Edit, EMclear, start < fin );
			end;
		end;
	end;

	procedure EnterMovableModal( var movable_state: Ptr );
		var
			menuList: MenuListHandle;
			i, nMenus: Integer;
			theMenu: MenuHandle;
			menuID: Integer;
			theDialog: DialogPtr;
			needEditMenu: Boolean;
			junk: OSErr;
			state: MovableModalStatePtr;
	begin
		theDialog := FrontWindow;
		needEditMenu := (theDialog <> nil) & (SelectedTextItem(theDialog) > 0);
		menuList := MenuListHandle(LMGetMenuList);
		nMenus := menuList^^.offsetToLastMenu div SizeOf(MenuEntry);

		junk := MNewPtr( movable_state, SizeOf(MovableModalState) );
		state := MovableModalStatePtr(movable_state);

		state^.magic := mm_magic;
{$ifc do_movable_modal_passwords}
		state^.password_count := 0;
{$endc}
		state^.mbsBarEnable := 0;
		for i := 0 to nMenus - 1 do begin
			theMenu := menuList^^.theMenus[i].hMenu;
			menuID := theMenu^^.menuID;

			if (menuID > kSystemMenuThreshold) then begin { do nothing if this is a system menu }

				if (menuID = M_Edit) then begin
					state^.mbsEditEnable := theMenu^^.enableFlags;
					if needEditMenu then begin
						theMenu^^.enableFlags := 1 + BSL(1, EMcut) + BSL(1, EMcopy) + BSL(1, EMpaste);
					end else begin
						DisableItem(theMenu, 0);
					end;
				end else begin { if this menu is enabled, disable it and set the corresponding bit  }
					if (btst(theMenu^^.enableFlags, 0)) then begin
						BSET(state^.mbsBarEnable, i);
						DisableItem(theMenu, 0);
					end;
				end;
			end;
		end;

		HiliteMenu(0);
		DrawMenuBar;
		CursorSetProcessing(false);
		CursorSetArrow;
		AssertValidMovableModalState( movable_state );
	end;

	procedure ExitMovableModal( var movable_state: Ptr );
		var
			menuList: MenuListHandle;
			i, nMenus: Integer;
			theMenu: MenuHandle;
			menuID: Integer;
			state: MovableModalStatePtr;
	begin
		AssertValidMovableModalState( movable_state );
		state := MovableModalStatePtr(movable_state);

		menuList := MenuListHandle(LMGetMenuList);
		nMenus := menuList^^.offsetToLastMenu div SizeOf(MenuEntry);

		for i := 0 to nMenus - 1 do begin
			theMenu := menuList^^.theMenus[i].hMenu;
			menuID := theMenu^^.menuID;

			if (menuID > kSystemMenuThreshold) then begin
				if (menuID = M_Edit) then begin
					theMenu^^.enableFlags := state^.mbsEditEnable;
				end else if (btst(state^.mbsBarEnable, i)) then begin
					EnableItem(theMenu, 0);
				end;
			end;
		end;

		state^.magic := '????';
		MDisposePtr( movable_state );
		DrawMenuBar;
	end;

{$ifc do_movable_modal_passwords}
	procedure AddMovableEnterPassword( movable_state: Ptr; password_data: Ptr );
		var
			state: MovableModalStatePtr;
	begin
		AssertValidMovableModalState( movable_state );
		state := MovableModalStatePtr(movable_state);
		Assert( state^.password_count < max_passwords );
		Inc(state^.password_count);
		state^.passwords[state^.password_count] := password_data;
	end;
{$endc}

	function DoMenuChoice ( state: MovableModalStatePtr; theDialog: DialogPtr; var theEvent: EventRecord; var itemHit: Integer; menuChoice: LongInt ): Boolean;
		var
			menuID, menuitem: Integer;
			currentEditField: Integer;
			junk: OSErr;
{$ifc do_movable_modal_passwords}
			password_index: integer;
			password_data: Ptr;
			offset: longint;
			data: Handle;
{$endc}
	begin
{$unused(theEvent)}
{$ifc not do_movable_modal_passwords}
{$unused(state)}
{$endc}
		DoMenuChoice := false;

		menuID := HiWord(menuChoice);
		menuitem := LoWord(menuChoice);

		if (menuID = M_Edit) then begin
			currentEditField := SelectedTextItem(theDialog);

			if (BAND(GetDItemKindF(theDialog, currentEditField), itemDisable) = 0) then begin { if the current edit field is an enabled item, exit from MovableModalDialog loop }
				DoMenuChoice := true;
				itemHit := currentEditField;
			end;
			
{$ifc do_movable_modal_passwords}
			password_data := nil;
			for password_index := 1 to state^.password_count do begin
				if IsEnterPassword( state^.passwords[password_index], theDialog, currentEditField ) then begin
					password_data := state^.passwords[password_index];
					leave;
				end;
			end;
			
			if password_data = nil then begin
{$endc}

				case menuitem of
					EMcut:  begin
						MyDialogCut(theDialog);
						junk := ZeroScrap;
						junk := TEToScrap;
					end;
					EMcopy: begin
						MyDialogCopy(theDialog);
						junk := ZeroScrap;
						junk := TEToScrap;
					end;
					EMpaste: begin
						junk := TEFromScrap;
						MyDialogPaste(theDialog);
					end;
					EMclear: begin
						MyDialogDelete(theDialog);
					end;
					otherwise begin
						{ do nothing }
					end;
				end;
			
{$ifc do_movable_modal_passwords}
			end else begin

				case menuitem of
					EMcut:  begin
						MyDialogCopy( theDialog );
						DeleteEnterPassword( password_data );
						junk := ZeroScrap;
						junk := TEToScrap;
					end;
					EMcopy: begin
						MyDialogCopy( theDialog );
						junk := ZeroScrap;
						junk := TEToScrap;
					end;
					EMpaste: begin
						junk := MNewHandle( data, 0 );
						if GetScrap( data, 'TEXT', offset ) > 0 then begin
							PasteEnterPassword( password_data, data );
						end;
						MDisposeHandle( data );
					end;
					EMclear: begin
						DeleteEnterPassword( password_data );
					end;
					otherwise begin
						{ do nothing }
					end;
				end;

			end;
{$endc}
		end;

		HiliteMenu(0);
	end;

	function HandleMouseDown ( state: MovableModalStatePtr; theDialog: DialogPtr; var theEvent: EventRecord; var itemHit: Integer): Boolean;
		var
			partCode: Integer;
			wind: WindowPtr;
			beeper: ProcPtr;
			dragRect: Rect;
	begin
		HandleMouseDown := false;

		partCode := FindWindow(theEvent.where, wind);

		if (partCode = inMenuBar) then begin
			SetEditMenu( theDialog );
			HandleMouseDown := DoMenuChoice( state, theDialog, theEvent, itemHit, MenuSelect(theEvent.where));
		end else if (not PtInRgn(theEvent.where, WindowPeek(theDialog)^.strucRgn)) then begin
			beeper := LMGetDABeeper;
			if (beeper <> nil) then begin
				CallSoundProc(1, beeper);
			end;
		end else if (partCode = inDrag) & (theDialog = wind) then begin { now, we have to Handle the only thing DialogSelect doesn't do for us: dragging }
			dragRect := GetGrayRgn^^.rgnBBox;
			DragWindow(wind, theEvent.where, dragRect);
			theEvent.what := nullEvent;
		end;
	end;

	procedure MovableModalDialog( movable_state: Ptr; filterProc: ProcPtr; var itemHit: integer );
		var
			dlg, junk_dialog: DialogPtr;
			er: EventRecord;
			dummy: Boolean;
			state: MovableModalStatePtr;
	begin
		AssertValidMovableModalState( movable_state );
		state := MovableModalStatePtr(movable_state);
		itemHit := 0;
		dlg := FrontWindow;
		if (dlg <> nil) then begin
			SetPort(dlg);
			SetEditMenu( dlg );
			repeat
				dummy := WaitNextEvent(kMovableModalEventMask, er, 0, nil);
				SetPort(dlg);
				SetEditMenu( dlg );

				if (filterProc <> nil) & CallModalFilterProc(dlg, er, itemHit, filterProc) then begin
					Leave;
				end;

				if er.what = kOSEvent then begin
					if EventIsSuspendResume( er ) then begin
						SetInForeground( EventHasResume( er ) );
					end;
				end;

				if (er.what = mouseDown) & HandleMouseDown( state, dlg, er, itemHit) then begin
					Leave;
				end;

				if EventIsKeyDown( er ) & EventHasCommandKey( er ) then begin
					SetEditMenu( dlg );
					if DoMenuChoice( state, dlg, er, itemHit, MenuKey( EventChar( er ) ) ) then begin
						Leave;
					end;
				end;

				if IsDialogEvent(er) & DialogSelect(er, junk_dialog, itemHit) then begin
					Leave;
				end;
			until false;
		end;

	end;

end.
	function HasHelpManager: Boolean;
		var
			response: LongInt;
	begin
		HasHelpManager := (Gestalt(gestaltHelpMgrAttr, response) = noErr) & btst(response, gestaltHelpMgrPresent);
	end;

