unit MyMovies;

interface

{ for music movies only }
{ SetMovieBox sets the display to empty }

	uses
		Files, Movies;
		
	var
		has_movies: boolean;
		
	procedure StartupMovies;
	procedure ConfigureMovies( patch_systask: boolean );
	
	function CreateTemporaryMovieFile( id: integer; var spec: FSSpec ): OSErr;
	function GetMovieFromID( var mov: Movie; id: integer ): OSErr;
	function GetMovieFromFile( var mov: Movie; const spec: FSSpec ): OSErr;
	procedure ReleaseMovie( var mov: Movie );
	
implementation

	uses
		Errors, Resources, Traps, 
		MyStartup, MyMemory, MyFileSystemUtils, MyUtils, MyResources, 
//		MySimpleProfile, 
		MyAssertions, MySystemGlobals, MyCallProc, PreserveA5;
	
{$ifc do_debug}
	var
		startup_check: integer;
{$endc}

	const
		MovieData_ResType = 'MDAT';
		MovieResource_ResType = 'moov';
	
	type
		MovieSpec = record
			id: integer;
			spec: FSSpec;
		end;
		MovieSpecArray = array[0..0] of MovieSpec;
		MovieSpecPtr = ^MovieSpecArray;
		MovieSpecHandle = ^MovieSpecPtr;
	
	type
		MovieArray = array[0..0] of Movie;
		MovieArrayPtr = ^MovieArray;
		MovieArrayHandle= ^MovieArrayPtr;
		
	var
		specs: MovieSpecHandle;
		movies: MovieArrayHandle;
		gPatchSysTask: boolean;
	
	function CountSpecs: integer;
	begin
		CountSpecs := MGetHandleSize( specs ) div SizeOf(MovieSpec);
	end;

	function CountMovies: integer;
	begin
		CountMovies := MGetHandleSize( movies ) div SizeOf(Movie);
	end;

	function GetMovieFromID( var mov: Movie; id: integer ): OSErr;
		var
			err: OSErr;
			spec: FSSpec;
	begin
		AssertDidStartup( startup_check );
		err := CreateTemporaryMovieFile( id, spec );
		if err = noErr then begin
			err := GetMovieFromFile( mov, spec );
		end;
		GetMovieFromID := err;
	end;
	
	function GetMovieFromFile( var mov: Movie; const spec: FSSpec ): OSErr;
		type
			BooleanPtr = ^Boolean;
		var
			err, err2: OSErr;
			rn: integer;
			empty: Rect;
	begin
		AssertDidStartup( startup_check );
		if not has_movies then begin
			err := -1;
		end else begin
			err := OpenMovieFile( spec, rn, fsRdPerm);
			if err = noErr then begin
				err := NewMovieFromFile( mov, rn, nil, nil, newMovieActive, nil );
				err2 := CloseMovieFile( rn );
			end;
			
			if err = noErr then begin
				GoToBeginningOfMovie( mov );	
				MoviesTask( mov, 0);
				SetRect( empty, 0, 0, 0, 0 );
				SetMovieBox( mov, empty );
				err2 := MAppendToHandle( movies, @mov, SizeOf(Movie) );
			end;
		end;
		GetMovieFromFile := err;
	end;
//	SetTimeBaseFlags( GetMovieTimeBase( theMovie ), loopTimeBase );
//	StartMovie(theMovie);
//	StopMovie(theMovie);
	
	procedure ReleaseMovie( var mov: Movie );
		var
			i: integer;
	begin
		AssertDidStartup( startup_check );
		Assert( mov <> nil );
		for i := 0 to CountMovies - 1 do begin
			if movies^^[i] = mov then begin
				MMungerDelete( movies, i * SizeOf(Movie), SizeOf(Movie) );
				leave;
			end;
		end;
		DisposeMovie( mov );
		mov := nil;
	end;
	
	function CreateTemporaryMovieFile( id: integer; var spec: FSSpec ): OSErr;
		var
			i: integer;
			junk, err, err2: OSErr;
			data: Handle;
			rn: integer;
	begin
		AssertDidStartup( startup_check );
		if not has_movies then begin
			err := -1;
		end else begin
			err := fnfErr;
			for i := 0 to CountSpecs - 1 do begin
				if specs^^[i].id = id then begin
					spec := specs^^[i].spec;
					err := noErr;
					leave;
				end;
			end;
			if err <> noErr then begin
				spec.name := 'Movie File ' + NumToStr( id );
				err := CreateTemporaryFile( spec );
				if err = noErr then begin
					
					// Data
					err := MGetResource( app_resfile, MovieData_ResType, id, data );
					if err = noErr then begin
						MHLock( data );
						err := FSpOpenDF( spec, fsRdWrPerm, rn );
						if err = noErr then begin
							AddOSErr( err, MyFSWrite( rn, MGetHandleSize( data ), data^ ) );
							AddOSErr( err, FSClose( rn ) );
						end;
						MReleaseResource( data );
					end;

					// Resource
					err := MGetResource( app_resfile, MovieResource_ResType, id, data );
					if err = noErr then begin
						HNoPurge( data );
						DetachResource( data );
						FSpCreateResFile( spec, '????', '????', 0 );
						rn := FSpOpenResFile( spec, fsRdWrPerm );
						if err = noErr then begin
							AddResource( data, MovieResource_ResType, 128, 'Movie' );
							WriteResource( data );
							CloseResFile( rn );
							data := nil;
						end else begin
							MDisposeHandle( data );
						end;
					end;
					
					if err = noErr then begin
						err2 := MGrowHandleSize( specs, MGetHandleSize( specs ) + SizeOf(MovieSpec) );
						if err2 = noErr then begin
							i := CountSpecs - 1;
							specs^^[i].id := id;
							specs^^[i].spec := spec;
						end;
					end else begin
						junk := FSpDelete( spec );
					end;
				end;
			end;
		end;
		CreateTemporaryMovieFile := err;
	end;
	
	var
		last_idle_time: UInt32;
		
	procedure IdleMovies;
		var
			i: integer;
	begin
		if TickCount > last_idle_time + 60 then begin
			last_idle_time := TickCount;
//			MarkProfile( 'Rest' );
			for i := 0 to CountMovies - 1 do begin
				MoviesTask( movies^^[i], 1 );
			end;
	//		MoviesTask( nil, 1 );
//			MarkProfile( 'IdleMovies' );
		end;
	end;
	
	var
		gOldSystemTaskAddr: UniversalProcPtr;
		
	procedure SystemTaskPatch;
		var
			a5: Ptr;
	begin
		a5 := SetPreservedA5;
		IdleMovies;
		CallPascal0( gOldSystemTaskAddr );
		RestoreA5( a5 );
	end;
	
	function InitMovies( var msg: integer ): OSStatus;
		var
			err: OSErr;
	begin
{$unused(msg)}
		AssertDidStartup( startup_check );
		has_movies := EnterMovies = noErr;
		last_idle_time := TickCount;
		err := MNewHandle( specs, 0 );
		err := MNewHandle( movies, 0 );
		if (err = noErr) & gPatchSysTask then begin
			gOldSystemTaskAddr := NGetTrapAddress( _SystemTask, ToolTrap );
			NSetTrapAddress( NewProc( @SystemTaskPatch, uppPascal0ProcInfo ), _SystemTask, ToolTrap );
		end;
		InitMovies := err;
	end;
	
	procedure FinishMovies;
		var
			junk: OSErr;
			i: integer;
	begin
		for i := 0 to CountMovies - 1 do begin
			DisposeMovie( movies^^[i] );
		end;
		MDisposeHandle( movies );

		if has_movies then begin
			ExitMovies;
		end;
		
		MHLock( specs );
		for i := 0 to CountSpecs - 1 do begin
			junk := FSpDelete( specs^^[i].spec );
		end;
		MDisposeHandle( specs );
	end;

	procedure StartupMovies;
	begin
//		StartupSimpleProfile;
		StartupPreserveA5;
		SetStartup( InitMovies, IdleMovies, 35, FinishMovies );
	end;
	
	procedure ConfigureMovies( patch_systask: boolean );
	begin
		DidStartup( startup_check );
		StartupMovies;
		gPatchSysTask := patch_systask;
	end;
	
end.
