unit MyNewPreferences;

interface

	uses
		MyCollections;

	var
		prefs: MyCollection;

	procedure InitNewPreferences( const pseudopath: string; fcreator: OSType);
	procedure FinishPreferences;
	procedure ZeroPrefsFile;
//	procedure DeletePrefsFile;
	function WritePrefsData: OSErr;
	procedure ReadPrefsData;
	procedure JustReadPrefsData;
	procedure ReadPrefsCollection (c: MyCollection; res_id: integer);
	function WritePrefsCollection (c: MyCollection; res_id: integer): OSErr;
	procedure ReadPrefsHandle (var hhhh: Handle; typ: ResType; id: integer);
	function WritePrefsHandle (hhhh: Handle; typ: ResType; id: integer; const name: string ): OSErr;
	procedure GetPrefsFSSpec( var spec: FSSpec );

implementation

	uses
		MyFileSystemUtils, MyFDFlags, MyMemory, MyMathUtils, MySystemGlobals,
		MyUtils, MyResources;

	const
		prefs_restype = 'PRFN';
		prefs_resid = 128;
		prefs_type = 'pref';

	var
		prefs_creator: OSType;
		prefs_mdate: UInt32;
		prefs_fs: FSSpec;

	procedure GetPrefsFSSpec( var spec: FSSpec );
	begin
		spec := prefs_fs;
	end;
(*	
	procedure DeletePrefsFile;
		var
			junk: OSErr;
	begin
		junk := FSpDelete( prefs_fs );
	end;
*)
	procedure ZeroPrefsFile;
	begin
		FSpRecreateResFile( prefs_fs, prefs_creator, prefs_type, kZapDF, kZapRF );
	end;

	procedure ReadPrefsHandle ( var hhhh: Handle; typ: ResType; id: integer );
		var
			junk: OSErr;
	begin
		junk := FSpGetResource( prefs_fs, typ, id, hhhh );
	end;

	procedure ReadPrefsCollection ( c: MyCollection; res_id: integer );
		var
			hhhh: Handle;
	begin
		ReadPrefsHandle(hhhh, prefs_restype, res_id);
		if hhhh <> nil then begin
			HackUpdateHandleToCollection(hhhh);
			c.SetDataHandle(hhhh);
		end;
	end;

	procedure JustReadPrefsData;
	begin
		ReadPrefsCollection(prefs, prefs_resid);
		prefs.safeget := true;
	end;

	function WritePrefsHandle (hhhh: Handle; typ: ResType; id: integer; const name: string ): OSErr;
		var
			err, junk: OSErr;
	begin
		FSpCreateResFile( prefs_fs, prefs_creator, prefs_type, 0 );
		err := FSpReplaceResource( prefs_fs, typ, id, name, hhhh );
		junk := FlushVol( nil, prefs_fs.vRefNum );
		WritePrefsHandle := err;
	end;

	function WritePrefsCollection (c: MyCollection; res_id: integer): OSErr;
		var
			hhhh: Handle;
	begin
		hhhh := c.GetDataHandle;
		WritePrefsCollection := WritePrefsHandle( hhhh, prefs_restype, res_id, 'prefs' );
	end;

	function WritePrefsData: OSErr;
	begin
		WritePrefsData := WritePrefsCollection(prefs, prefs_resid);
	end;

	procedure ReadPrefsData;
		var
			nmoddate: UInt32;
	begin
		MyGetModDate(prefs_fs, nmoddate);
		if nmoddate <> prefs_mdate then begin
			prefs_mdate := nmoddate;
			JustReadPrefsData;
		end;
	end;

	procedure FixPrefType;
		var
			fi: FInfo;
			err: OSErr;
	begin
		err := FSpGetFInfo( prefs_fs, fi );
		if (err = noErr) & (fi.fdCreator = prefs_creator) & (fi.fdType <> prefs_type) then begin
			fi.fdType := prefs_type;
			fi.fdFlags := BAND(fi.fdFlags, bnot(fdInited));
			err := FSpSetFInfo( prefs_fs, fi );
		end;
	end;

	procedure InitNewPreferences( const pseudopath: string; fcreator: OSType);
		var
			junk: OSErr;
//			name: Str255;
	begin
		if not system7 then begin
			ExitToShell;
		end;
		prefs_creator := fcreator;
		MNewObject( prefs );
		prefs.Create(0, false, true);
		prefs.safeget := true;

//		SetupPreferenceFolder;
//		SetupOurPreferenceFolder( strhId, folderindex );
//		GetIndString(name, strhId, fileindex);

		junk := PseudoMakeFSSpec( pseudopath, kCreateFolder, prefs_fs );
		Assert( (junk = noErr) | (junk = fnfErr) );

		FixPrefType;

		MyGetModDate(prefs_fs, prefs_mdate);
		JustReadPrefsData;
	end;

	procedure FinishPreferences;
	begin
		prefs.Destroy;
		prefs := nil;
	end;

end.
(*
	procedure SetDefaultLong (code: OSType; def: longint);
	begin
		if not prefs.ExistsTag(code) then begin
			prefs.SetTagLong(code, def);
		end;
	end;

		the_prefs_folder_vrn: integer;
		the_prefs_folder_dirID: longint;
		our_prefs_folder_vrn: integer;
		our_prefs_folder_dirID: longint;
	procedure SetupPreferenceFolder;
		var
			oe: OSErr;
			sysenv: SysEnvRec;
			name: Str255;
			dummy: longint;
			pb: CInfoPBRec;
	begin
		if not has_FindFolder 
				| (FindFolder(kOnSystemDisk, kPreferencesFolderType, kCreateFolder, the_prefs_folder_vrn, the_prefs_folder_dirID) <> noErr) then begin
			oe := SysEnvirons(1, sysenv);
			the_prefs_folder_vrn := sysenv.sysVRefNum;
			the_prefs_folder_dirID := 0;
			name := 'Preferences';
			oe := DirCreate(the_prefs_folder_vrn, the_prefs_folder_dirID, name, dummy);
			oe := MyGetCatInfo(the_prefs_folder_vrn, the_prefs_folder_dirID, name, 0, pb);
			if (oe = noErr) & (BAND(pb.ioFlAttrib, ioDirMask) <> 0) then begin
				the_prefs_folder_vrn := pb.ioVRefNum;
				the_prefs_folder_dirID := pb.ioDirID;
			end;
		end;
	end;

	procedure SetupOurPreferenceFolder(strhId, folderindex: integer);
		var
			oe: OSErr;
			name: Str255;
			dummy: longint;
			pb: CInfoPBRec;
	begin
		our_prefs_folder_vrn := the_prefs_folder_vrn;
		our_prefs_folder_dirID := the_prefs_folder_dirID;
		if folderindex > 0 then begin
			GetIndString(name, strhId, folderindex);
			oe := DirCreate(our_prefs_folder_vrn, our_prefs_folder_dirID, name, dummy);
			oe := MyGetCatInfo(our_prefs_folder_vrn, our_prefs_folder_dirID, name, 0, pb);
			if (oe = noErr) & (BAND(pb.ioFlAttrib, $10) <> 0) then begin
				our_prefs_folder_vrn := pb.ioVRefNum;
				our_prefs_folder_dirID := pb.ioDirID;
			end;
		end;
	end;

	procedure SetupPreferenceFolder;
	procedure SetupOurPreferenceFolder(strhId, folderindex: integer);

	function PutResource (hhhh: Handle; typ: ResType; id: integer): OSErr;
		var
			err: OSErr;
			old: Handle;
			xid: integer;
			xtyp: ResType;
			name: Str255;
	begin
		name := '';
		old := Get1Resource(typ, id);
		if old <> nil then begin
			GetResInfo(old, xid, xtyp, name);
			MRemoveResource(old);
			MDisposeHandle(old);
		end;
		MAddResource( hhhh, typ, id, name );
		err := ResError;
		if err = noErr then begin
			WriteResource(hhhh);
			err := ResError;
			MDetachResource(hhhh);
		end;
		PutResource := err;
	end;
	function PutResource (hhhh: Handle; typ: ResType; id: integer): OSErr; { Handle remains unchanged, and is detatched }

*)