unit MyOOAbout;

interface

	uses
		MacTypes;

	type
		GetAboutStringProc = function (index: integer): Str255;

	const
		kAboutStringLongVersion = 0;
		kAboutStringShortVersion = 1;
		kAboutStringName = 2;
		kAboutStringThanks = 3;

	procedure StartupAbout;
	procedure ConfigureAbout( get_about_string_proc: GetAboutStringProc );
	procedure UpdateAboutBox;
	procedure OpenAboutBox;
	procedure CloseAboutBox;
	function DefaultGetAboutString (index: integer): Str255;
	function DoAboutWithDelay( var wait_til: longint ): Boolean; // true for Register
	procedure FinishAboutDelay( wait_til: longint );
	function LaunchRegister: OSErr;

implementation

	uses
		MacMemory, Fonts, TextEdit, Resources, Icons,Quickdraw, MacWindows, TextUtils, Dialogs, AppleEvents,
		Events, Sound, 
		MyMenus, MyDialogs, MyStrings, MyVersionResource, MySystemGlobals, MyWindows, MyAssertions,
		MyFMenus, AERegistry, MyAEUtils, AEObjects, MyStrh, MyUtils, MyTypes, MyStartup, MyOOMenus, 
		MyRecordedMenuCommands, MyOOMainLoop, MyLowLevel, MyResources, MySystemGlobals, MyMathUtils, 
		MyRegions, MyProcesses, MyErrors, MyInternetConfig, MyEvents, MyAboutEntries, 
		BaseGlobals;

	const
		agwtAbout = 'About';

	const
		thanks_strh_id = 929;
		about_error_strh_id = 930;
		kThanksURL = 'thank:';
		kRegisterURL = 'register:';
		kSerialURL = 'serial:';
		kNotYetURL = 'notyet:';
		kICONStyledString = 'ICON:';

	type
		AboutObject = object(DObject)
				procedure Create (id: integer);
				override;
				procedure DoItem (item: integer);
				override;
				procedure DrawUserItem( item: integer );
				override;
				function TrackGroup( group:integer ):Boolean;
			end;

{$ifc do_debug}
	var
		startup_check: integer;
{$endc}

	var
		gGetAboutString: GetAboutStringProc;
		about_object: AboutObject;
		random_thanks_index: integer;

	procedure NewThanksIndex;
		var
			newrti: integer;
	begin
		if CountStrs(thanks_strh_id) <= 1 then begin
			random_thanks_index := 1;
		end else begin
			repeat
				newrti := BAND(Random, $7FFF) mod CountStrs(thanks_strh_id) + 1;
			until newrti <> random_thanks_index;
			random_thanks_index := newrti;
		end;
	end;

	function LaunchRegister: OSErr;
	begin
		LaunchRegister := LaunchAppWithHint( app_fs.vRefNum, app_fs.parID, 'Regi', 'APPL', true );
	end;

	function DefaultGetAboutString (index: integer): Str255;
		var
			s: Str255;
	begin
		case index of
			kAboutStringLongVersion: begin
				DefaultGetAboutString := version.longVersion;
			end;
			kAboutStringShortVersion: begin
				DefaultGetAboutString := version.shortVersion;
			end;
			kAboutStringName: begin
				DefaultGetAboutString := version.name;
			end;
			kAboutStringThanks: begin
				GetIndString(s, thanks_strh_id, random_thanks_index);
				DefaultGetAboutString := s;
			end;
			otherwise begin
{				Assert( false );}
				DefaultGetAboutString := '???';
			end;
		end;
	end;

	function AboutObject.TrackGroup( group:integer ):Boolean;
	begin
		TrackGroup := TrackAboutEntriesGroup( window, group );
	end;

	procedure AboutObject.DoItem (item: integer);
		var
			frame: Rect;
			entry: AboutDataEntry;
			err: OSErr;
	begin
		GetGroupEntry( item, entry );
		if entry.url = kThanksURL then begin
			NewThanksIndex;
			GetDItemRect( window, item, frame );
			SetPort( window );
			InvalRect( frame );
		end else begin
			if (GetDItemKindF( window, item ) = kButtonDialogItem) | TrackGroup( entry.group ) then begin
				if entry.url = kRegisterURL then begin
					err := LaunchRegister;
				end else begin
					err := MyLaunchURL( '', entry.url );
				end;
				if err <> noErr then begin
					DisplayErrorString( GetIndStr( about_error_strh_id, 1 ), err );
				end;
			end;
		end;
	end;

	procedure DrawDisplayItem( window: WindowPtr; item: integer; selected: Boolean );
		var
			s, t: Str255;
			i, n: integer;
			entry: AboutDataEntry;
			id: longint;
			frame: Rect;
	begin
		SetPort( window );
		GetAboutDataEntry( item, entry );
		Assert( entry.display <> '' );
		s := entry.display;
		if IsPrefix( s, kICONStyledString ) then begin
			Delete( s, 1, length(kICONStyledString) );
			StringToNum( s, id );
			GetDItemRect( window, item, frame );
			SafePlotCIcon( id, frame, selected );
		end else begin
			i := 1;
			while (i < length(s)) do begin
				if s[i] = '^' then begin
					n := ord(s[i+1])-48;
					if n>= 10 then begin
						n := n-7;
					end;
					t := gGetAboutString(n);
					Delete(s, i, 2);
					Insert(t, s, i);
					i := i + length(t);
				end else begin
					i := i + 1;
				end;
			end;
			DisplayStyledString( window, item, s, selected );
		end;
	end;

	procedure AboutObject.DrawUserItem( item: integer );
	begin
		DrawDisplayItem( window, item, false );
	end;

	procedure AboutObject.Create (id: integer);
		var
			s: Str255;
	begin
		UseResFile(app_resfile);
		inherited Create(id);
		SetPort(window);
		close_hides_window := true;
		AppleGuideWindowType := agwtAbout;
		SetMyFont(MFT_Geneva12);
		GetWTitle(window, s);
		SPrintS3(s, s, version.name, '', '');
		SetWTitle(window, s);
		HandleAllUserItems;
	end;

	procedure DoAbout;
	begin
		NewThanksIndex;
		if GetWindowVisible(about_object.window) then begin
			if FrontWindow <> about_object.window then begin
				SelectWindow(about_object.window);
			end;
		end else begin
			SelectWindow(about_object.window);
			ShowWindow(about_object.window);
		end;
	end;

	function AboutEnabled: Boolean;
	begin
		AboutEnabled := not IsWObjectFront(about_object);
	end;

	procedure UpdateAboutBox;
	begin
		AssertDidStartup( startup_check );
		SetPort(about_object.window);
		InvalRect(about_object.window^.portRect);
	end;

	procedure CloseAboutBox;
	begin
		AssertDidStartup( startup_check );
		about_object.DoClose;
	end;

	procedure OpenAboutBox;
	begin
		AssertDidStartup( startup_check );
		DoAbout;
		SetPort(about_object.window);
		DrawDialog(about_object.window);
		ValidRect(about_object.window^.portRect);
	end;

	const
		wait_time = 5 * second_in_ticks;
		max_about_time = 5 * minute_in_ticks;

	var
		start_about_time: longint;

	function MyModalFilter( dlg: DialogPtr; var event: EventRecord; var item: integer ): Boolean;
	begin
		MyModalFilter := false;
		if TickCount > start_about_time + wait_time then begin
			SetDControlEnable( dlg, FindAboutEntriesURL( kNotYetURL ), true );
		end;
		if TickCount > start_about_time + max_about_time then begin
			item := FindAboutEntriesURL( kNotYetURL );
			MyModalFilter := true;
		end else if EventIsKeyDown( event ) & EventHasOK( event ) then begin
			item := FindAboutEntriesURL( kSerialURL );
			MyModalFilter := true;
		end;
	end;

	function DoAboutWithDelay( var wait_til: longint ): Boolean; // true for Register
		var
			obj: AboutObject;
			item: integer;
			filter: UniversalProcPtr;
			junk: OSErr;
			entry: AboutDataEntry;
			psn: ProcessSerialNumber;
	begin
		NewThanksIndex;
		DoAboutWithDelay := false;
		start_about_time := TickCount;
		AssertDidStartup( startup_check );
		new(obj);
		obj.Create(about_id+1);
		CenterWindow( obj.window, qd.screenBits.bounds );
		filter := NewModalFilterProc( MyModalFilter );
		SetDControlEnable( obj.window, FindAboutEntriesURL( kNotYetURL ), false );
		junk := SetDialogDefaultItem( obj.window, FindAboutEntriesURL( kSerialURL ) );
		ShowWindow(obj.window);
		DrawDialog( obj.window );
		ValidRect(obj.window^.portRect);
		junk := GetCurrentProcess( psn );
		junk := SetFrontProcess( psn );
		while true do begin
			ModalDialog( filter, item );
			GetGroupEntry( item, entry );
			if entry.url = kThanksURL then begin
				obj.DoItem( item );
			end else if entry.url = kSerialURL then begin
				DoAboutWithDelay := true;
				leave;
			end else if entry.url = kNotYetURL then begin
				leave;
			end;
		end;
		DisposeRoutineDescriptor( filter );
		obj.Destroy;
		wait_til := start_about_time + wait_time;
	end;

	procedure FinishAboutDelay( wait_til: longint );
	begin
		if TickCount < wait_til then begin
			SysBeep( 1 );
			while TickCount < wait_til do begin
			end;
		end;
	end;

	function InitAbout(var msg: integer): OSStatus;
	begin
{$unused(msg)}
		AssertDidStartup( startup_check );
		random_thanks_index := -1;
		NewThanksIndex;
		SetRecordedMenuCommand( kAECoreSuite, kAEAbout, Cabout, AboutEnabled, DoAbout );
		if gGetAboutString = nil then begin
			gGetAboutString := DefaultGetAboutString;
		end;
		new(about_object);
		about_object.Create(about_id);
{$ifc do_debug}
		ValidateAboutEntryData( about_object.window );
{$endc}
		InitAbout := noErr;
	end;

	procedure ConfigureAbout( get_about_string_proc: GetAboutStringProc );
	begin
		StartupAbout;
		gGetAboutString := get_about_string_proc;
		DidStartup( startup_check );
	end;

	procedure StartupAbout;
	begin
		ConfigureAboutEntries( DrawDisplayItem );
		StartupDialogs;
		StartupOOMenus;
		StartupFMenus;
		StartupMainLoop;
		StartupOOMenus;
		StartupInternetConfig;
		StartupRecordedMenuCommands;
		SetStartup(InitAbout, nil, 0, nil);
	end;

end.
