unit MyOTUtils;

interface

	uses
		MacOS;

	function HasOT: boolean;

	function OTLFSndUData(ref: EndpointRef; var udata: TUnitData): OSStatus;
	function OTLFRcvUData(ref: EndpointRef; var udata: TUnitData; var flags: OTFlags): OSStatus;
	
	function MapStatusToErr( err: OSStatus ): OSErr;

	function MyOTCreateConfiguration( path: Str255 ): OTConfigurationPtr;
	function MyOTOpenEndpointSync( const path: Str255; var ep: EndpointRef ): OSStatus;
	function MyOTInitDNSAddress( var addr: DNSAddress; name: Str255 ): size_t;

implementation

	uses
		MyCStrings,
		GestaltEqu, CodeFragments;
		
	function HasOTLib:boolean;
	begin
{$IFC GENERATINGPOWERPC}
		HasOTLib := longint(@OTInstallNotifier) <> kUnresolvedCFragSymbolAddress;
{$ELSEC}
		HasOTLib := true;
{$ENDC}
	end;

	function HasOT: boolean;
		var
			gv: longint;
	begin
		HasOT := HasOTLib & (Gestalt(gestaltOpenTpt, gv) = noErr) & 
			btst(gv, gestaltOpenTptPresentBit) & btst(gv, gestaltOpenTptTCPPresentBit);
	end;

	function MapStatusToErr( err: OSStatus ): OSErr;
	begin
		if err < -32768 then begin
			err := -32768;
		end;
		if err > 32767 then begin
			err := 32767;
		end;
		MapStatusToErr := err;
	end;
	
	function OTLFSndUData(ref: EndpointRef; var udata: TUnitData): OSStatus;
		var
			err, junk: OSStatus;
			i: integer;
	begin
		for i := 1 to 20 do begin
			err := OTSndUData( ref, @udata );
			if err = kOTLookErr then begin
				junk := OTRcvUDErr( ref, nil );
			end else begin
				leave;
			end;
		end;
		OTLFSndUData := err;
	end;

	function OTLFRcvUData(ref: EndpointRef; var udata: TUnitData; var flags: OTFlags): OSStatus;
		var
			err, junk: OSStatus;
			i: integer;
	begin
		for i := 1 to 20 do begin
			err := OTRcvUData( ref, @udata, flags );
			if err = kOTLookErr then begin
				junk := OTRcvUDErr( ref, nil );
			end else begin
				leave;
			end;
		end;
		OTLFRcvUData := err;
	end;

	function MyOTCreateConfiguration( path: Str255 ): OTConfigurationPtr;
	begin
		P2C( @path );
		MyOTCreateConfiguration := OTCreateConfiguration( @path );
	end;
	
	function MyOTOpenEndpointSync( const path: Str255; var ep: EndpointRef ): OSStatus;
		var
			err: OSStatus;
	begin
		ep := OTOpenEndpoint( MyOTCreateConfiguration( path ), 0, nil, err );
		if (ep = nil) & (err = noErr) then begin
			err := -17028;
		end;
		if (err <> noErr) then begin
			ep := nil;
		end;
		MyOTOpenEndpointSync := err;
	end;
	
	function MyOTInitDNSAddress( var addr: DNSAddress; name: Str255 ): size_t;
	begin
		P2C( @name );
		MyOTInitDNSAddress := OTInitDNSAddress( addr, @name );
	end;
	
end.
