unit MyProcesses;

interface

	uses
		MacOS;

	const
		application = 'APPL';
		kApplicationToFront = true;
		kApplicationInBackground = false;
//		keyNull = 'NULL'; // Note: typeNull is 'null' (lower case), but Apple "owns" lowercase
		
	type
		ExtraParamRecord = record
			key: AEKeyword;
			desc: AEDesc;
			optional: Boolean;
		end;
		ExtraParamArray = array[0..0] of ExtraParamRecord;
		ExtraParamArrayPtr = ^ExtraParamArray;

	function FindApplication (creator: OSType; var fs: FSSpec): OSErr;
	function FindProcess (creator, typ: OSType; var process: ProcessSerialNumber; var fs: FSSpec): Boolean;
	function FindAppWithHint(vrn:integer; dirID:longint; creator, typ: OSType; var app:FSSpec; var psn: ProcessSerialNumber; var isrunning:Boolean):OSErr;
	function FindControlPanel (fcreator: OSType; var fs: FSSpec): OSErr;
	function FindInFolder(vrn:integer; dirID:longint; fcreator, ftype: OSType; var fs:FSSpec):OSErr;

	function IsRunning (creator, typ: OSType): Boolean;

	function LaunchWithDocument (creator, typ: OSType; fs: FSSpec; tofront: Boolean):OSErr;
	function LaunchWithDocumentWithExtraParam( creator, typ: OSType; fs: FSSpec; tofront: Boolean; extra_count: longint; extras: ExtraParamArrayPtr ):OSErr;
	function LaunchApp (creator, typ: OSType; tofront: Boolean):OSErr;
	function LaunchAppWithHint(vrn:integer; dirID:longint; creator, typ: OSType; tofront: Boolean):OSErr;
	function LaunchFSSpec (var fs: FSSpec; tofront: Boolean):OSErr;

	function SendQuitToApplication(process: ProcessSerialNumber):OSErr;
	procedure QuitApplication (creator, typ: OSType);

	function OpenControlPanel (fcreator: OSType): Boolean;
	function TellFinderToLaunch (fs: FSSpec; tofront: Boolean): Boolean;

implementation

	uses
		MyTypes, MySystemGlobals, MyFileSystemUtils, MyUtils, MyAEUtils, MyMemory;

	function FindInFolder(vrn:integer; dirID:longint; fcreator, ftype: OSType; var fs:FSSpec):OSErr;
		var
			err:OSErr;
			pb: HParamBlockRec;
			i: integer;
	begin
		fs.vRefNum := vrn;
		fs.parID := dirID;
		i := 1;
		repeat
			pb.ioNamePtr := @fs.name;
			pb.ioVRefNum := vrn;
			pb.ioDirID := dirID;
			pb.ioFDirIndex := i;
			i := i + 1;
			err := PBHGetFInfoSync(@pb);
			if err = noErr then begin
				if (pb.ioFlFndrInfo.fdCreator = fcreator) & (pb.ioFlFndrInfo.fdType = ftype) then begin
					leave;
				end;
			end;
		until (err <> noErr);
		FindInFolder := err;
	end;

	function FindControlPanel (fcreator: OSType; var fs: FSSpec): OSErr;
		var
			err: OSErr;
	begin
		err := MyFindFolder( kControlPanelFolderType, kDontCreateFolder, fs.vRefNum, fs.parID);
		if err = noErr then begin
			err := FindInFolder(fs.vRefNum, fs.parID, fcreator, 'cdev', fs);
		end;
		FindControlPanel := err;
	end;

	function TellFinderToLaunch (fs: FSSpec; tofront: Boolean): Boolean;
		var
			process: ProcessSerialNumber;
			err, junk: OSErr;
			targetAddress: AEDesc;
			fileList: AEDescList;
			theEvent, theReply: AppleEvent;
			sendmode: AESendMode;
			gv: longint;
			finder_fs: FSSpec;
	begin
		err := -17029;
		if (Gestalt(gestaltFinderAttr, gv) = noErr) & btst(gv, gestaltOSLCompliantFinder) then begin
			if FindProcess('MACS', 'FNDR', process, finder_fs) then begin
				AECreate(theEvent);
				AECreate(theReply);
				AECreate(fileList);
				AECreate(targetAddress);
				err := CreateProcessSerialNumberDesc(process, targetAddress);
				if err = noErr then begin
					err := AECreateAppleEvent(kCoreEventClass, kAEOpenDocuments, targetAddress, kAutoGenerateReturnID, kAnyTransactionID, theEvent);
				end;
				AEDestroy(targetAddress);
				if err = noErr then begin
					err := AECreateList(nil, 0, false, fileList);
				end;
				if err = noErr then begin
					AddFSSpecToAEList(fileList, 1, fs);
				end;
				if err = noErr then begin
					err := AEPutParamDesc(theEvent, keyDirectObject, fileList);
				end;
				if err = noErr then begin
					sendmode := kAENoReply;
					if not tofront then begin
						sendmode := sendmode + kAENeverInteract;
					end;
					err := AESend(theEvent, theReply, sendmode, kAEHighPriority, kNoTimeOut, nil, nil);
				end;
				AEDestroy(theEvent);
				AEDestroy(theReply);
				AEDestroy(fileList);
				if (err = noErr) & tofront then begin
					junk := SetFrontProcess(process);
				end;
			end;
		end;
		TellFinderToLaunch := err = noErr;
	end;

	function OpenControlPanel (fcreator: OSType): Boolean;
		var
			fs: FSSpec;
	begin
		OpenControlPanel := false;
		if FindControlPanel(fcreator, fs) = noErr then begin
			OpenControlPanel := TellFinderToLaunch(fs, true);
		end;
	end;

	function ConfirmApplicationExists (creator: OSType; var fs: FSSpec): OSErr;
		var
			err: OSErr;
			info: FInfo;
	begin
		err := HGetFInfo(fs.vRefNum, fs.parID, fs.name, info);
		if err = noErr then begin
			if (info.fdType <> application) or (info.fdCreator <> creator) then begin
				err := fnfErr;
			end; (* if *)
		end; (* if *)
		ConfirmApplicationExists := err;
	end;

	function FindApplication (creator: OSType; var fs: FSSpec): OSErr;
		var
			i: integer;
			pbdt: DTPBRec;
			crdate: longint;
			oe: OSErr;
			found: Boolean;
	begin
		found := false;
		if system7 then begin
			i := 1;
			repeat
				fs.vRefNum := 0;
				oe := GetVolInfo(fs.name, fs.vRefNum, i, crdate);
				i := i + 1;
				if oe = noErr then begin
					with pbdt do begin
						fs.name := '';
						ioNamePtr := @fs.name;
						ioVRefNum := fs.vRefNum;
						oe := PBDTGetPath(@pbdt);
						if oe = noErr then begin
							ioIndex := 0;
							ioFileCreator := creator;
							oe := PBDTGetAPPLSync(@pbdt);
							if oe = noErr then begin
								fs.parID := pbdt.ioAPPLParID;
								found := ConfirmApplicationExists(creator,fs)=noErr;
							end;
						end;
					end;
					oe := noErr;
				end;
			until found or (oe <> noErr);
		end;
		if found then begin
			oe := noErr;
		end else begin
			oe := fnfErr;
			fs.vRefNum := 0;
			fs.parID := 2;
			fs.name := '';
		end;
		FindApplication := oe;
	end;

	function FindProcess (creator, typ: OSType; var process: ProcessSerialNumber; var fs: FSSpec): Boolean;
		var
			info: ProcessInfoRec;
	begin
		FindProcess := false;
		if has_LaunchControl then begin
			process.highLongOfPSN := 0;
			process.lowLongOfPSN := kNoProcess;
			info.processInfoLength := sizeof(ProcessInfoRec);
			info.processName := nil;
			info.processAppSpec := @fs;
			while GetNextProcess(process) = noErr do begin
				if GetProcessInformation(process, info) = noErr then begin
					if (info.processType = longint(typ)) and (info.processSignature = creator) then begin
						FindProcess := true;
						leave;
					end;
				end;
			end;
		end;
	end;

	function IsRunning (creator, typ: OSType): Boolean;
		var
			process: ProcessSerialNumber;
			fs: FSSpec;
	begin
		IsRunning := FindProcess(creator, typ, process, fs);
	end;

	procedure PrepareToLaunch (var theEvent: AppleEvent; tofront: Boolean; var launchThis: LaunchParamBlockRec);
		var
			junk: OSErr;
			launchDesc: AEDesc;
	begin
		junk := AECoerceDesc(theEvent, typeAppParameters, launchDesc);
		MHLock( launchDesc.dataHandle );
		launchThis.launchAppParameters := AppParametersPtr(launchDesc.dataHandle^);
		launchThis.launchBlockID := extendedBlock;
		launchThis.launchEPBLength := extendedBlockLen;
		launchThis.launchFileFlags := 0;
		launchThis.launchControlFlags := launchContinue + launchNoFileFlags;
		if not tofront then begin
			launchThis.launchControlFlags := launchThis.launchControlFlags + launchDontSwitch;
		end;
	end;

	function LaunchApplicationOptionallyMinimum(var launchThis: LaunchParamBlockRec):OSErr;
		var
			err:OSErr;
	begin
		err := LaunchApplication(@launchThis);
		if err = memFullErr then begin
			launchThis.launchControlFlags := BOR(launchThis.launchControlFlags, launchUseMinimum);
			err := LaunchApplication(@launchThis);
		end;
		LaunchApplicationOptionallyMinimum:=err;
	end;

	function CreateOpenAppleEvent( const targetAddress: AEDesc; const spec: FSSpec; extra_count: longint; extras: ExtraParamArrayPtr; var theEvent: AppleEvent ): OSErr;
		var
			err: OSErr;
			fileList: AEDescList;
			i: integer;
			optionals: AEDescList;
	begin
		Assert( (extra_count = 0) | (extras <> nil) );
		err := noErr;
		AddOSErr( err, AECreateAppleEvent(kCoreEventClass, kAEOpenDocuments, targetAddress, kAutoGenerateReturnID, kAnyTransactionID, theEvent) );

		AddOSErr( err, AECreateList(nil, 0, false, fileList) );
		AddFSSpecToAEList(fileList, 1, spec);
		AddOSErr( err, AEPutParamDesc(theEvent, keyDirectObject, fileList) );
		AEDestroy(fileList);

		AECreate( optionals );
		for i := 1 to extra_count do begin
			AddOSErr( err, AEPutKeyDesc( theEvent, extras^[i-1].key, extras^[i-1].desc ) );
			AEDestroy( extras^[i-1].desc );
			if extras^[i-1].optional then begin
				if AEisNull( optionals ) then begin
					AddOSErr( err, AECreateList( nil, 0, false, optionals ) );
				end;
				AddOSErr( err, AEPutPtr( optionals, 0, typeKeyword, @extras^[i-1].key, SizeOf(extras^[i-1].key) ) );
			end;
		end;
		if not AEisNull( optionals ) then begin
			AddOSErr( err, AEPutAttributeDesc( theEvent, keyOptionalKeywordAttr, optionals ) );
		end;
		AEDestroy( optionals );

		CreateOpenAppleEvent := err;
	end;

	function LaunchWithDocumentWithExtraParam( creator, typ: OSType; fs: FSSpec; tofront: Boolean; extra_count: longint; extras: ExtraParamArrayPtr ):OSErr;
		var
			err: OSErr;
			psn: ProcessSerialNumber;
			targetAddress: AEDesc;
			theEvent, theReply: AppleEvent;
			app_fs: FSSpec;
			launchThis: LaunchParamBlockRec;
			junk: OSErr;
			sendmode: AESendMode;
			t, c: longint;
	begin
		err := unimpErr;
		PurgeSpace(t, c);
		if has_LaunchControl & (c > 4096) then begin
			if FindProcess(creator, typ, psn, app_fs) then begin
				junk := CreateProcessSerialNumberDesc(psn, targetAddress);
				junk := CreateOpenAppleEvent( targetAddress, fs, extra_count, extras, theEvent );
				AEDestroy(targetAddress);

				sendmode := kAENoReply;
				if not tofront then begin
					sendmode := sendmode + kAENeverInteract;
				end;
				AECreate( theReply );
				junk := AESend(theEvent, theReply, sendmode, kAEHighPriority, kNoTimeOut, nil, nil);
				AEDestroy(theEvent);
				AEDestroy(theReply);
				if tofront then begin
					err := SetFrontProcess(psn);
				end else begin
					err := noErr;
				end;
			end else begin
				err := FindApplication(creator, app_fs);
				if err = noErr then begin
					junk := CreateSignatureDesc(creator, targetAddress);
					junk := CreateOpenAppleEvent( targetAddress, fs, extra_count, extras, theEvent );
					AEDestroy(targetAddress);

					launchThis.launchAppSpec := @app_fs;
					PrepareToLaunch(theEvent, tofront, launchThis);
					err := LaunchApplicationOptionallyMinimum(launchThis);
					AEDestroy(theEvent);
				end;
			end;
		end;
		LaunchWithDocumentWithExtraParam := err;
	end;

	function LaunchWithDocument (creator, typ: OSType; fs: FSSpec; tofront: Boolean):OSErr;
		var
			null_desc: AEDesc;
	begin
		AECreate( null_desc );
		LaunchWithDocument := LaunchWithDocumentWithExtraParam( creator, typ, fs, tofront, 0, nil );
		AEDestroy( null_desc );
	end;

	function LaunchFSSpec (var fs: FSSpec; tofront: Boolean):OSErr;
		var
			err: OSErr;
			launchThis: LaunchParamBlockRec;
	begin
		err := unimpErr;
		if has_LaunchControl then begin
			launchThis.launchBlockID := extendedBlock;
			launchThis.launchEPBLength := extendedBlockLen;
			launchThis.launchFileFlags := 0;
			launchThis.launchControlFlags := launchContinue + launchNoFileFlags + launchUseMinimum;
			if not tofront then begin
				launchThis.launchControlFlags := launchThis.launchControlFlags + launchDontSwitch;
			end;
			launchThis.launchAppSpec := @fs;
			launchThis.launchAppParameters := nil;
			err := LaunchApplicationOptionallyMinimum(launchThis);
		end;
		LaunchFSSpec := err;
	end;

	function LaunchApp (creator, typ: OSType; tofront: Boolean):OSErr;
		var
			err: OSErr;
			psn: ProcessSerialNumber;
			app: FSSpec;
	begin
		err := unimpErr;
		if has_LaunchControl then begin
			if FindProcess(creator, typ, psn, app) then begin
				if tofront then begin
					err := SetFrontProcess(psn);
				end else begin
					err := noErr;
				end;
			end else begin
				err := FindApplication(creator, app);
				if err = noErr then begin
					err := LaunchFSSpec(app, tofront);
				end;
			end;
		end;
		LaunchApp := err;
	end;

	function FindAppWithHint(vrn:integer; dirID:longint; creator, typ: OSType; var app:FSSpec; var psn: ProcessSerialNumber; var isrunning:Boolean):OSErr;
		var
			err:OSErr;
	begin
		err := unimpErr;
		if has_LaunchControl then begin
			if FindProcess(creator, typ, psn, app) then begin
				isrunning := true;
				err := noErr;
			end else begin
				isrunning := false;
				err := FindInFolder(vrn, dirID, creator, typ, app);
				if err <> noErr then begin
					err := FindApplication(creator, app);
				end;
			end;
		end;
		FindAppWithHint := err;
	end;

	function LaunchAppWithHint(vrn:integer; dirID:longint; creator, typ: OSType; tofront: Boolean):OSErr;
		var
			err:OSErr;
			psn: ProcessSerialNumber;
			app: FSSpec;
			isrunning: Boolean;
	begin
		err := FindAppWithHint(vrn, dirID, creator, typ,  app, psn, isrunning);
		if err = noErr then begin
			if isrunning then begin
				if tofront then begin
					err := SetFrontProcess(psn);
				end;
			end else begin
				err := LaunchFSSpec(app, tofront);
			end;
		end;
		LaunchAppWithHint := err;
	end;

	function SendQuitToApplication(process: ProcessSerialNumber):OSErr;
		var
			err, junk: OSErr;
			targetAddress: AEAddressDesc;
			AEvent, AReply: AppleEvent;
	begin
		junk := CreateProcessSerialNumberDesc(process, targetAddress);
		junk := AECreateAppleEvent(kCoreEventClass, kAEQuitApplication, targetAddress, kAutoGenerateReturnID, kAnyTransactionID, AEvent);
		AEDestroy(targetAddress);
		err := AESend(AEvent, AReply, kAENoReply, kAEHighPriority, 5 * second_in_ticks, nil, nil);
		AEDestroy(AEvent);
		AEDestroy(AReply);
		SendQuitToApplication := err;
	end;

	procedure QuitApplication (creator, typ: OSType);
		var
			junk:OSErr;
			process: ProcessSerialNumber;
			fs: FSSpec;
	begin
		if FindProcess(creator, typ, process, fs) then begin
			junk:=SendQuitToApplication(process);
		end;
	end;

end.