unit MyRecordedMenuCommands;

interface

	uses
		MacOS;
		
	type
		RecordedEnabledProc = function:Boolean;
		RecordedActionProc = procedure;

	procedure StartupRecordedMenuCommands;
	procedure SetRecordedMenuCommand( class_id, event_id: AEEventID; command: OSType; enabled: RecordedEnabledProc; action: RecordedActionProc );

implementation

	uses
		MyMemory, MyStartup, MyAssertions, MyFMenus, MyAEUtils, MyMenus, MySystemGlobals;

	type
		EntryRecord = record
			class_id, event_id: AEEventID;
			command: OSType;
			enabled: RecordedEnabledProc;
			action: RecordedActionProc;
		end;
		EntryArray = array[1..10000] of EntryRecord;
		EntryPtr = ^EntryArray;
		EntryHandle = ^EntryPtr;

{$ifc do_debug}
	var
		startup_check: integer;
{$endc}

	var
		gEntries: EntryHandle;
		gEntriesCount: longint;
		gHandler: AEEventHandlerUPP;

	procedure FindCommand( command: OSType; var index: longint );
		var
			i: integer;
	begin
		Assert( gEntries <> nil );
		index := -1;
		for i := 1 to gEntriesCount do begin
			if gEntries^^[i].command = command then begin
				index := i;
				leave;
			end;
		end;
		Assert( index > 0 );
	end;

	function IsIndexEnabled( index: integer ): Boolean;
		var
			proc: RecordedEnabledProc;
	begin
		Assert( gEntries <> nil );
		proc := gEntries^^[index].enabled;
		if @proc <> nil then begin
			IsIndexEnabled := proc();
		end else begin
			IsIndexEnabled := true;
		end;
	end;

	procedure DoIndexAction( index: integer );
		var
			sigh: RecordedActionProc;
	begin
		Assert( gEntries <> nil );
		sigh := gEntries^^[index].action;
		sigh();
	end;

	function HandleIndexAction (var event, reply: AppleEvent; index: UInt32): OSErr;
	begin
{$unused(event, reply)}
		if IsIndexEnabled( index ) then begin
			DoIndexAction( index );
			HandleIndexAction := noErr;
		end else begin
			HandleIndexAction := -17030;
		end;
	end;

	procedure DoAction;
		var
			index: longint;
			command: OSType;
	begin
		Assert( gEntries <> nil );
		GetCommand( thefmenu, thefitem, command );
		FindCommand( command, index );
		if has_AppleEvents then begin
			SendSelfSimpleEvent( gEntries^^[index].class_id, gEntries^^[index].event_id );
		end else if IsIndexEnabled( index ) then begin
			DoIndexAction( index );
		end;
	end;

	procedure SetAction (themenu, theitem: integer);
		var
			index: longint;
			command: OSType;
	begin
		Assert( gEntries <> nil );
		GetCommand( themenu, theitem, command );
		FindCommand( command, index );
//		SetIDItemEnable(themenu, theitem, gEntries^^[index].enabled() );
		SetIDItemEnable(themenu, theitem, IsIndexEnabled( index ) )
	end;

	procedure SetRecordedMenuCommand( class_id, event_id: AEEventID; command: OSType; enabled: RecordedEnabledProc; action: RecordedActionProc );
		var
			err: OSErr;
			entry: EntryRecord;
	begin
		AssertDidStartup( startup_check );
		Assert( gEntries <> nil );
		entry.class_id := class_id;
		entry.event_id := event_id;
		entry.command := command;
		entry.enabled := enabled;
		entry.action := action;
		err := MAppendToHandle( gEntries, @entry, SizeOf(entry));
		if err = noErr then begin
			Inc(gEntriesCount);
			if has_AppleEvents then begin
				err := AEInstallEventHandler( class_id, event_id, gHandler, gEntriesCount, false );
			end;
			SetFBoth( command, DoAction, SetAction );
		end;
	end;

	function InitRecordedMenuCommands( var msg: integer ): OSStatus;
		var
			err: OSStatus;
	begin
{$unused(msg)}
		DidStartup( startup_check );
		err := MNewHandle( gEntries, 0 );
		gHandler := NewAEEventHandlerProc(HandleIndexAction);
		InitRecordedMenuCommands := noErr;
	end;
	
	procedure FinishRecordedMenuCommands;
	begin
		MDisposeHandle( gEntries );
	end;

	procedure StartupRecordedMenuCommands;
	begin
		SetStartup( InitRecordedMenuCommands, nil, 0, FinishRecordedMenuCommands );
	end;

end.
