unit MyResources;

interface

	uses
		MacOS;
		
	const
		kResChain = -13001;
		kCurrentResFile = -13002;

	function MGetResource( resfile: integer; typ: OSType; id: integer; var data: univ Handle ): OSErr;
	function MGetIndResource( resfile: integer; typ: OSType; index: integer; var data: univ Handle ): OSErr;
	function MCountResources( resfile: integer; typ: OSType; var count: integer ): OSErr;
	function MGetDetachedResource( resfile: integer; typ: OSType; id: integer; var data: univ Handle ): OSErr;
	function MGetIndDetachedResource( resfile: integer; typ: OSType; index: integer; var data: univ Handle ): OSErr;
	function MGetPicture( id: integer; var ph: PicHandle ): OSErr;

	function FSpGetResource( const spec: FSSpec; typ: OSType; id: integer; var data: univ Handle ): OSErr;
	function FSpGetIndResource( const spec: FSSpec; typ: OSType; index: integer; var data: univ Handle ): OSErr;

	function MReplaceResource( resfile: integer; typ: OSType; id: integer; const name: Str255; data: univ Handle ): OSErr;

	function FSpReplaceResource( const spec: FSSpec; typ: OSType; id: integer; const name: Str255; data: univ Handle ): OSErr;

	procedure MReleaseResource( var data: univ Handle );
	

implementation

	uses
		MyAssertions, MyUtils, MyMemory;

{ Damn Resource Manager is so broken! }
	function MGetResError( data: Handle ): OSErr;
		var
			err: OSErr;
	begin
		if data <> nil then begin
			err := noErr;
		end else begin
			err := ResError;
			if err = noErr then begin
				err := -17031;
			end;
		end;
		MGetResError := err;
	end;

	function MGetPicture( id: integer; var ph: PicHandle ): OSErr;
		var
			err: OSErr;
	begin
		err := noErr;
		ph := GetPicture( id );
		if (ph = nil) | (ph^ = nil) then begin
			err := MemError;
			if err = noErr then begin
				err := resNotFound;
			end;
			ph := nil;
		end;
		MGetPicture := err;
	end;

	function MCountResources( resfile: integer; typ: OSType; var count: integer ): OSErr;
		var
			err: OSErr;
			saved: integer;
	begin
		case resfile of
			kResChain: begin
				count := CountResources( typ );
				err := ResError;
			end;
			kCurrentResFile: begin
				count := Count1Resources( typ );
				err := ResError;
			end;
			otherwise begin
				saved := CurResFile;
				UseResFile( resfile );
				count := Count1Resources( typ );
				err := ResError;
				UseResFile( saved );
			end;
		end;
		if err <> noErr then begin
			count := 0;
		end;
		MCountResources := err;
	end;

	function MGetResource( resfile: integer; typ: OSType; id: integer; var data: univ Handle ): OSErr;
		var
			err: OSErr;
			saved: integer;
	begin
		case resfile of
			kResChain: begin
				data := GetResource( typ, id );
				err := MGetResError( data );
			end;
			kCurrentResFile: begin
				data := Get1Resource( typ, id );
				err := MGetResError( data );
			end;
			otherwise begin
				saved := CurResFile;
				UseResFile( resfile );
				data := Get1Resource( typ, id );
				err := MGetResError( data );
				UseResFile( saved );
			end;
		end;
		MGetResource := err;
	end;

	function MGetIndResource( resfile: integer; typ: OSType; index: integer; var data: univ Handle ): OSErr;
		var
			err: OSErr;
			saved: integer;
	begin
		case resfile of
			kResChain: begin
				data := GetIndResource( typ, index );
				err := MGetResError( data );
			end;
			kCurrentResFile: begin
				data := Get1IndResource( typ, index );
				err := MGetResError( data );
			end;
			otherwise begin
				saved := CurResFile;
				UseResFile( resfile );
				data := Get1IndResource( typ, index );
				err := MGetResError( data );
				UseResFile( saved );
			end;
		end;
		MGetIndResource := err;
	end;

	const
		kByIndex = true;
		kByID = false;
		
	function MGetMaybeDetachedResource( resfile: integer; typ: OSType; indexed: boolean; id: integer; var data: univ Handle ): OSErr;
		var
			err: OSErr;
	begin
		if indexed then begin
			err := MGetIndResource( resfile, typ, id, data );
		end else begin
			err := MGetResource( resfile, typ, id, data );
		end;
		if err = noErr then begin
			MHNoPurge( data );
			MDetachResource( data );
		end;
		MGetMaybeDetachedResource := err;
	end;

	function MGetDetachedResource( resfile: integer; typ: OSType; id: integer; var data: univ Handle ): OSErr;
	begin
		MGetDetachedResource := MGetMaybeDetachedResource( resfile, typ, kByID, id, data );
	end;

	function MGetIndDetachedResource( resfile: integer; typ: OSType; index: integer; var data: univ Handle ): OSErr;
	begin
		MGetIndDetachedResource := MGetMaybeDetachedResource( resfile, typ, kByIndex, index, data );
	end;

	function FSpGetMaybeResource( const spec: FSSpec; typ: OSType; indexed: boolean; id: integer; var data: univ Handle ): OSErr;
		var
			err: OSErr;
			savedresfile: integer;
			resfile: integer;
	begin
		data := nil;
		savedresfile := CurResFile;
		SetResLoad( false );
		resfile := FSpOpenResFile( spec, fsRdPerm );
		SetResLoad( true );
		if resfile <> -1 then begin
			err := MGetMaybeDetachedResource( resfile, typ, indexed, id, data );
			CloseResFile( resfile );
		end else begin
			err := ResError;
			AddOSErr( err, fnfErr );
		end;
		UseResFile( savedresfile );
		FSpGetMaybeResource := err;
	end;
	
	function FSpGetResource( const spec: FSSpec; typ: OSType; id: integer; var data: univ Handle ): OSErr;
	begin
		FSpGetResource := FSpGetMaybeResource( spec, typ, kByID, id, data );
	end;
	
	function FSpGetIndResource( const spec: FSSpec; typ: OSType; index: integer; var data: univ Handle ): OSErr;
	begin
		FSpGetIndResource := FSpGetMaybeResource( spec, typ, kByIndex, index, data );
	end;
	
	function MReplaceResource( resfile: integer; typ: OSType; id: integer; const name: Str255; data: univ Handle ): OSErr;
		var
			err: OSErr;
			saved: integer;
			old: Handle;
	begin
		Assert( resfile <> kResChain );
		saved := CurResFile;
		if resfile <> kCurrentResFile then begin
			UseResFile( resfile );
		end;
		old := Get1Resource( typ, id );
		if old <> nil then begin
			MRemoveResource( old );
			MDisposeHandle( old );
		end;
		MAddResource( data, typ, id, name );
		err := ResError;
		ChangedResource( data );
		AddOSErr( err, ResError );
		WriteResource( data );
		AddOSErr( err, ResError );
		MDetachResource( data );
		UseResFile( saved );
		MReplaceResource := err;
	end;

	function FSpReplaceResource( const spec: FSSpec; typ: OSType; id: integer; const name: Str255; data: univ Handle ): OSErr;
		var
			err: OSErr;
			savedresfile: integer;
			resfile: integer;
	begin
		savedresfile := CurResFile;
		SetResLoad( false );
		resfile := FSpOpenResFile( spec, fsRdWrPerm );
		SetResLoad( true );
		if resfile <> -1 then begin
			err := MReplaceResource( resfile, typ, id, name, data );
			CloseResFile( resfile );
			AddOSErr( err, ResError );
		end else begin
			err := ResError;
			AddOSErr( err, fnfErr );
		end;
		UseResFile( savedresfile );
		FSpReplaceResource := err;
	end;
	
	procedure MReleaseResource( var data: univ Handle );
	begin
		Assert( data <> nil );
		ReleaseResource( data );
		data := nil;
	end;
	
end.
