unit MySimpleProfile;

interface

	uses
		MacTypes;

	procedure StartupSimpleProfile;

	// You can call StartupSimpleProfile or alternatively call Init/Finish manually
	function InitSimpleProfile(var msg: integer): OSStatus;
	procedure FinishSimpleProfile;

	procedure ResetProfile;
	procedure MarkProfile(const mark:string);

implementation

	uses
		Files,Timer,ToolUtils, TextUtils, MacMemory, 
		MyMemory, MySystemGlobals, MyUtils, MyStartup;

	const
		do_simple_profile = true;
		
	const
		max_profile_marks = 20000;

	type
		MarkString = string[9];

	type
		ProfileMark=record
			time:longint;
			mark:MarkString;
		end;
		ProfileMarksArray=array[1..max_profile_marks] of ProfileMark;
		ProfileMarksArrayPtr=^ProfileMarksArray;

	var
		start_time: UnsignedWide;
		profile_mark:longint;
		profile_marks:ProfileMarksArrayPtr;

	procedure ResetProfile;
	begin
		profile_mark := 0;
	end;
	
	procedure MarkProfile(const mark:string);
		var
			current_time: UnsignedWide;
	begin
		if do_simple_profile & (profile_marks<>nil) & (profile_mark<max_profile_marks) then begin
			Microseconds(current_time);
			Inc(profile_mark);
			profile_marks^[profile_mark].mark := mark;
			profile_marks^[profile_mark].time := current_time.lo - start_time.lo;
		end;
	end;

	function InitSimpleProfile(var msg: integer): OSStatus;
	begin
{$unused(msg)}
		profile_mark := 0;
		InitSimpleProfile := MNewPtr(profile_marks, SizeOf(ProfileMarksArray));
	end;

	procedure FinishSimpleProfile;
		var
			rn:integer;
			data: Handle;
		procedure FlushHandle;
			var
				junk: OSErr;
				count: longint;
		begin
			count := GetHandleSize( data );
			HLock( data );
			junk := FSWrite(rn, count, data^);
			HUnlock( data );
			SetHandleSize( data, 0 );
		end;
		var
			fs:FSSpec;
			junk,err:OSErr;
			i:longint;
			s:Str255;
			lasttime,thistime:longint;
	begin
		fs := app_fs;
		fs.name := 'Simple Profile Dump';
		junk:=FSpDelete(fs);
		if profile_mark > 0 then begin
			err := FSpCreate(fs,'R*ch','TEXT',0);
			err := FSpOpenDF(fs,fsRdWrPerm,rn);
			if err=noErr then begin
				data := NewHandle( 0 );
				lasttime := profile_marks^[1].time;
				for i := 1 to profile_mark do begin
					s := profile_marks^[i].mark;
					thistime := profile_marks^[i].time;
					s := concat(s, chr(9), UNumToStr( thistime ), chr(9), NumToStr( thistime-lasttime ));
					lasttime := thistime;
					s := concat(s, chr(13));
					err := PtrAndHand( @s[1], data, length(s) );
					if GetHandleSize( data ) > 8192 then begin
						FlushHandle;
					end;
				end;
				FlushHandle;
				junk := FSClose(rn);
			end;
		end;
		MDisposePtr(profile_marks);
	end;

	procedure StartupSimpleProfile;
	begin
		SetStartup(InitSimpleProfile, nil, 0, FinishSimpleProfile);
	end;

end.
