unit MyStandardFile;

interface

	uses
		MySystemGlobals;

	type
		MyStandardGetFileReplyHandle = ^FSSpecArrayPtr;
		
		MyStandardPutFileReply = record
			good: boolean;
			spec: FSSpec;
		end;
		
{$ifc UseNavigation}
{$definec MyCompleteSave(complete_save_info,translate) MyCompleteSaveCode(complete_save_info,translate)}
{$elsec}
{$definec MyCompleteSave(complete_save_info,translate)}
{$endc}

	procedure MyStandardGetFile( vrn: integer; dirID: longint; numTypes: integer; typeList: SFTypeList; var reply: MyStandardGetFileReplyHandle );
	procedure MyStandardPutFile( vrn: integer; dirID: longint; const prompt, defaultName: Str255; fcreator, ftype: OSType; var reply: MyStandardPutFileReply; var complete_save_info: Ptr );

	procedure GetSFLocation (var vrn: integer; var dirID: longint);
	procedure SetSFLocation(vRefNum: integer; dirID: longint);

{$ifc UseNavigation}
	procedure MyCompleteSaveCode( var complete_save_info: Ptr; translate: Boolean );
	
	procedure OpenUselessNavPort( var port: GrafPtr ); // Stupid Nav Services 1.0 requires a port, sigh
	procedure CloseUselessNavPort( var port: GrafPtr );
	
	function NewUselessNavEventHandler: NavEventUPP;
{$endc}
	
implementation

	uses
		MacOS, 
{$ifc UseCursors}
		MyCursors, 
{$endc}
		MyCallProc, MyFileSystemUtils, MyMemory, MyAEUtils;

{$ifc UseNavigation}
	procedure OpenUselessNavPort( var port: GrafPtr ); // Stupid Nav Services 1.0 requires a port, sigh
	begin
		if FrontWindow <> nil then begin
			port := nil;
			SetPort( FrontWindow );
		end else if MNewPtr( port, SizeOf(GrafPort) ) = noErr then begin
			OpenPort( port );
			SetPort( port );
		end;
	end;
	
	procedure CloseUselessNavPort( var port: GrafPtr );
	begin
		if port <> nil then begin
			ClosePort( port );
			MDisposePtr( port );
		end;
		if FrontWindow <> nil then begin
			SetPort( FrontWindow );
		end;
	end;
	
	procedure UselessNavEventHandler( callBackSelector: NavEventCallbackMessage; callBackParms: NavCBRecPtr; callBackUD: NavCallBackUserData );
	begin
{$unused(callBackSelector,callBackParms,callBackUD)}
//		junkb := FrontObject.HandleEvents( callBackParms^.eventData.event^ );
		{ do nothing }
	end;

	function NewUselessNavEventHandler: NavEventUPP;
	begin
		NewUselessNavEventHandler := NavEventUPP(NewProc( @UselessNavEventHandler, uppNavEventProcInfo ));
	end;

{$endc}

	procedure GetSFLocation (var vrn: integer; var dirID: longint);
	begin
		vrn:= -LMGetSFSaveDisk;
		dirID:=LMGetCurDirStore;
	end;

	procedure SetSFLocation(vRefNum: integer; dirID: longint);
		var
			b21: Ptr;
			sysVersion: longint;
	begin
	{	from Mark Romano @ Symantec: System 7.5 has a low-memory global that		}
	{	controls Standard File. To force it to use SFSaveDisk/CurDirStore, clear bit 3.	}
		if (Gestalt(gestaltSystemVersion, sysVersion) = noErr) & (sysVersion >= $0750) then begin
			b21 := Pointer($0B21);
			b21^ := BAND(b21^, not($04));
		end;
		LMSetSFSaveDisk(-vRefNum);
		LMSetCurDirStore(dirID);
	end;

	function OldStandardGetFile( vrn: integer; dirID: longint; numTypes: integer; typeList: SFTypeList; var spec: FSSpec ): boolean;
		var
			reply: StandardFileReply;
	begin
		if vrn <> 0 then begin
			SetSFLocation( vrn, dirID );
		end;
		StandardGetFile( nil, numTypes, @typeList, reply );
		spec := reply.sfFile;
		OldStandardGetFile := reply.sfGood;
	end;
	
	function OldStandardPutFile( vrn: integer; dirID: longint; const prompt, defaultName: Str255; var spec: FSSpec ): boolean;
		var
			reply: StandardFileReply;
	begin
		if vrn <> 0 then begin
			SetSFLocation( vrn, dirID );
		end;
		StandardPutFile( prompt, defaultName, reply );
		spec := reply.sfFile;
		OldStandardPutFile := reply.sfGood;
	end;
	
{$ifc UseNavigation}
	procedure GetNavDefaultLocation( var defaultLocationp: AEDescPtr; vrn: integer; dirID: longint );
		var
			err: OSErr;
			spec: FSSpec;
	begin
		defaultLocationp := nil;
		if (vrn <> 0) & (GetDirFSSpec( vrn, dirID, spec ) = noErr) then begin
			err := MNewPtr( defaultLocationp, SizeOf(AEDesc) );
			if err = noErr then begin
				err := CreateFSSpecDesc( spec,  defaultLocationp^ );
				if err <> noErr then begin
					MDisposePtr( defaultLocationp );
				end;
			end;
		end;
	end;
	
	procedure DestroyNavDefaultLocation( var defaultLocationp: AEDescPtr );
	begin
		if defaultLocationp <> nil then begin
			AEDestroy( defaultLocationp^ );
			MDisposePtr( defaultLocationp );
		end;
	end;

	procedure NavStandardGetFile( vrn: integer; dirID: longint; numTypes: integer; typeList: SFTypeList; var specs: MyStandardGetFileReplyHandle );
		var
			theReply: NavReplyRecord;
			err, junk: OSErr;
			useless_event_handler: NavEventUPP;
			defaultLocationp: AEDescPtr;
			tlist: NavTypeListHandle;
			port: GrafPtr;
			len: longint;
			spec: FSSpec;
			index: longint;
			key: AEKeyword;
			typ: DescType;
	begin
		err := MNewHandle( specs, 0 );
		if err = noErr then begin
			OpenUselessNavPort( port );

			GetNavDefaultLocation( defaultLocationp, vrn, dirID );
			useless_event_handler := NewUselessNavEventHandler;
			if numTypes > 0 then begin
				Assert( SizeOf(NavTypeList) = 12 );
				if MNewHandle( tlist, SizeOf(NavTypeList) - SizeOf(OSType) + numTypes * SizeOf(NavTypeList)) = noErr then begin
					tlist^^.componentSignature := app_creator;
					tlist^^.reserved := 0;
					tlist^^.osTypeCount := numTypes;
					BlockMoveData( @typeList, @tlist^^.osType, numTypes * SizeOf(OSType) );
				end;
			end else begin
				tlist := nil;
			end;

	//		err := NavGetFile( nil, theReply, nil, nil, nil, nil, nil, 0 );
			err := NavGetFile( defaultLocationp, theReply, nil, useless_event_handler, nil, nil, tlist, nil );

			DisposeRoutineDescriptor( useless_event_handler );
			DestroyNavDefaultLocation( defaultLocationp );
			MDisposeHandle( tlist );
		
			if (err = noErr) & not theReply.validRecord then begin
				err := -17032;
			end;
			if err = noErr then begin
//WriteLineToTranscript( 'here' );
				index := 1;
				while true do begin
					err := AEGetNthPtr( theReply.selection, index, typeFSS, key, typ, @spec, SizeOf(spec), len);
//WriteLineToTranscript( NumToStr( index ) + ' ' + NumToStr(err) );
					if err <> noErr then begin
						err := noErr;
						leave;
					end;
					Inc(index);
					Assert( len = SizeOf(spec) );
					err := MAppendToHandle( specs, @spec, SizeOf(spec) );
//WriteLineToTranscript( NumToStr(err) );
				end;
//				err := GetFSSpecFromAEDesc( theReply.selection, spec );
				junk := NavDisposeReply( theReply );
			end;
		
			CloseUselessNavPort( port );
		end;
		if (err <> noErr) | (MGetHandleSize( specs ) = 0) then begin
			MDisposeHandle( specs );
		end;
	end;

	function NavStandardPutFile( vrn: integer; dirID: longint; const prompt, defaultName: Str255; fcreator, ftype: OSType; var spec: FSSpec; var complete_save_info: Ptr ): Boolean;
		var
			err, junk: OSErr;
			useless_event_handler: NavEventUPP;
			defaultLocationp: AEDescPtr;
			dialogOptions: NavDialogOptions;
			nrrp: NavReplyRecordPtr;
			port: GrafPtr;
	begin
		OpenUselessNavPort( port );
		junk := MNewPtr( nrrp, SizeOf(NavReplyRecord) );
		junk := NavGetDefaultDialogOptions( dialogOptions );
		dialogOptions.message := prompt;
		dialogOptions.savedFileName := defaultName;

		GetNavDefaultLocation( defaultLocationp, vrn, dirID );
		useless_event_handler := NewUselessNavEventHandler;
		err := NavPutFile( defaultLocationp, nrrp^, @dialogOptions, useless_event_handler, ftype, fcreator, nil );
		DisposeRoutineDescriptor( useless_event_handler );
		DestroyNavDefaultLocation( defaultLocationp );

		if (err = noErr) & not nrrp^.validRecord then begin
			err := -17033;
		end;
		if err = noErr then begin
			err := GetFSSpecFromAEDesc( nrrp^.selection, spec );
		end else begin
			MDisposePtr( nrrp );
		end;
		complete_save_info := Ptr(nrrp);
		
		CloseUselessNavPort( port );
		NavStandardPutFile := err = noErr;
	end;

{$endc}
	
	procedure MyStandardGetFile( vrn: integer; dirID: longint; numTypes: integer; typeList: SFTypeList; var reply: MyStandardGetFileReplyHandle );
		var
			spec: FSSpec;
			junk: OSErr;
	begin
{$ifc UseCursors}
		CursorSetArrow;
		CursorSetProcessing(false);
{$endc}
		if has_NavigationServices then begin
{$ifc UseNavigation}
			NavStandardGetFile( vrn, dirID, numTypes, typeList, reply );
{$endc}
		end else begin
			if OldStandardGetFile( vrn, dirID, numTypes, typeList, spec ) then begin
				junk := MNewHandleWith( reply, @spec, SizeOf(spec) );
				Assert( junk = noErr );
			end else begin
				reply := nil;
			end;
		end;
	end;

{$ifc UseNavigation}
	procedure MyCompleteSaveCode( var complete_save_info: Ptr; translate: Boolean );
		var
			err: OSErr;
			nrrp: NavReplyRecordPtr;
	begin
		if complete_save_info <> nil then begin
			nrrp := NavReplyRecordPtr(complete_save_info);
			if not translate then begin
				nrrp^.translationNeeded := false;
			end;
			err := NavCompleteSave( nrrp^, kNavTranslateInPlace );
			err := NavDisposeReply( nrrp^ );
			MDisposePtr( complete_save_info );
		end;
	end;
{$endc}
	
	procedure MyStandardPutFile( vrn: integer; dirID: longint; const prompt, defaultName: Str255; fcreator, ftype: OSType; var reply: MyStandardPutFileReply; var complete_save_info: Ptr );
	begin
{$ifc UseCursors}
		CursorSetArrow;
		CursorSetProcessing(false);
{$endc}
		if has_NavigationServices then begin
{$ifc UseNavigation}
			reply.good := NavStandardPutFile( vrn, dirID, prompt, defaultName, fcreator, ftype, reply.spec, complete_save_info );
{$elsec}
{$unused(fcreator, ftype)}
{$endc}
		end else begin
			reply.good := OldStandardPutFile( vrn, dirID, prompt, defaultName, reply.spec );
			complete_save_info := nil;
		end;
	end;

end.
		err := NavGetFile( nil, theReply, nil, nil, nil, nil, nil, 0 );
		if (err = noErr) & theReply.validRecord then begin
			junk := NavDisposeReply( theReply );
		end;

