unit MyStandardGetFolder;

interface

	uses
		MacOS;

	type
		MyStandardGetFolderReply = record
			good: boolean;
			vrn: integer;
			dirID: longint;
			folder_name: Str63;
		end;

	procedure MyStandardGetFolder( const message: Str255; var reply: MyStandardGetFolderReply);

implementation

	uses
		MySystemGlobals, MyCallProc, 
		MyStrings, MyFileSystemUtils, MyStandardFile, MyOOMainLoop, MyDialogs, MyAEUtils, MyAssertions;

	const
		rGetFolderButton = 10;
		rGetFolderMessage = 11;
		rGetFolderSelectString = 12;
		kFolderBit = $0010;
		rGetFolderDialog = 2008;

	type
		StandardFileReplyPtr = ^StandardFileReply;

	var
		gCurrentSelectedFolder: Str255;

	function MyCustomGetDirectoryFileFilter (pb: CInfoPBPtr; ignored: Ptr): Boolean;
	begin
{$unused(ignored)}
		MyCustomGetDirectoryFileFilter := BAND(pb^.ioFlAttrib, kFolderBit) = 0;
	end;

	function MyCustomGetDirectoryDlogHook (item: integer; theDialog: DialogPtr; mySFRPtr: StandardFileReplyPtr): integer;

		procedure SetButtonTitle (name: Str255);
			var
				resultCode: integer;
				width: integer;
				template, s: Str255;
				itemRect: Rect;
		begin
			if gCurrentSelectedFolder <> name then begin
				gCurrentSelectedFolder := name;
				GetItemText(theDialog, rGetFolderSelectString, template); { "Select ^1" template }
				GetDItemRect(theDialog, rGetFolderButton, itemRect);
				SPrintS3 (s,template,'','','');
				width := (itemRect.right - itemRect.left) - StringWidth(s);
				resultCode := TruncString(width, name, smTruncEnd);
				SPrintS3 (s,template,name,'','');
				SetDControlTitle(theDialog, rGetFolderButton, s);
				ValidRect(itemRect);
			end;
		end;

		procedure SetFolderButtonTitle (vrn: integer; dirID: longint);
			var
				name: Str63;
				pb: CInfoPBRec;
				err: OSErr;
		begin
			err := MyGetCatInfo(vrn, dirID, name, -1, pb);
			if err = noErr then begin
				SetButtonTitle(name);
			end;
		end;

		var
			wrefcon:longint;
	begin
		wrefcon:=GetWRefCon(theDialog);
		if OSType(wrefcon) = sfMainDialogRefCon then begin
			if item = sfHookFirstCall then begin
				SetItemText(theDialog, rGetFolderMessage, gCurrentSelectedFolder);
				gCurrentSelectedFolder := '';
			end else begin
				if mySFRPtr^.sfFile.name = '' then begin
					GetSFLocation(mySFRPtr^.sfFile.vRefNum, mySFRPtr^.sfFile.parID); { these aren't always set properly }
					SetFolderButtonTitle(mySFRPtr^.sfFile.vRefNum, mySFRPtr^.sfFile.parID);
				end else begin
					SetButtonTitle(mySFRPtr^.sfFile.name);
				end;
			end;

			if item = rGetFolderButton then begin
				item := sfItemCancelButton;
				mySFRPtr^.sfGood := true;
			end;

		end;
		MyCustomGetDirectoryDlogHook := item;
	end;

	function OldStandardGetFolder( const message: Str255; var spec: FSSpec ): Boolean;
		var
			MyCustomGetDirectoryFileFilterProc: FileFilterYDUPP;
			MyCustomGetDirectoryDlogHookProc: DlgHookYDUPP;
			mySFReply: StandardFileReply;
	begin
		gCurrentSelectedFolder := message;

		MyCustomGetDirectoryFileFilterProc := NewFileFilterYDProc(@MyCustomGetDirectoryFileFilter);
		MyCustomGetDirectoryDlogHookProc := NewDlgHookYDProc(@MyCustomGetDirectoryDlogHook);
		CustomGetFile(MyCustomGetDirectoryFileFilterProc, -1, nil, mySFReply, rGetFolderDialog, Point($FFFFFFFF), MyCustomGetDirectoryDlogHookProc, nil, nil, nil, @mySFReply);
		DisposeRoutineDescriptor(MyCustomGetDirectoryFileFilterProc);
		DisposeRoutineDescriptor(MyCustomGetDirectoryDlogHookProc);
		
		OldStandardGetFolder := mySFReply.sfGood;
		spec := mySFReply.sfFile;
	end;

{$ifc UseNavigation}

	function NavStandardGetFolder( const message: Str255; var spec: FSSpec ): Boolean;
		var
			theReply: NavReplyRecord;
			dialogOptions: NavDialogOptions;
			err, junk: OSErr;
			useless_event_handler: NavEventUPP;
			port: GrafPtr;
	begin
		OpenUselessNavPort( port );
		junk := NavGetDefaultDialogOptions( dialogOptions );
		dialogOptions.message := message;
	
		useless_event_handler := NewUselessNavEventHandler;
		err := NavChooseFolder( nil, theReply, @dialogOptions, useless_event_handler, nil, nil );
		DisposeRoutineDescriptor( useless_event_handler );
	
		if (err = noErr) & not theReply.validRecord then begin
			err := -17034;
		end;
		if err = noErr then begin
			err := GetFSSpecFromAEDesc( theReply.selection, spec );
			junk := NavDisposeReply( theReply );
		end;
		CloseUselessNavPort( port );
		NavStandardGetFolder := err = noErr;
	end;

{$endc}

	procedure MyStandardGetFolder( const message: Str255; var reply: MyStandardGetFolderReply );
		var
			spec: FSSpec;
			isfolder, wasaliased: Boolean;
			err: OSErr;
			pb: CInfoPBRec;
	begin
		if has_NavigationServices then begin
{$ifc UseNavigation}
			reply.good := NavStandardGetFolder( message, spec );
{$endc}
		end else begin
			reply.good := OldStandardGetFolder( message, spec );
		end;

		if reply.good then begin
			if spec.name <> '' then begin
				err := ResolveAliasFile( spec, true, isfolder, wasaliased );
				if (err = noErr) & not isfolder then begin
					err := dirNFErr;
				end;
				if err = noErr then begin
					err := FSpGetDirID( spec, reply.dirID );
				end;
				reply.vrn := spec.vRefNum;
				reply.good := err = noErr;
			end else begin
				reply.vrn := spec.vRefNum;
				reply.dirID := spec.parID;
				reply.good := true;
			end;
		end;

		if reply.good then begin
			reply.folder_name := '';
			reply.good := MyGetCatInfo( reply.vrn, reply.dirID, reply.folder_name, -1, pb ) = noErr;
		end;

	end;

end.