unit MyStartup;

interface

	uses
		MyAssertions;

{$ifc not do_debug}
{$definec DidStartup(b) if false then begin end else begin end }
{$definec AssertDidStartup(b) if false then begin end else begin end }
{$elsec}
{$definec DidStartup(b) DidStartupCode(b)}
{$definec AssertDidStartup(b) AssertDidStartupCode(b)}
{$endc}

	var
		current_time: longint;

	type
		StartupMessages = (SMT_None, SMT_Startup, SMT_Generic, SMT_NeedsSystem7, SMT_FailedToInitTCP, SMT_Rest);
		StartupInitProc = function(var msg: integer): OSStatus;
		StartupIdleProc = procedure;
		StartupFinishProc = procedure;

	procedure InitStartup;
	procedure SetStartup(init: StartupInitProc; idle: StartupIdleProc; idle_period: longint; finish: StartupFinishProc);
	function Startup(var msg: integer): OSStatus;
	procedure IdleStartup;
	procedure FinishStartup;
	procedure FireAtIdle(idle: StartupIdleProc);
{$ifc do_debug}
	procedure DidStartupCode( var check: integer );
	procedure AssertDidStartupCode( check: integer );
{$endc}

implementation

	uses
		MyMemory, MyUtils,
		MySystemGlobals;

	type
		EntryRecord = record
			init: StartupInitProc;
			idle: StartupIdleProc;
			idle_period: longint;
			next_idle: longint;
			finish: StartupFinishProc;
		end;
		EntryArray = array[1..10000] of EntryRecord;
		EntryPtr = ^EntryArray;
		EntryHandle = ^EntryPtr;

	const
		idles_max = 100;
		did_startup_check = $5748;

{$ifc do_debug}
	var
		startup_check: integer;
{$endc}

	var
		gIdles: array[1..idles_max] of StartupIdleProc;
		gMaxIdles: longint;
		gStartupError: OSStatus;
		gEntries: EntryHandle;
		gEntriesCount: longint;

{$ifc do_debug}
	procedure DidStartupCode( var check: integer );
	begin
		check := did_startup_check;
	end;

	procedure AssertDidStartupCode( check: integer );
	begin
		Assert( check = did_startup_check );
	end;
{$endc}

	procedure FireAtIdle(idle: StartupIdleProc);
		var
			i: longint;
			found: Boolean;
			hack: StartupIdleProc;
	begin
		found := false;
		for i := 1 to idles_max do begin
			hack := gIdles[i];
			if hack= nil then begin
				gIdles[i] := idle;
				if i > gMaxIdles then begin
					gMaxIdles := i;
				end;
				found := true;
				leave;
			end;
		end;
		if not found then begin
			idle();
		end;
	end;

	procedure SetStartup(init: StartupInitProc; idle: StartupIdleProc; idle_period: longint; finish: StartupFinishProc);
		var
			found: Boolean;
			entry: EntryRecord;
			i: longint;
	begin
		if (gStartupError = noErr) & (gEntries = nil) then begin
			gStartupError := MNewHandle(gEntries, 0);
		end;
		if gStartupError = noErr then begin
			found := false;
			for i := 1 to gEntriesCount do begin
				if (gEntries^^[i].init = init) & (gEntries^^[i].idle = idle) & (gEntries^^[i].idle_period = idle_period) & (gEntries^^[i].finish = finish) then begin
					found := true;
					leave;
				end;
			end;
			if not found then begin
				entry.init := init;
				entry.idle := idle;
				entry.idle_period := idle_period;
				entry.next_idle := TickCount;
				entry.finish := finish;
				gStartupError := MAppendToHandle( gEntries, @entry, SizeOf(entry));
				if gStartupError = noErr then begin
					Inc(gEntriesCount);
				end;
			end;
		end;
	end;

	procedure IdleStartup;
		var
			i: longint;
			tmp_hack: StartupIdleProc;
	begin
		current_time := TickCount;
		for i := 1 to gEntriesCount do begin
			tmp_hack := gEntries^^[i].idle;
			if (tmp_hack <> nil) & (current_time >= gEntries^^[i].next_idle) then begin
				gEntries^^[i].next_idle := current_time + gEntries^^[i].idle_period;
				tmp_hack;
			end;
		end;
		for i := 1 to gMaxIdles do begin
			tmp_hack := gIdles[i];
			if (tmp_hack<> nil) then begin
				tmp_hack();
				gIdles[i] := nil;
			end;
		end;
	end;

	procedure InitStartup;
		var
			i: longint;
	begin
		DidStartup( startup_check );
		gEntries := nil;
		gEntriesCount := 0;
		gStartupError := noErr;
		gMaxIdles := 0;
		for i := 1 to idles_max do begin
			gIdles[i] := nil;
		end;
	end;

	function Startup(var msg: integer): OSStatus;
		var
			i: longint;
			tmp_hack: StartupFinishProc;
			tmp_hack_init: StartupInitProc;
	begin
		AssertDidStartup( startup_check );
		msg := ord(SMT_Startup);
		i := 0;
		while (gStartupError = noErr) & (i < gEntriesCount) do begin
			i := i + 1;
			tmp_hack_init := gEntries^^[i].init;
			if tmp_hack_init <> nil then begin
				msg := ord(SMT_Generic);
				gStartupError := tmp_hack_init(msg);

// if gStartupError <> noErr then gStartupError := i;

			end;
		end;
		if gStartupError <> noErr then begin
			i := i - 1;
			while i > 0 do begin
				tmp_hack := gEntries^^[i].finish;
				if tmp_hack <> nil then begin
					tmp_hack;
				end;
				i := i - 1;
			end;
			MDisposeHandle(gEntries);
			gEntriesCount := 0;
		end;
		if gStartupError = noErr then begin
			msg := ord(SMT_None);
		end;
		Startup := gStartupError;
	end;

	procedure FinishStartup;
		var
			i: longint;
			tmp_hack: StartupFinishProc;
	begin
		AssertDidStartup( startup_check );
		if gEntries <> nil then begin
			i := gEntriesCount;
			while i > 0 do begin
				tmp_hack := gEntries^^[i].finish;
				if tmp_hack <> nil then begin
					tmp_hack;
				end;
				i := i - 1;
			end;
			MDisposeHandle(gEntries);
			gEntriesCount := 0;
		end;
	end;

end.
