unit MySystemGlobals;

interface

	uses
		BasePrefix, 
		MyVersionResource;

	var
		system7: Boolean;
		system8: Boolean;
	
{$ifc GENERATINGPOWERPC}
	var
		system85: Boolean;
{$elsec}
	const
		system85 = false;
{$endc}

	var
		system_version: longint;
		has_AppleEvents: Boolean;
		has_FindFolder: Boolean;
		has_AliasManager: Boolean;
		has_NewStandardFile: Boolean;
		has_HelpManager: Boolean;
		has_ColourQuickDraw: Boolean;
		has_LaunchControl: Boolean;
		has_VirtualMemory: Boolean;
		app_resfile: integer;
		app_fs: FSSpec;
		app_creator: OSType;
		in_foreground: Boolean;
		version: versionRecord;
		sound_manager_version: NumVersion;
		quicktime_version: NumVersion;

{$ifc GENERATINGPOWERPC}
	const
		has_PowerPC = true;
{$elsec}
	const
		has_PowerPC = false;
{$endc}

{$ifc UseAppearance}
	var
		has_Appearance: Boolean;
		appearance_version: longint;
{$elsec}
	const
		has_Appearance = false;
		appearance_version = 0;
{$endc}

{$ifc UseNavigation}
	var has_NavigationServices: Boolean;
{$elsec}
	const has_NavigationServices = false;
{$endc}

	procedure InitSystemGlobals;
	function RefNumToFSSpec(rn:integer; var fs:FSSpec):OSErr;
	procedure SetInForeground (fore: Boolean);
	function InForeground: Boolean;

implementation

{
	function TPbtst(value:longint; bit:integer):Boolean;
	begin
		TPbtst := btst(value, bit);
	end;
}
	procedure SetInForeground (fore: Boolean);
	begin
		in_foreground := fore;
	end;

	function InForeground: Boolean;
		var
			gv: longint;
			ourpsn, frontpsn: ProcessSerialNumber;
			front: Boolean;
	begin
		if (Gestalt(gestaltOSAttr, gv) = noErr) & (btst(gv, gestaltLaunchControl)) then begin
			if (GetCurrentProcess(ourpsn) = noErr) & (GetFrontProcess(frontpsn) = noErr) then begin
				if SameProcess(ourpsn, frontpsn, front) = noErr then begin
					in_foreground := front;
				end;
			end;
		end;
		InForeground := in_foreground;
	end;

	function RefNumToFSSpec(rn:integer; var fs:FSSpec):OSErr;
		var
			pb: FCBPBRec;
	begin
		pb.ioNamePtr := @fs.name;
		pb.ioVRefNum := 0;
		pb.ioRefNum := rn;
		pb.ioFCBIndx := 0;
		RefNumToFSSpec := PBGetFCBInfoSync(@pb);
		fs.vRefNum := pb.ioFCBVRefNum;
		fs.parID := pb.ioFCBParID;
	end;

{$ifc UseNavigation & not GENERATINGCFM}
	function NavServicesAvailable: Boolean; external;
{$endc}
{$ifc UseNavigation & GENERATINGCFM}
	function NavServicesAvailable: Boolean;
	begin
		NavServicesAvailable := (longint(@NavLibraryVersion) <> kUnresolvedCFragSymbolAddress);
	end;
{$endc}
{$ifc not UseNavigation}
	function NavServicesAvailable: Boolean;
	begin
		NavServicesAvailable := false;
	end;
{$endc}

	procedure InitSystemGlobals;
		var
			junk: OSErr;
			gv: longint;
			sysenv: SysEnvRec;
			fi: FInfo;
{$ifc UseNavigation}
			nonav: Str255;
{$endc}
	begin
		has_ColourQuickDraw := (SysEnvirons(1, sysenv) = noErr) & sysenv.hasColorQD; { Gestalt has a bug that causes hasColourQD to always be set }
		if (Gestalt(gestaltSystemVersion, system_version) <> noErr) then begin
			system_version := $0600;
		end;
		system7 := system_version >= $0700;
		system8 := system_version >= $0800;
{$ifc GENERATINGPOWERPC}
		system85 := system_version >= $0850;
{$endc}
		app_resfile := CurResFile;
		junk := RefNumToFSSpec(app_resfile, app_fs);
		junk := FSpGetFInfo( app_fs, fi );
		app_creator := fi.fdCreator;
		has_AppleEvents := (Gestalt(gestaltAppleEventsAttr, gv) = noErr) & (btst(gv, gestaltAppleEventsPresent));
		has_FindFolder := (Gestalt(gestaltFindFolderAttr, gv) = noErr) & (btst(gv, gestaltFindFolderPresent));
		has_NewStandardFile := (Gestalt(gestaltStandardFileAttr, gv) = noErr) & (btst(gv, gestaltStandardFile58));
		has_HelpManager := (Gestalt(gestaltHelpMgrAttr, gv) = noErr) & (btst(gv, gestaltHelpMgrPresent));
		has_AliasManager := (Gestalt(gestaltAliasMgrAttr, gv) = noErr) & (btst(gv, gestaltAliasMgrPresent));
		has_LaunchControl := (Gestalt(gestaltOSAttr, gv) = noErr) & (btst(gv, gestaltLaunchControl));
		has_VirtualMemory := (Gestalt(gestaltVMAttr, gv) = noErr) & (btst(gv, gestaltVMPresent));
		
{$ifc UseAppearance}
		has_Appearance := (Gestalt( gestaltAppearanceAttr, gv ) = noErr) & (btst(gv, gestaltAppearanceExists));
{$ifc GENERATINGPOWERPC}
		has_Appearance := has_Appearance & (longint(@IdleControls) <> kUnresolvedCFragSymbolAddress);
{$endc}
		if has_Appearance & (Gestalt( gestaltAppearanceVersion, gv ) = noErr) then begin
			appearance_version := gv;
		end else begin
			appearance_version := 0;
		end;
{$endc}

// has_Appearance := false;

{$ifc UseNavigation}
		GetIndString( nonav, 9990, 1 );
		if nonav = 'NO NAVIGATION SERVICES' then begin
			has_NavigationServices := false;
		end else begin
			has_NavigationServices := NavServicesAvailable() & NavServicesCanRun();
		end;
{$endc}

		sound_manager_version := SndSoundManagerVersion;
		if Gestalt( gestaltQuickTimeVersion, longint(quicktime_version) ) <> noErr then begin
			longint(quicktime_version) := 0;
		end;
		in_foreground := true;
		GetVersion(app_resfile, version);
	end;

end.
