unit MyTracking;

interface

	uses
		MacOS;

	type
		TrackingObject = object
			region: RgnHandle;
			window: WindowPtr;
			
			procedure Create( inwindow: WindowPtr );
			procedure Destroy; // disposes of self
			
			procedure AddRect( const r: Rect );
			procedure AddRegion( rgn: RgnHandle );
			procedure AddItem( item: integer );
			
			function Track: Boolean;
			procedure Draw( selected: Boolean );
				// Called only on transtition, called initially with true, called finally with false
				// Default is invert in hilite colour
			function ButtonStillDown: Boolean;
				// Override this if you want time or to abort the track
		end;
		
	type // Not recomended!
		IconIDTackingObject = object(TrackingObject)
			id: integer;
			frame: Rect;

			procedure CreateWithIconID( inwindow: WindowPtr; iconid: integer; const iconframe: Rect );
			procedure Draw( selected: Boolean );
			override;
		end;
		
	type
		IconSuiteTackingObject = object(TrackingObject)
			suite: Handle;
			frame: Rect;

			procedure CreateWithIconSuite( inwindow: WindowPtr; iconsuite: Handle; const iconframe: Rect );
			procedure Draw( selected: Boolean );
			override;
		end;
		
implementation

	uses
		MyRegions, MyDialogs, MyUtils, MyAssertions, MyMemory, MyMathUtils;
		
	procedure TrackingObject.Create( inwindow: WindowPtr );
	begin
		window := inwindow;
		region := MNewRgn;
	end;
	
	procedure TrackingObject.Destroy;
	begin
		MDisposeRgn( region );
		MDisposeObject( self );
	end;

	procedure TrackingObject.AddRect( const r: Rect );
	begin
		UnionRgnRect( region, r );
	end;
	
	procedure TrackingObject.AddRegion( rgn: RgnHandle );
	begin
		UnionRgn( region, rgn, region );
	end;
	
	procedure TrackingObject.AddItem( item: integer );
		var
			frame: Rect;
	begin
		GetDItemRect( window, item, frame );
		AddRect( frame );
	end;

	procedure TrackingObject.Draw( selected: Boolean );
	begin
{$unused(selected)}
		HiliteInvertRgn( region );
	end;
	
	function TrackingObject.ButtonStillDown: Boolean;
	begin
		ButtonStillDown := StillDown;
	end;
	
	function TrackingObject.Track: Boolean;
		var
			inside,newinside:Boolean;
			mouse:Point;
	begin
		Assert( not EmptyRgn( region ) );
		SetPort( window );
		inside := true;
		Draw( inside );
		while ButtonStillDown do begin
			GetMouse( mouse );
			newinside := PtInRgn( mouse, region );
			if newinside <> inside then begin
				inside := newinside;
				Draw( inside );
			end;
		end;
		if inside then begin
			Draw( false );
		end;
		Track := inside;
	end;
	
	procedure IconIDTackingObject.Draw( selected: Boolean );
		var
			err: OSErr;
	begin
		err := PlotIconID( frame, atNone, Choose( selected, ttSelected, ttNone), id );
		Assert( err = noErr );
	end;
	
	procedure IconIDTackingObject.CreateWithIconID( inwindow: WindowPtr; iconid: integer; const iconframe: Rect );
		var
			rgn: RgnHandle;
	begin
		Create( inwindow );
		SetPort( inwindow );
		id := iconid;
		frame := iconframe;
		rgn := NewRgn;
		Assert( rgn <> nil );
		if (IconIDToRgn( rgn, frame, atNone, id ) <> noErr)  | EmptyRgn( rgn ) then begin // Sometimes this fails, weird!
			Assert( false );
			RectRgn( rgn, frame );
		end;
		AddRegion( rgn );
		Assert( not EmptyRgn( region ) );
		DisposeRgn( rgn );
	end;
	
	procedure IconSuiteTackingObject.Draw( selected: Boolean );
		var
			err: OSErr;
	begin
		err := PlotIconSuite( frame, atNone, Choose( selected, ttSelected, ttNone), suite );
		Assert( err = noErr );
	end;
	
	procedure IconSuiteTackingObject.CreateWithIconSuite( inwindow: WindowPtr; iconsuite: Handle; const iconframe: Rect );
		var
			rgn: RgnHandle;
	begin
		Create( inwindow );
		SetPort( inwindow );
		suite := iconsuite;
		frame := iconframe;
		rgn := NewRgn;
		Assert( rgn <> nil );
		if (IconSuiteToRgn( rgn, frame, atNone, suite ) <> noErr)  | EmptyRgn( rgn ) then begin
//			Assert( false );
			RectRgn( rgn, frame );
		end;
		Assert( not EmptyRgn( rgn ) );
		AddRegion( rgn );
		DisposeRgn( rgn );
	end;
	
end.
