unit MyVersionResource;

interface

	uses
		MacOS;
		
{$PUSH}
{$ALIGN MAC68K}

	type
		versionRecord = packed record
				numericVersion: NumVersion;
				countryCode: integer;
				shortVersion: Str15;
				longVersion: Str255;
				name: Str63;
			end;

{$ALIGN RESET}
{$POP}

	procedure GetVersion (resfile: integer; var vers: versionRecord);
	function GetNumericVersion( resfile: integer ): longint;
	procedure SetVersionParamText (resfile: integer; c3: Str255);
	function NumericToShortVersion( version: UInt32 ): Str255;

implementation

	uses
		MyStrings, MyMemory, MyMathUtils, MyUtils, MyLowLevel, MyResources;

	function GetNumericVersion( resfile: integer ): longint;
		var
			versh: VersRecHndl;
	begin
		GetNumericVersion := 0;
		if MGetResource( resfile, 'vers', 1, versh ) = noErr then begin
			GetNumericVersion := longint(versh^^.numericVersion);
		end;
	end;

	procedure GetVersion (resfile: integer; var vers: versionRecord);
		var
			vh: Handle;
			p: integer;
			offset: longint;
	begin
		if MGetResource( resfile, 'vers', 1, vh ) <> noErr then begin
			longint(vers.numericVersion) := $00002000;
			vers.countryCode := 0;
			vers.shortVersion := '0.0.0';
			vers.longVersion := 'Unknown v0.0.0';
			vers.name := 'Unknown';
		end else begin
			MHLock( vh );
			BlockMoveData(vh^, @vers, Min( MGetHandleSize(vh), SizeOf(vers)));
			offset := SubPtrPtr( @vers.shortVersion, @vers ) + length(vers.shortVersion) + 1;
			BlockMoveData(AddPtrLong(vh^, offset), @vers.longVersion, Min( MGetHandleSize(vh) - offset, SizeOf(vers.longVersion) ));
			if length(vers.shortVersion) >= SizeOf(vers.shortVersion) then begin
				vers.shortVersion[0] := chr(SizeOf(vers.shortVersion) - 1);
			end;
			MReleaseResource( vh );
		end;
		p := Pos(vers.shortVersion, vers.longVersion);
		while (p > 0) & (vers.longVersion[p] <> ' ') do begin
			p := p - 1;
		end;
		p := p - 1;
		if p < 1 then begin
			p := 255;
		end;
		vers.name := TPcopy(vers.longVersion, 1, p);
	end;

	procedure SetVersionParamText (resfile: integer; c3: Str255);
		var
			vers: versionRecord;
	begin
		GetVersion(resfile, vers);
		ParamText(vers.shortVersion, vers.longVersion, vers.name, c3);
	end;
	
	function NumericToShortVersion( version: UInt32 ): Str255;
		var
			ver:NumVersion;
			result: Str255;
	begin
		ver := NumVersion(version);
		result:=concat(NumToStr(ver.majorRev),'.',
							NumToStr(BAND(ver.minorAndBugRev div 16,$0F)),'.',
							NumToStr(BAND(ver.minorAndBugRev,$0F))
							);
		if (ver.stage<>$80) or (ver.nonRelRev<>0) then begin
			case ver.stage of 
				$20:begin
					result:=concat(result,'d');
				end;
				$40:begin
					result:=concat(result,'a');
				end;
				$60:begin
					result:=concat(result,'b');
				end;
				$80:begin
					result:=concat(result,'f');
				end;
				otherwise begin
					result:=concat(result,'<',NumToStr(ver.stage),'>');
				end;
			end;
			if ver.nonRelRev <> 0 then begin
				result:=concat(result,NumToStr(ver.nonRelRev));
			end;
		end;
		NumericToShortVersion := result;
	end;

end.