unit MyWindowsMenu;

interface

	procedure StartupWindowsMenu;
	procedure UpdateWindowsMenu;
	function DoWindowsMenuItem (themenu, theitem: integer): Boolean;

implementation

	uses
		MyFMenus, MyTypes, MyAssertions, MyStartup, MyMenus, MyOOMainLoop, MyStrings;

	var
		windows_menu: MenuHandle;
		gWindowMenuSkipItems, gWindowMenuBaseItem: integer;
		gWindowMenuWasEnabled: Boolean;

{$ifc do_debug}
	var
		startup_check: integer;
{$endc}

	function GetNextWindow (var wp: WindowPeek): Boolean;
	begin
		if wp = nil then begin
			wp := WindowPeek(FrontWindow);
		end else begin
			wp := wp^.nextWindow;
		end;
		while (wp <> nil) & (not wp^.visible | (GetWType(WindowPtr(wp)) = WT_NotMine)) do begin
			wp := wp^.nextWindow;
		end;
		GetNextWindow := wp <> nil;
	end;

	procedure SetEnableMenu (on: Boolean);
	begin
		if (gWindowMenuSkipItems > 0) then begin
			SetFMenu(M_Windows);
			if (BAND(windows_menu^^.enableFlags, BSL(1, gWindowMenuSkipItems) - 1) <> 0) then begin
				on := true;
			end;
		end;
		if gWindowMenuWasEnabled <> on then begin
			gWindowMenuWasEnabled := on;
			SetItemEnable(windows_menu, 0, on);
			DrawMenuBar;
		end;
	end;

	procedure UpdateWindowsMenu;
		var
			wp: WindowPeek;
			title: Str255;
			i: integer;
			first: Boolean;
	begin
		AssertDidStartup( startup_check );
		Assert( windows_menu <> nil );
		for i := CountMItems(windows_menu) downto gWindowMenuSkipItems + 1 do begin
			DeleteMenuItem(windows_menu, i);
		end;
		wp := nil;
		first := true;
		while GetNextWindow(wp) do begin
			if first and (gWindowMenuSkipItems > 0) then begin
				AppendMenu(windows_menu, '(-');
			end;
			GetWTitle(WindowPtr(wp), title);
			LimitStringLength(title, 40, '');
			if title = '' then begin
				title := '?';
			end;
			SafeAppendMenu(windows_menu, title);
			if first & (WindowPtr(wp) = FrontWindow) then begin
				SetItemMark(windows_menu, gWindowMenuBaseItem, chr(checkMark));
			end;
			first := false;
		end;
		SetEnableMenu(not first);
	end;

	function DoWindowsMenuItem (themenu, theitem: integer): Boolean;
		var
			wp: WindowPeek;
	begin
		AssertDidStartup( startup_check );
		DoWindowsMenuItem := false;
		if (themenu = M_Windows) and (theitem >= gWindowMenuBaseItem) then begin
			DoWindowsMenuItem := true;
			wp := nil;
			while (theitem >= gWindowMenuBaseItem) do begin
				if not GetNextWindow(wp) then begin
					leave;
				end;
				theitem := theitem - 1;
			end;
			if wp <> nil then begin
				SelectWindow(WindowPtr(wp));
			end;
			HiliteMenu(0);
		end;
	end;

	procedure SetSendToBack(themenu,theitem:integer);
		var
			wp:WindowPeek;
	begin
		wp:=nil;
		SetIDItemEnable(themenu, theitem, GetNextWindow(wp) & GetNextWindow(wp)); { At least two windows }
	end;

	procedure DoSendToBack;
		var
			wp,first, last:WindowPeek;
	begin
		wp:=nil;
		if GetNextWindow(wp) then begin
			first := wp;
			last:=nil;
			while GetNextWindow(wp) do begin
				last := wp;
			end;
			if last<>nil then begin
				SendBehind(WindowPtr(first),WindowPtr(last));
			end;
		end;
	end;

	function InitWindowsMenu( var msg: integer ): OSStatus;
	begin
{$unused(msg)}
		DidStartup( startup_check );
		windows_menu := GetFMenu(M_Windows);
		Assert( windows_menu <> nil );
		InsertMenu(windows_menu, 0);
		gWindowMenuSkipItems := CountMItems(windows_menu);
		gWindowMenuBaseItem := gWindowMenuSkipItems + 1 + ord(gWindowMenuSkipItems > 0);
		gWindowMenuWasEnabled := true;
		UpdateWindowsMenu;
		SetFBoth('stbk', DoSendToBack, SetSendToBack);
		InitWindowsMenu := noErr;
	end;

	procedure StartupWindowsMenu;
	begin
		StartupFMenus;
		SetStartup( InitWindowsMenu, nil, 0, nil );
	end;

end.