unit LZRW1KH;

interface

	uses
		Types;

	const
		BufferMaxSize = 32768;
		BufferMax = BufferMaxSize - 1;
		FLAG_Copied = $80;
		FLAG_Compress = $40;

	type
		BufferArray = packed array[0..BufferMax] of UInt8;
		BufferPtr = ^BufferArray;

	function LZRW1KHCompress (Source, Dest: BufferPtr; SourceSize: longint): longint;
	function LZRW1KHDecompress (Source, Dest: BufferPtr; SourceSize: longint): longint;

implementation

	uses
		Memory,
		MyUtils;

	type
		BufferIndex = longint;
		HashTable = array[0..4095] of INTEGER;

	function LZRW1KHCompress (Source, Dest: BufferPtr; SourceSize: longint): longint;
		var
			Hash: HashTable;

		function GetMatch (X: BufferIndex; var Size: longint; var Pos: BufferIndex): BOOLEAN;
			var
				HashValue: longint;
		begin
			HashValue := BAND(BSR(40543 * BXOR(BSL(BXOR(BSL(Source^[X], 4), Source^[X + 1]), 4), Source^[X + 2]), 4), $0FFF);

			GetMatch := FALSE;
			if (Hash[HashValue] <> -1) and (X - Hash[HashValue] < 4096) then begin
				Pos := Hash[HashValue];
				Size := 0;
				while ((Size < 18) & (Source^[X + Size] = Source^[Pos + Size]) & (X + Size < SourceSize)) do begin
					Size := Size + 1;
				end;
				GetMatch := (Size >= 3)
			end;
			Hash[HashValue] := X
		end;

		var
			Key, Bit, Command, Size: longint;
			X, Y, Z, Pos: BufferIndex;
	begin
		for Key := 0 to 4095 do begin
			Hash[Key] := -1;
		end;
		Dest^[0] := FLAG_Compress;
		X := 0;
		Y := 3;
		Z := 1;
		Bit := 0;
		Command := 0;
		while (X < SourceSize) & (Y <= SourceSize) do begin
			if (Bit > 15) then begin
				Dest^[Z] := BAND(BSR(Command, 8), $0FF);
				Dest^[Z + 1] := BAND(Command, $0FF);
				Z := Y;
				Bit := 0;
				Y := Y + 2;
			end;
			Size := 1;
			while ((Source^[X] = Source^[X + Size]) & (Size < $0FFF) & (X + Size < SourceSize)) do begin
				Size := Size + 1;
			end;
			if (Size >= 16) then begin
				Dest^[Y] := 0;
				Dest^[Y + 1] := BAND(BSR(Size - 16, 8), $0FF);
				Dest^[Y + 2] := BAND(Size - 16, $0FF);
				Dest^[Y + 3] := Source^[X];
				Y := Y + 4;
				X := X + Size;
				Command := BSL(Command, 1) + 1;
			end else if (GetMatch(X, Size, Pos)) then begin
				Key := BSL(X - Pos, 4) + (Size - 3);
				Dest^[Y] := BAND(BSR(Key, 8), $0FF);
				Dest^[Y + 1] := BAND(Key, $0FF);
				Y := Y + 2;
				X := X + Size;
				Command := BSL(Command, 1) + 1;
			end else begin
				Dest^[Y] := Source^[X];
				Y := Y + 1;
				X := X + 1;
				Command := BSL(Command, 1);
			end;
			Bit := Bit + 1;
		end;
		Command := BSL(Command, 16 - Bit);
		Dest^[Z] := BAND(BSR(Command, 8), $0FF);
		Dest^[Z + 1] := BAND(Command, $0FF);
		if (Y > SourceSize) then begin
			BlockMove(@Source^[0], @Dest^[1], SourceSize);
			Dest^[0] := FLAG_Copied;
			Y := SourceSize + 1;
		end;
		LZRW1KHCompress := Y
	end;

	function LZRW1KHDecompress (Source, Dest: BufferPtr; SourceSize: longint): longint;
		var
			X, Y, Pos: BufferIndex;
			Command, Size, K: longint;
			Bit: BYTE;
	begin
		if (Source^[0] = FLAG_Copied) then begin
			BlockMove(@Source^[1], @Dest^[0], SourceSize - 1);
			Y := SourceSize - 1;
		end else begin
			Y := 0;
			X := 3;
			Command := BSL(Source^[1], 8) + Source^[2];
			Bit := 16;
			while (X < SourceSize) do begin
				if (Bit = 0) then begin
					Command := BSL(Source^[X], 8) + Source^[X + 1];
					Bit := 16;
					X := X + 2;
				end;
				if (BAND(Command, $08000) = 0) then begin
					Dest^[Y] := Source^[X];
					Y := Y + 1;
					X := X + 1;
				end else begin
					Pos := BSL(Source^[X], 4) + BSR(Source^[X + 1], 4);
					if (Pos = 0) then begin
						Size := BSL(Source^[X + 1], 8) + Source^[X + 2] + 15;
						for K := 0 to Size do begin
							Dest^[Y + K] := Source^[X + 3];
						end;
						X := X + 4;
						Y := Y + Size + 1;
					end else begin
						Size := BAND(Source^[X + 1], $0F) + 2;
						for K := 0 to Size do begin
							Dest^[Y + K] := Dest^[Y - Pos + K];
						end;
						X := X + 2;
						Y := Y + Size + 1;
					end;
				end;
				Command := BSL(Command, 1);
				Bit := Bit - 1;
			end
		end;
		LZRW1KHDecompress := Y
	end;

end.